function fout = advect_solution(z_ct,parameters)
%fout = advect_solution(z_ct,parameters)
%advect_solution computes the hyperbolic flux q obtained in the small delta
%limit of the temperate ice equations,
%   q = Pe_phi*u*phi + g*phi^alpga
%where phi satisfies
%   (Pe_phi*u + g*phi^(alpha-1))*dphi/dz = a
%as a function of position z, assuming the problem is of outflow type (so
%that a porosity boundary condition applies at the temperate boundary z_t).
%
%The hyperbolic flux is expressed as a function of a notional
%cold-temperate boundary location z_ct.
%
%advect_solution also computes to the heat equation
% Pe*u*T_z - T_zz = a
% T = T_0 at z = z_c, T = 0 at z - z_ct
%It computes the heat flux q_ct = -T_z at the boundary z = z_ct as a
%function of z_ct, and in addition provides the corresponding constants of
%integration A and B in the temperature solution as defined in the
%documentation to tempsol.m
%
%A true solution of the hyperbolic outflow problem is achieved at the
%location z_ct for which q_ct = q. The output of advect_solution.m is
%intended to allow this location, and corresponding values of A and B, to
%be obtained approximately by graphical means, and for these to be used as
%improved initial guesses through the input field parameters.temp_guess
%
%Input arguments:
%   z_ct:   vector of possible cold-temperate boundary positions at which q
%           and q_ct should be computed. If an empty array is supplied,
%           defaults to 1001 equally spaced points covering the interval
%           [z_t,z_c]
%   parameters: the same parameter structure as used in steady_solver; only
%       the fields parameters.model and parameters.display are used here.
%       If 'display' is set, solutions for q, q_ct and porosity phi at the
%       cold-temperate boundary are plotted in figure 98 as a function of
%       z_ct. The colour scheme used is
%           phi:        green
%           q:          blue
%           q_ct:       r
%
%Output:
%   fout:   solution structure with fields
%       z_ct:   z_ct
%       phi:    porosity at the cold-temperate boundary as a function of
%               z_ct
%       q:      the hyperbolic flux at z_ct
%       q_ct:   the conductive heat flux at z_ct
%       A:      the constant of integration A corresponding to a
%               cold-temperate boundary at location z_ct (see tempsol.m for
%               the definition of A)
%       B:      the  constant of integration B corresponding to a
%               cold-temperate boundary at location z_ct (see tempsol.m for
%               the definition of B)
%
%Christian Schoof, January 2015

if isempty(z_ct)
    z_ct = linspace(parameters.model.z_t,parameters.model.z_c,1001);
end

[fout.z_ct, fout.phi] = ode45(@(z,phi)temperate_advect(z,phi,parameters.model),z_ct,parameters.model.phi_0);
fout.q = enthalpy_flux(fout.z_ct,fout.phi,parameters.model);
[fout.q_ct, fout.A, fout.B]  = heat_flux(fout.z_ct,parameters.model);
if isfield(parameters,'display') && parameters.display
    figure(98), hold on
    plot(fout.z_ct,fout.phi,'g')
    plot(fout.z_ct,fout.q,'b')
    plot(fout.z_ct,fout.q_ct,'r')
end
end


function v_out = temperate_advect(z,phi,params)
%height-dependence equation for the hyperbolic version of temperate ice in steady
%state
Pe_phi = params.Pe_phi;
g = params.g;
alpha = params.alpha;
u_0 = params.u_0;
u_z = params.u_z;
a = params.a;
v_out = a./(Pe_phi*(u_0+u_z*z) + alpha*phi.^(alpha-1)*g);
end

function q = enthalpy_flux(z,phi,params)
Pe_phi = params.Pe_phi;
g = params.g;
alpha = params.alpha;
u_0 = params.u_0;
u_z = params.u_z;

q = Pe_phi*(u_0+u_z*z).*phi + phi.^alpha*g;
end


function [q_ct, A, B] = heat_flux(z_ct_list,params)
%extract solution degrees of freedom
Pe = params.Pe;
u_0 = params.u_0;
u_z = params.u_z;
a = params.a;
z_c = params.z_c;
T_0 = params.T_0;

%extract nodes and weights
[x_int, w_int] = gaussquad(64,1);

q_ct = zeros(size(z_ct_list));
A = zeros(size(z_ct_list));
B = zeros(size(z_ct_list));
for ii = 1:length(z_ct_list)    %lazy version, should be vectorized; exapnd x_int, w_int, z_ct to matrices using repmat
    z_ct = z_ct_list(ii);
    %compute output, go trough different cases
    if u_0 ~= 0
        if u_z ~= 0, error('This code is currently not set up for base velocity and velocity gradient to be simultaneously non-zero'), end
        A(ii) = (T_0*Pe*u_0 - a*(z_c-z_ct))/(exp(Pe*u_0*z_c) -exp(Pe*u_0*z_ct));
        B(ii) = T_0 - A(ii)/(Pe*u_0)*exp(Pe*u_0*z_c) - a*z_c/(Pe*u_0);
        q_ct(ii) = - ( A(ii)*exp(Pe*u_0*z_ct) + a/(Pe*u_0));
    elseif u_z > 0
        Pe = Pe*u_z;
        A(ii) = (T_0 + 2*a/Pe*sum(w_int.*dawson_minus(x_int*sqrt(Pe/2)*z_c)*sqrt(Pe/2)*z_c)...
            - 2*a/Pe*sum(w_int.*dawson_minus(x_int*sqrt(Pe/2)*z_ct)*sqrt(Pe/2)*z_ct))...
            /(sqrt(2/Pe)*exp(Pe*z_c^2/2)*dawson_plus(sqrt(Pe/2)*z_c) - sqrt(2/Pe)*exp(Pe*z_ct^2/2)*dawson_plus(sqrt(Pe/2)*z_ct));
        B(ii) =  T_0 + 2*a/Pe*sum(w_int.*dawson_minus(x_int*sqrt(Pe/2)*z_c)*sqrt(Pe/2)*z_c)...
            - A(ii)*sqrt(2/Pe)*exp(Pe*z_c^2/2)*dawson_plus(sqrt(Pe/2)*z_c); 
        q_ct(ii) = -( A(ii)*exp(Pe*z_ct^2/2) - a*sqrt(2/Pe)*dawson_minus(sqrt(Pe/2)*z_ct) );
    elseif u_z < 0
        Pe = Pe*abs(u_z);
        A(ii) = (T_0 + 2*a/Pe*sum(w_int.*dawson_plus(x_int*sqrt(Pe/2)*z_c)*sqrt(Pe/2)*z_c) ...
            - 2*a/Pe*sum(w_int.*dawson_plus(x_int*sqrt(Pe/2)*z_ct)*sqrt(Pe/2)*z_ct))...
            /(sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z_c) - sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z_ct));
        B(ii) = T_0 + 2*a/Pe*sum(w_int.*dawson_plus(x_int*sqrt(Pe/2)*z_c)*sqrt(Pe/2)*z_c) ...
            - A(ii)*sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z_c);
        q_ct(ii) = -( A(ii)*exp(-Pe*z_ct^2/2) - a*sqrt(2/Pe)*dawson_plus(sqrt(Pe/2)*z_ct) ); 
    end
end
end