function [phi, N, T, q] = asymptotic_base_solution(parameters,z_in)
%v_out = asymptotic_solution(parameters,z)
%computes approximate solutions based on a hyperbolic approximation to
%the compaction problem for small compaction lengths.
%
%The solution for effective pressure and porosity in the temperate region
%is the outer solution satisfying
%dphi/dz = a/(Pe_phi*u + alpha*g*phi^(alpha-1))
%N = (a-Pe_phi*u*dphi/dz)/phi = alpha*g*phi^(alpha-2)/(Pe_phi*u + alpha*g*phi^(alpha-1))
%The present code takes no account of any possible boundary layers (see
%asymptotic_solution_v2.m for a codde that can handle most boundary layers)
%
%The solution in the cold part of the domain solves
%Pe*u*T_z - T_zz = a
%as described in the documentation to tempsol.m
%
%The code permits velocity u to be either of the form u = constant or u =
%u_z*z with u_z constant. In the latter case, the domain must be chosen
%such that u does not change sign in the domain (and in fact, does not
%vanish, though it can be made very small near a boundary).
%
%Boundary conditions are as follows (the same as for the full problem
%solved by steady_solver): At a cold boundary z_c, a surface temperature
%T_0 is specified. If the problem is of "outflow" type, meaning ice velocity
%points from temperate ice into cold ice, a porosity phi_0 must also be
%specified at z_t, otherwise Cauchy conditions on phi are that porosity
%vanishes at the cold-temperate boundary when the problem is of "inflow"
%type, meaning velocity points from cold to temperate.
%
%At the cold-temperate boundary (which is determined as part of the
%solution), we have the boundary condition T = 0, as well as
%T_z = Pe_phi*u*phi + g*phi^alpha
%for outflow problems, or
%T_z = 0
%for inflow problems. In the former condition, phi is computed from the
%outer problem for phi. The location of z_ct is solved for using Newton's
%method, integrating for phi using the ode45 o.d.e. solver.
%ansform.grid (see inputs below)
%
%Input:
%   parameters: parameter structure following fields
%       model: mandatory substructure with fields
%           Pe_phi: porosity Peclet number
%           Pe:     temperature Peclet number
%           g:      gravity (signed, so -1 if z points upwards)
%           delta:  compaction length parameter
%           alpha:  premeability exponent, must be >= 2
%           a:      dissipation rate
%           u_0:    constant contribution to velocity
%           u_z:    strain rate (signed)
%           z_c:    cold boundary 
%           z_t:    temperate boundary
%           N_0:    effective pressure boundary condition at z_t
%           phi_0:  effective pressure boundary condition at z_t, if
%                   applicable
%           NOTE: only one of u_0 and u_z can be non-zero in current code
%       partition:  optional substructure giving partition of domain for
%                   shooting problems over the untransformed independent
%                   variable z, used in the integration of the boundary
%                   layers near the temperate exterior boundary z_t that
%                   appear when either u or the characteristic velocity
%                   Pe_phi*u + alpha*g*phi^(alpha-1) nearly vanish near the
%                   boundary. Substructure has a single partition, 'field',
%                   which gives the partition of a domain (0,1) that is
%                   then stretched from z_t to z_farfield
%       integration: optional substructure containing fields
%           nodes:  integration nodes for the interval [0,1]
%           weights:corresponding weights for integration over the same
%                   interval
%           defaults to 64-point Gaussian integration; used for solution of
%           temperature problem in terms of Dawson's functions when u_z is
%           non-zero. NOTE: nodes and weights must be column vectors
%       srchparams: optional substructure containing some or all of the
%           fields specfieid under 'sruchparams' in Newton_single.m
%       display:    boolean, if set to 'true', displays solution on figure
%                   99. Colour scheme as for steady_solver /
%                   grid_projection: 
%                       T:      magenta
%                       phi:    green
%                       N:      blue
%                       q:      red
%                   but displayed as dashed lines
%   z:  vector of grid locations at which solution is to be evaluated
%Output:
%   N:  effective pressure at locations z, NaN in cold region or where the
%       inflow boundary solution was not solved for successfully (can
%       happen epending on how close points in z are and the integration
%       interval specified by parameters.transform.grid aobve)
%   phi:porosity at locations z, 0 in cold region
%   T:  temperature at locations z, 0 in temperate region
%   q:  total enthalpy flux, Pe_phi*u + g*phi^alpha from outer solution in
%       temperate region, Pe*u*T - T_z in cold region
%
%Dependencies: make sure Newton_single, shoot, tempsol, gaussquad,
%dawson_minus, dawson_plus and faddeeva are in current path
%
%Christian Schoof, January 2015

%% set transform flag if not set
if ~isfield(parameters,'transform') || ~isfield(parameters.transform,'transform')
    parameters.transform.transform = true;
end

%% first, compute temperature parameters and free boundary location
if ~isfield(parameters,'temp_guess')
    if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
        %make initial guess based on a small T_0*(Pe*u)^2/a approximation for the
        %obstacle problem version of the temperature problem
        v_0 = zeros(3,1);
        u_c = (parameters.model.u_0+parameters.model.u_z*parameters.model.z_c);
        v_0(1) = parameters.model.z_c + sign(parameters.model.z_t-parameters.model.z_c)*(-2*parameters.model.T_0/parameters.model.a)^(1/2);
        v_0(2) = -parameters.model.a*exp(-parameters.model.Pe*u_c*v_0(1))/(parameters.model.Pe*u_c);
        v_0(3) = -parameters.model.a/(parameters.model.Pe*u_c)*(v_0(1) - 1/(parameters.model.Pe*u_c) );
    else
        %Stefan condition version
        v_0 = zeros(3,1);
        u_c = (parameters.model.u_0+parameters.model.u_z*parameters.model.z_c);
        q_c =  parameters.model.Pe_phi*parameters.model.u_0*parameters.model.phi_0 + parameters.model.g*parameters.model.phi_0^parameters.model.alpha...
            + (parameters.model.a+parameters.model.Pe_phi*parameters.model.u_z*parameters.model.phi_0)*(parameters.model.z_c-parameters.model.z_t);
        v_0(1) = parameters.model.z_c + parameters.model.T_0/(q_c*(1+parameters.model.a*parameters.model.T_0/q_c));
        v_0(2) = q_c*exp(-parameters.model.Pe*u_c*v_0(1));
        v_0(3) =  -parameters.model.a/(parameters.model.Pe*u_c)*(v_0(1) + q_c);
    end
else
    v_0 = parameters.temp_guess;
end
%solve for actual location of cold-temperate boundary in small delta limit
%(exact for inflow problems)
if isfield(parameters,'srchparams')
    v_temp = Newton_single(@steady_advect,v_0,parameters,parameters.srchparams);
else
    v_temp = Newton_single(@steady_advect,v_0,parameters);
end
z_ct = v_temp(1);   %this is the cold-temperate boundary location
parameters.model.z_ct = z_ct;
z_t = parameters.model.z_t;

%% Re-order grid
z = reshape(z_in,1,length(z_in));

%% integrate hyperbolic model for phi from appropriate boundary, taking
%% account of grid ordering
direction = sign(parameters.model.z_t - parameters.model.z_c);
func = @(z,v)temperate_advect(z,v,parameters.model);
if direction == sign(parameters.model.u_0) || direction == sign(parameters.model.u_z)   %inflow case first
    if direction == sign(z(end)-z(1))
        z_span = [z_ct z(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0)];
        if z_span(2) == z_span(1)
            z_span = z_span(2:end);
            [z_temp phi_temp] = ode45(func,z_span,0);
        else
            [z_temp phi_temp] = ode45(func,z_span,0);
            z_temp = z_temp(2:end); phi_temp = phi_temp(2:end);
        end
    else
        z_span = [z(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0) z_ct];
        if z_span(end) == z_span(end-1)
            z_span = z_span(end-1:-1:1);
            [z_temp phi_temp] = ode45(func,z_span,0);
            z_temp = z_temp(end:-1:1); phi_temp = phi_temp(end:-1:1);
        else
            z_span = z_span(end:-1:1);
            [z_temp phi_temp] = ode45(func,z_span,0);
            z_temp = z_temp(end:-1:2); phi_temp = phi_temp(end:-1:2);
        end
    end
else
    if direction == sign(z(end)-z(1))
        z_span = [z(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0) parameters.model.z_t];
        if z_span(end) == z_span(end-1)
            z_span = z_span(end-1:-1:1);
            [z_temp phi_temp] = ode45(func,z_span,parameters.model.phi_0);
            z_temp = z_temp(end:-1:1); phi_temp = phi_temp(end:-1:1);
        else
            z_span = z_span(end:-1:1);
            [z_temp phi_temp] = ode45(func,z_span,parameters.model.phi_0);
            z_temp = z_temp(end:-1:2); phi_temp = phi_temp(end:-1:2);
        end
    else
        z_span = [parameters.model.z_t z(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0)];
        if z_span(2) == z_span(1)
            z_span = z_span(2:end);
            [z_temp phi_temp] = ode45(func,z_span,parameters.model.phi_0);
        else
            [z_temp phi_temp] = ode45(func,z_span,parameters.model.phi_0);
            z_temp = z_temp(2:end); phi_temp = phi_temp(2:end);
        end
    end
end
phi_temp = reshape(phi_temp,1,length(phi_temp)); %make phi_grid a row vector;

%% post-process: compute locations, effective pressures and net flux
z_temp = reshape(z_temp,1,length(z_temp));
N_temp = (parameters.model.a - parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_temp).*temperate_advect(z_temp,phi_temp,parameters.model))./phi_temp;
q_temp = parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_temp).*phi_temp + parameters.model.g*phi_temp.^parameters.model.alpha;

%% produce output

%temperature
[T, q] = temp_profile(z,v_temp,parameters);

N = zeros(size(z));
phi = zeros(size(z));
N((direction*(z-z_ct) >= 0) & direction*(z_t-z) >= 0) = N_temp;
N((direction*(z-z_ct) < 0) | direction*(z_t-z) < 0) = NaN;
phi(direction*(z-z_ct) >= 0 & direction*(z_t-z) >= 0) = phi_temp;
phi( direction*(z_t-z) < 0) = NaN;
q(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0) = q_temp;
q(direction*(z_t-z) < 0) = NaN;
T(direction*(z_t-z) < 0) = NaN;

%display, if required
if isfield(parameters,'display')
    figure(99), hold on
    plot(z,T,'m--')
    plot(z,phi,'g--')
    plot(z,N,'b--')
    plot(z,q,'r--')
end
end

function [T, q] = temp_profile(z,v_0,parameters)

Pe = parameters.model.Pe;
a = parameters.model.a;
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
z_c = parameters.model.z_c;
z_ct = v_0(1);
A = v_0(2);
B = v_0(3);
if u_0 ~= 0
    T = A/(Pe*u_0)*exp(Pe*u_0*z) + a*z/(Pe*u_0) + B;
    q = Pe*u_0*T - ( A*exp(Pe*u_0*z) + a/(Pe*u_0));
else
    if isfield(parameters,'integration')
        x_int = parameters.integration.nodes;
        w_int = parameters.integration.weights;
    else
       [x_int,w_int] =  gaussquad(64,1);
    end
    z_aux = reshape(z,1,length(z));
    z_aux = repmat(z_aux,length(x_int),1);
    x_aux = repmat(x_int,1,length(z));
    w_aux = repmat(w_int,1,length(z));
    if u_z > 0
        Pe = Pe*u_z;
        T = -2*a/Pe*sum(w_aux.*dawson_minus(x_aux*sqrt(Pe/2).*z_aux)*sqrt(Pe/2).*z_aux,1)...
            + A*sqrt(2/Pe)*exp(Pe*z.^2/2).*dawson_plus(sqrt(Pe/2)*z) + B;
        q = Pe*u_z*z.*T -( A*exp(Pe*z.^2/2) - a*sqrt(2/Pe)*dawson_minus(sqrt(Pe/2)*z) );
    else       
        Pe = Pe*abs(u_z);
        T = -2*a/Pe*sum(w_aux.*dawson_plus(x_aux*sqrt(Pe/2).*z_aux)*sqrt(Pe/2).*z_aux,1)...
           + A*sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z)+ B;
        q = Pe*sign(u_z)*z.*T -( A*exp(-Pe*z.^2/2) - a*sqrt(2/Pe).*dawson_plus(sqrt(Pe/2)*z) );
    end
    T = reshape(T,size(z));
    q = reshape(q,size(q));
end
T(z<min(z_c,z_ct)|z>max(z_c,z_ct)) = 0;
q(z<min(z_c,z_ct)|z>max(z_c,z_ct)) = 0;

end

function [v_out Dv_out] = steady_advect(v_in,parameters)
if nargout == 1
    ftemp1 = tempsol(v_in,parameters);
    v_out = ftemp1;
else
    [ftemp1 Dftemp1] = tempsol(v_in,parameters);
    v_out = ftemp1;
    Dv_out = Dftemp1;
end
if (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
    phi_0 = parameters.model.phi_0;
    z_ct = v_in(1);
    z_span = [parameters.model.z_t z_ct];
    func = @(z,v)temperate_advect(z,v,parameters.model);
    options = odeset('NonNegative',1);
    [zsol phisol] = ode45(func,z_span,phi_0,options);
    phi_ct = phisol(end);
    ftemp2 = parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct)*phi_ct + parameters.model.g*phi_ct^parameters.model.alpha;   %enthalpy flux in positive z-direction
    v_out(3) = v_out(3) - ftemp2;
    if nargout > 1
        Dphi_ct = temperate_advect(z_ct,phi_ct,parameters.model);
        Dftemp2 = parameters.model.Pe_phi*parameters.model.u_z*phi_ct + ...
            (parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct) + parameters.model.g*parameters.model.alpha*phi_ct^(parameters.model.alpha-1))*Dphi_ct;
        Dv_out(3,1) = Dv_out(3,1) - Dftemp2;
    end
end
end

function v_out = temperate_advect(z,phi,params)
%height-dependence equation for the hyperbolic version of temperate ice in steady
%state
Pe_phi = params.Pe_phi;
g = params.g;
alpha = params.alpha;
u_0 = params.u_0;
u_z = params.u_z;
a = params.a;
v_out = a./(Pe_phi*(u_0+u_z*z) + alpha*phi.^(alpha-1)*g);
end
