function [phi, N, T, q] = asymptotic_solution_v2(parameters,z_in)
%v_out = asymptotic_solution(parameters,z)
%computes approximate solutions based on a hyperbolic approximation to
%the compaction problem for small compaction lengths.
%
%The solution for effective pressure and porosity in the temperate region
%is a composite of the outer solution satisfying
%dphi/dz = a/(Pe_phi*u + alpha*g*phi^(alpha-1))
%N = (a-Pe_phi*u*dphi/dz)/phi = alpha*g*phi^(alpha-2)/(Pe_phi*u + alpha*g*phi^(alpha-1))
%and appropriate boundary layers. 
%
%The solution in the cold part of the domain solves
%Pe*u*T_z - T_zz = a
%as described in the documentation to tempsol.m
%
%The code permits velocity u to be either of the form u = constant or u =
%u_z*z with u_z constant. In the latter case, the domain must be chosen
%such that u does not change sign in the domain (and in fact, does not
%vanish, though it can be made very small near a boundary).
%
%Boundary conditions are as follows (the same as for the full problem
%solved by steady_solver): At a cold boundary z_c, a surface temperature
%T_0 is specified. At a temperate boundary z_t, an effective pressure N_0
%is specified. If the problem is of "outflow" type, meaning ice velocity
%points from temperate ice into cold ice, a porosity phi_0 must also be
%specified at z_t, otherwise Cauchy conditions on phi are that porosity
%vanishes at the cold-temperate boundary when the problem is of "inflow"
%type, meaning velocity points from cold to temperate.
%
%At the cold-temperate boundary (which is determined as part of the
%solution), we have the boundary condition T = 0, as well as
%T_z = Pe_phi*u*phi + g*phi^alpha
%for outflow problems, or
%T_z = 0
%for inflow problems. In the former condition, phi is computed from the
%outer problem for phi. The location of z_ct is solved for using Newton's
%method, integrating for phi using the ode45 o.d.e. solver.
%
%The boundary layers computed for phi and N area s follows:
%
%At boundaries satisfying Dirichlet conditions on effective pressure N,
%there is a boundary layer solution satisfying (in the original variables)
%N = N_D + (N_0-N_D)*exp(-sqrt((Pe_phi*u_D+alpha*g*phi_D^(alpha-1))(delta*Pe_phi*u_D*phi_D^(alpha-1)))*|z-z_D|);
%where z_D is the location of the boundary, and N_D, u_D and phi_D are
%given by the outer solution, evaluated at the boundary, and N_0 is the
%prescribed boundary value for N.
%
%This boundary layer is appropriate except in two cases: first, if the
%velocity u_D near a temperate outflow boundary is small, of O(delta), but
%increases linearly with z (so u_z is non-zero) then phi and N in the
%boundary layer satisfy
%dphi/dz = (a - phi*N)/(Pe_phi*u)
%dN/dz = (q_D/phi^alpha -g)/delta
%where q_D is the flux in the outer problem at the boundary, with matching
%condition phi -> phi_D in the far field, where phi_D is the near-boundary
%porosity in the outer problem, and boundary condition N = N_D at the
%boundary. The code solves the problem over the domain (z_t,z_ct) using a
%multiple shooting method
%
%Second, if the characteristic velocity Pe_phi*u + alpha*g*phi^(alpha-1)
%vanishes within a distance of O(delta^(2/5)) of an outflow boundary z_t
%(whether the characteristic velocity vanishes within the domain or not),
%yet another boundary model applies. We define phi_f as the porosity at
%which the characteristic velocity vanishes and z_f as the location at
%which this happens (which is where the hyperbolic solution breaks down),
%so
%   Pe_phi*u(z_f) + alpha*g*phi_f^(alpha-1) = 0
%with phi_f = phi(z_f), computed by integrating from the cold-temperate
%inflow boundary at z=z_f, and continuing the definiton of a and u smoothly
%outside the domain if necessary. The, in the boundary layer we have
%   phi = phi_f + phi_correct
%with
%   1/2*g*alpha*(alpha-1)*phi_f^(alpha-2)*phi_correct^2 +
%   delta*phi_f^alpha*dN/dz = a*(z-z_f)
%   Pe_phi*u_f*dphi/dz + phi_f*N = 0
%with matching condition
%   N = -Pe_phi*u_f*a/(g*alpha*(alpha-1)*phi_f^(alpha-1)*phi_correct);
%in the far field, and N = N_0 (strictly, N = 0 to leading order in delta)
%at the boundary. As above, this is solved over the finite domain
%(z_ct,z_t) using a multiple shooting method
%
%At the cold-temperate boundary, there may be additional boundary layers.
%
%In an outflow problem, where velocity points into the cold region,
%the boundary layer problem is (in the original variables)
%dphi/dz = -phi*N/(Pe_phi*u_ct);
%dN/dz = ( q_ct/phi^alpha - Pe_phi*u_ct/phi^(alpha-1) - g ) / delta;
%with (phi,N) -> (phi_ct,0) as |z-z_ct| -> infinity, and
% phi^alpha*(g+delta*dN/dz) = 0
%at z = z_ct. Here z_ct is the location of the cold-temperate boundary, and
%u_ct, q_ct and phi_ct are given by the outer solution evaluated at the
%cold-temperate boundary, with q_ct = Pe_phi*u_ct*phi_ct + phi_ct^alpha*g.
%Note that this boundary layer (and therefore the asymptotic solution) only
%exist if the fixed point (phi_ct,0) is a saddle point, which requires the
%sign of Pe_phi*u_ct + alpha*g*phi_ct^(alpha-1) to be the same as the sign
%of Pe_phi*u_ct; basically, this amounts to saying that the total
%hyperbolic flux |Pe_phi*u + g*phi^alpha| must increase with porosity phi.
%This boundary layer problem is solved using the ode45 o.d.e. solver with
%an event function.
%
%For inflow problems, where velociy points into the temperate region, we
%require a boundary layer at the cold-temperate boundary if the parameter
%alpha lies in the range 2 < alpha < 3, in which case N near the boundary
%satisfies
%delta*d/dz((a/(Pe_phi*u_ct)*(z-z_ct))^alpha*dN/dz) -
%a/(Pe_phi*u_ct)*(z-z_ct)*N =
%-alpha*g*(a/(Pe_phi*|u_ct|))^alpha*|z-z_ct|^(alpha-2)*(z-z_ct)
%subject to N ~
%alpha*g*(a/(Pe_phi*|u_ct|))^(alpha-1)*|z-z_ct|^(alpha-3)*(z-z_ct)
%and N remaining finite as z -> z_ct
%
%Computationally, the latter is solved by transforming to
%   zeta:       d|z-z_ct|/dzeta = phi
%   phi         = a/(Pe_phi*|u_ct|)*|z-z_ct|
%   Psi         = phi^(alpha-2)*N
%   chi_tilde    = 1/a*(delta*dPsi/dzeta + delta*a(2-alpha)*Psi/(Pe_phi*|u_ct|) + g_n*phi^(alpha-1) 
%                = 1/(a*phi)*phi^alpha*(g_n + delta*dN/dz)
%g_n being the normal component of g, g_n = g*sign(z_t-z-ct).
%The transformation turns the singular boundary value problem into the
%problem of solving for an orbit into a fixed point (phi,Psi,chi_tilde) =
%(0,0,0) (with a stable manifold and a stable centre manifold); the orbit
%must satisfy chi_tilde ~ g*phi^(alpha-1), Psi ~
%a/(Pe_phi*u_ct)*g*phi^(2*alpha-4) as zeta -> infinity. This boundary layer
%is solved for using a multiple shooting method on a finite interval with a
%grid defined by parameters.transform.grid (see inputs below)
%
%Input:
%   parameters: parameter structure following fields
%       model: mandatory substructure with fields
%           Pe_phi: porosity Peclet number
%           Pe:     temperature Peclet number
%           g:      gravity (signed, so -1 if z points upwards)
%           delta:  compaction length parameter
%           alpha:  premeability exponent, must be >= 2
%           a:      dissipation rate
%           u_0:    constant contribution to velocity
%           u_z:    strain rate (signed)
%           z_c:    cold boundary 
%           z_t:    temperate boundary
%           N_0:    effective pressure boundary condition at z_t
%           phi_0:  effective pressure boundary condition at z_t, if
%                   applicable
%           NOTE: only one of u_0 and u_z can be non-zero in current code
%           phi_ftm_farfield: optional parameter setting the value of phi
%                   at "infinity" in transformed inflow boundary layer
%                   problem
%           zero_velocity_tolerance: The boundary layer structure at a
%                   temperate exterior boundary with prescribed effective
%                   pressure changes if the velocity near that boundary is
%                   close to zero (assuming flow u towards the boundary) as
%                   described above
%                   zero_velocity_tolerance sets the tolerance for how
%                   close to zero the velocity at the boundary needs to be
%                   in order to switch to this alternative boundary layer
%                   formulation: if the absolute value of velocity at the
%                   boundary is less than delta times
%                   zero_velocity_tolerance, then the alternative boundary
%                   layer formulation is used. The standard boundary layer
%                   model is used if zero_velocity_tolerance is not
%                   supplied as a field.
%           zero_characteristic_tolerance: The boundary layer structure at a
%                   temperate exterior boundary with prescribed effective
%                   pressure changes if the characteristic velocity near
%                   that boundary is close to zero (assuming flow towards
%                   the boundary), as described above..
%                   zero_characteristic_tolerance sets the tolerance for how
%                   close to zero the velocity at the boundary needs to be
%                   in order to switch to this alternative boundary layer
%                   formulation: if the absolute value of velocity at the
%                   boundary is less than delta^(1/5) times
%                   zero_characteristic_tolerance, then the alternative boundary
%                   layer formulation is used. The standard boundary layer
%                   model is used if zero_velocity_tolerance is not
%                   supplied as a field.
%           z_farfield: optional location at which far field boundary
%                   conditions are applied for the temperate boundary
%                   layers for near-vanishing velocity or characteristic
%                   velocity. z_farfield is similar to phi_tfm_farfield in
%                   that these far field boundary conditions need to be
%                   applied at a finite but large distance. Integration of
%                   the boundary layer is from z_farfield to z_t. If
%                   z_farfield lies on the wrong side of z_t (i.e.
%                   sgn(z_t-z_farfield) ~= sgn(z_t-z_ct)), then the code
%                   redefines z_farfield as
%                   z_farfield = z_t - (z_farfield-z_t) = 2*z_t -
%                   z_farfield
%                   z_farfield defaults to z_ct
%       partition:  optional substructure giving partition of domain for
%                   shooting problems over the untransformed independent
%                   variable z, used in the integration of the boundary
%                   layers near the temperate exterior boundary z_t that
%                   appear when either u or the characteristic velocity
%                   Pe_phi*u + alpha*g*phi^(alpha-1) nearly vanish near the
%                   boundary. Substructure has a single partition, 'field',
%                   which gives the partition of a domain (0,1) that is
%                   then stretched from z_t to z_farfield
%       transform:  optional substructure containing fields
%           transform: boolean; here determines whether inflow boundary
%                   layer is solved for when 2 < alpha < 3; defaults to
%                   true
%           grid:   parition of zeta-interval over which transformed inflow
%                   boundary layer problem is solved, defaults to
%                   0:-.5:-100
%       integration: optional substructure containing fields
%           nodes:  integration nodes for the interval [0,1]
%           weights:corresponding weights for integration over the same
%                   interval
%           defaults to 64-point Gaussian integration; used for solution of
%           temperature problem in terms of Dawson's functions when u_z is
%           non-zero. NOTE: nodes and weights must be column vectors
%       srchparams: optional substructure containing some or all of the
%           fields specfieid under 'sruchparams' in Newton_single.m
%       display:    boolean, if set to 'true', displays solution on figure
%                   99. Colour scheme as for steady_solver /
%                   grid_projection: 
%                       T:      magenta
%                       phi:    green
%                       N:      blue
%                       q:      red
%                   but displayed as dashed lines
%   z_in:  vector of grid locations at which solution is to be evaluated
%Output:
%   N:  effective pressure at locations z, NaN in cold region or where the
%       inflow boundary solution was not solved for successfully (can
%       happen epending on how close points in z are and the integration
%       interval specified by parameters.transform.grid aobve)
%   phi:porosity at locations z, 0 in cold region
%   T:  temperature at locations z, 0 in temperate region
%   q:  total enthalpy flux, Pe_phi*u + g*phi^alpha from outer solution in
%       temperate region, Pe*u*T - T_z in cold region
%
%Dependencies: make sure Newton_single, shoot, tempsol, gaussquad,
%dawson_minus, dawson_plus and faddeeva are in current path
%
%Christian Schoof, January 2015

%% set transform flag if not set
if ~isfield(parameters,'transform') || ~isfield(parameters.transform,'transform')
    parameters.transform.transform = true;
end

%% first, compute temperature parameters and free boundary location
if ~isfield(parameters,'temp_guess')
    if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
        %make initial guess based on a small T_0*(Pe*u)^2/a approximation for the
        %obstacle problem version of the temperature problem
        v_0 = zeros(3,1);
        u_c = (parameters.model.u_0+parameters.model.u_z*parameters.model.z_c);
        v_0(1) = parameters.model.z_c + sign(parameters.model.z_t-parameters.model.z_c)*(-2*parameters.model.T_0/parameters.model.a)^(1/2);
        v_0(2) = -parameters.model.a*exp(-parameters.model.Pe*u_c*v_0(1))/(parameters.model.Pe*u_c);
        v_0(3) = -parameters.model.a/(parameters.model.Pe*u_c)*(v_0(1) - 1/(parameters.model.Pe*u_c) );
    else
        %Stefan condition version
        v_0 = zeros(3,1);
        u_c = (parameters.model.u_0+parameters.model.u_z*parameters.model.z_c);
        q_c =  parameters.model.Pe_phi*parameters.model.u_0*parameters.model.phi_0 + parameters.model.g*parameters.model.phi_0^parameters.model.alpha...
            + (parameters.model.a+parameters.model.Pe_phi*parameters.model.u_z*parameters.model.phi_0)*(parameters.model.z_c-parameters.model.z_t);
        v_0(1) = parameters.model.z_c + parameters.model.T_0/(q_c*(1+parameters.model.a*parameters.model.T_0/q_c));
        v_0(2) = q_c*exp(-parameters.model.Pe*u_c*v_0(1));
        v_0(3) =  -parameters.model.a/(parameters.model.Pe*u_c)*(v_0(1) + q_c);
    end
else
    v_0 = parameters.temp_guess;
end
%solve for actual location of cold-temperate boundary in small delta limit
%(exact for inflow problems)
if isfield(parameters,'srchparams')
    v_temp = Newton_single(@steady_advect,v_0,parameters,parameters.srchparams);
else
    v_temp = Newton_single(@steady_advect,v_0,parameters);
end
z_ct = v_temp(1);   %this is the cold-temperate boundary location
parameters.model.z_ct = z_ct;
z_t = parameters.model.z_t;

%% Re-order grid
z = reshape(z_in,1,length(z_in));

%% integrate hyperbolic model for phi from appropriate boundary, taking
%% account of grid ordering
direction = sign(parameters.model.z_t - parameters.model.z_c);
func = @(z,v)temperate_advect(z,v,parameters.model);
if direction == sign(parameters.model.u_0) || direction == sign(parameters.model.u_z)   %inflow case first
    if direction == sign(z(end)-z(1))
        z_span = [z_ct z(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0)];
        if z_span(2) == z_span(1)
            z_span = z_span(2:end);
            [z_temp phi_temp] = ode45(func,z_span,0);
        else
            [z_temp phi_temp] = ode45(func,z_span,0);
            z_temp = z_temp(2:end); phi_temp = phi_temp(2:end);
        end
    else
        z_span = [z(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0) z_ct];
        if z_span(end) == z_span(end-1)
            z_span = z_span(end-1:-1:1);
            [z_temp phi_temp] = ode45(func,z_span,0);
            z_temp = z_temp(end:-1:1); phi_temp = phi_temp(end:-1:1);
        else
            z_span = z_span(end:-1:1);
            [z_temp phi_temp] = ode45(func,z_span,0);
            z_temp = z_temp(end:-1:2); phi_temp = phi_temp(end:-1:2);
        end
    end
else
    if direction == sign(z(end)-z(1))
        z_span = [z(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0) parameters.model.z_t];
        if z_span(end) == z_span(end-1)
            z_span = z_span(end-1:-1:1);
            [z_temp phi_temp] = ode45(func,z_span,parameters.model.phi_0);
            z_temp = z_temp(end:-1:1); phi_temp = phi_temp(end:-1:1);
        else
            z_span = z_span(end:-1:1);
            [z_temp phi_temp] = ode45(func,z_span,parameters.model.phi_0);
            z_temp = z_temp(end:-1:2); phi_temp = phi_temp(end:-1:2);
        end
    else
        z_span = [parameters.model.z_t z(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0)];
        if z_span(2) == z_span(1)
            z_span = z_span(2:end);
            [z_temp phi_temp] = ode45(func,z_span,parameters.model.phi_0);
        else
            [z_temp phi_temp] = ode45(func,z_span,parameters.model.phi_0);
            z_temp = z_temp(2:end); phi_temp = phi_temp(2:end);
        end
    end
end
phi_temp = reshape(phi_temp,1,length(phi_temp)); %make phi_grid a row vector;

%% post-process: compute locations, effective pressures and net flux
z_temp = reshape(z_temp,1,length(z_temp));
N_temp = (parameters.model.a - parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_temp).*temperate_advect(z_temp,phi_temp,parameters.model))./phi_temp;
q_temp = parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_temp).*phi_temp + parameters.model.g*phi_temp.^parameters.model.alpha;

%% Add boundary layers

%identify critical case
char_vel = parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_temp) + parameters.model.alpha*parameters.model.g*phi_temp.^(parameters.model.alpha-1);
if (direction ~= sign(parameters.model.u_0) && direction ~= sign(parameters.model.u_z)) || ~isfield(parameters.model,'zero_characteristic_tolerance') || (isreal(phi_temp) && min(abs(char_vel)) > parameters.model.zero_characteristic_tolerance*parameters.model.delta^(1/5))
    %not near critical point, use standard procedure
    
    %effective pressure boundary condition boundary layer
    [phi_correct, N_correct] = pressure_correct(z_temp,phi_temp,N_temp,parameters);
    N_temp = N_temp + N_correct;
    phi_temp = phi_temp + phi_correct;

    if (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
        %correct for zero Darcy flux condition at outflow boundary
        [phi_correct, N_correct] = flux_correct(z_temp,phi_temp,q_temp,parameters);
        N_temp = N_temp+N_correct;
        phi_temp = phi_temp + phi_correct;
    end
else
    %recompute outer solution 'the other way around' in order to identify critical point location, then add new effective pressure boundary condition boundary near critical point 
    func = @(phi,v)temperate_advect_inverted(phi,v,parameters.model);
    options = odeset('Events',@(phi,v)outer_critical(phi,v,parameters.model),'RelTol',1e-12,'AbsTol',1e-12);
    u_max = parameters.model.u_0 + parameters.model.u_z*max(parameters.model.z_t,parameters.model.z_c);
    phi_max = 1.5*(abs(parameters.model.Pe*u_max/(-parameters.model.alpha*parameters.model.g)))^(1/(parameters.model.alpha-1));
    %first, determine critical point location
    [phi_temp2 z_temp2,phi_f,z_f,i_f] = ode45(func,[0 phi_max],z_ct,options);
    parameters.model.z_f = z_f;
    parameters.model.phi_f = phi_f;
    q_f = parameters.model.Pe_phi*(parameters.model.u_0+parameters.model.u_z*z_f)*phi_f + parameters.model.g*phi_f^parameters.model.alpha;
    %next integrate up to critical point, determining phi at predefined
    %locations for z
    options = odeset('Events',@(phi,v)temperate_advect_inverted_projection(phi,v,z_temp),'RelTol',1e-12,'AbsTol',1e-12);
    [phi_temp2,z_temp2,phi_p,z_p,i_p] = ode45(func,[0 phi_f],z_ct,options);
    phi_p = phi_p.'; z_p = z_p.'; i_p = i_p.';
    phi_temp2 = zeros(size(phi_temp));
    phi_temp2(i_p)=phi_p;
    phi_1 = -(2*parameters.model.a*(z_p-z_f)/(parameters.model.alpha*(parameters.model.alpha-1)*parameters.model.g*phi_f^(parameters.model.alpha-2))).^(1/2);
    phi_temp2(i_p) = phi_temp2(i_p)-phi_f-phi_1;
    N_temp2 = zeros(size(phi_temp2));
    N_temp2(i_p) = parameters.model.alpha*phi_p.^(parameters.model.alpha-2)*parameters.model.g*parameters.model.a./(parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_p) + parameters.model.alpha*phi_p.^(parameters.model.alpha-1)*parameters.model.g);
    N_temp2(i_p) = N_temp2(i_p) - parameters.model.a./((parameters.model.alpha-1)*phi_1) ...
        - 2*parameters.model.a*(parameters.model.alpha-2)/(3*(parameters.model.alpha-1)*phi_f);%+ parameters.model.a/(parameters.model.alpha-1)*(2*parameters.model.a*(z_p-z_f)/(parameters.model.alpha*(parameters.model.alpha-1)*parameters.model.g*phi_f^(parameters.model.alpha-2))).^(-1/2);
    q_temp2 = zeros(size(phi_temp2));
    q_temp2(i_p) = parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_p).*phi_p + parameters.model.g*phi_p.^parameters.model.alpha;
    q_temp2(i_p) = q_temp2(i_p)-q_f-parameters.model.a*(z_p-z_f);
    %now add effective pressure boundary layer
    [phi_correct N_correct] = pressure_correct_critical(z_temp,parameters);
    phi_temp = phi_temp2 + phi_f + phi_correct;
    N_temp = N_correct + N_temp2;
    q_temp = q_temp2 + q_f + parameters.model.a*(z_temp-z_f);
end

% Inflow boundary layer, if necessary
if parameters.model.alpha < 3 && parameters.model.alpha > 2 && (direction == sign(parameters.model.u_0) || direction == sign(parameters.model.u_z)) && parameters.transform.transform
    N_correct = transform_correct(z_temp,parameters);
    N_temp = N_temp +N_correct;
end

%% produce output

%temperature
[T, q] = temp_profile(z,v_temp,parameters);

N = zeros(size(z));
phi = zeros(size(z));
N((direction*(z-z_ct) >= 0) & direction*(z_t-z) >= 0) = N_temp;
N((direction*(z-z_ct) < 0) | direction*(z_t-z) < 0) = NaN;
phi(direction*(z-z_ct) >= 0 & direction*(z_t-z) >= 0) = phi_temp;
phi( direction*(z_t-z) < 0) = NaN;
q(direction*(z-z_ct) > 0 & direction*(z_t-z) >= 0) = q_temp;
q(direction*(z_t-z) < 0) = NaN;
T(direction*(z_t-z) < 0) = NaN;

%display, if required
if isfield(parameters,'display')
    figure(99), hold on
    plot(z,T,'m--')
    plot(z,phi,'g--')
    plot(z,N,'b--')
    plot(z,q,'r--')
end
end

function [T, q] = temp_profile(z,v_0,parameters)

Pe = parameters.model.Pe;
a = parameters.model.a;
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
z_c = parameters.model.z_c;
z_ct = v_0(1);
A = v_0(2);
B = v_0(3);
if u_0 ~= 0
    T = A/(Pe*u_0)*exp(Pe*u_0*z) + a*z/(Pe*u_0) + B;
    q = Pe*u_0*T - ( A*exp(Pe*u_0*z) + a/(Pe*u_0));
else
    if isfield(parameters,'integration')
        x_int = parameters.integration.nodes;
        w_int = parameters.integration.weights;
    else
       [x_int,w_int] =  gaussquad(64,1);
    end
    z_aux = reshape(z,1,length(z));
    z_aux = repmat(z_aux,length(x_int),1);
    x_aux = repmat(x_int,1,length(z));
    w_aux = repmat(w_int,1,length(z));
    if u_z > 0
        Pe = Pe*u_z;
        T = -2*a/Pe*sum(w_aux.*dawson_minus(x_aux*sqrt(Pe/2).*z_aux)*sqrt(Pe/2).*z_aux,1)...
            + A*sqrt(2/Pe)*exp(Pe*z.^2/2).*dawson_plus(sqrt(Pe/2)*z) + B;
        q = Pe*u_z*z.*T -( A*exp(Pe*z.^2/2) - a*sqrt(2/Pe)*dawson_minus(sqrt(Pe/2)*z) );
    else       
        Pe = Pe*abs(u_z);
        T = -2*a/Pe*sum(w_aux.*dawson_plus(x_aux*sqrt(Pe/2).*z_aux)*sqrt(Pe/2).*z_aux,1)...
           + A*sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z)+ B;
        q = Pe*sign(u_z)*z.*T -( A*exp(-Pe*z.^2/2) - a*sqrt(2/Pe).*dawson_plus(sqrt(Pe/2)*z) );
    end
    T = reshape(T,size(z));
    q = reshape(q,size(q));
end
T(z<min(z_c,z_ct)|z>max(z_c,z_ct)) = 0;
q(z<min(z_c,z_ct)|z>max(z_c,z_ct)) = 0;

end

function [v_out Dv_out] = steady_advect(v_in,parameters)
if nargout == 1
    ftemp1 = tempsol(v_in,parameters);
    v_out = ftemp1;
else
    [ftemp1 Dftemp1] = tempsol(v_in,parameters);
    v_out = ftemp1;
    Dv_out = Dftemp1;
end
if (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
    phi_0 = parameters.model.phi_0;
    z_ct = v_in(1);
    z_span = [parameters.model.z_t z_ct];
    func = @(z,v)temperate_advect(z,v,parameters.model);
    options = odeset('NonNegative',1);
    [zsol phisol] = ode45(func,z_span,phi_0,options);
    phi_ct = phisol(end);
    ftemp2 = parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct)*phi_ct + parameters.model.g*phi_ct^parameters.model.alpha;   %enthalpy flux in positive z-direction
    v_out(3) = v_out(3) - ftemp2;
    if nargout > 1
        Dphi_ct = temperate_advect(z_ct,phi_ct,parameters.model);
        Dftemp2 = parameters.model.Pe_phi*parameters.model.u_z*phi_ct + ...
            (parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct) + parameters.model.g*parameters.model.alpha*phi_ct^(parameters.model.alpha-1))*Dphi_ct;
        Dv_out(3,1) = Dv_out(3,1) - Dftemp2;
    end
end
end

function v_out = temperate_advect(z,phi,params)
%height-dependence equation for the hyperbolic version of temperate ice in steady
%state
Pe_phi = params.Pe_phi;
g = params.g;
alpha = params.alpha;
u_0 = params.u_0;
u_z = params.u_z;
a = params.a;
v_out = a./(Pe_phi*(u_0+u_z*z) + alpha*phi.^(alpha-1)*g);
end

function v_out = temperate_advect_inverted(phi,z,params)
%height-dependence equation for the hyperbolic version of temperate ice in steady
%state, inverted to give dz/dphi
Pe_phi = params.Pe_phi;
g = params.g;
alpha = params.alpha;
u_0 = params.u_0;
u_z = params.u_z;
a = params.a;
v_out = (Pe_phi*(u_0+u_z*z) + alpha*phi.^(alpha-1)*g)./a;
end

function [fout, isterminal, direction] = outer_critical(phi,z,params)
%Event function that determines when the maximum of the q-phi relationship
%is reached, integration for z in terms of phi needs to be terminated at
%this point
Pe_phi = params.Pe_phi;
g = params.g;
alpha = params.alpha;
u_0 = params.u_0;
u_z = params.u_z;
a = params.a;
fout = (Pe_phi*(u_0+u_z*z) + alpha*phi.^(alpha-1)*g)./a;
isterminal = 1;
direction = 0;
end

function [fout, isterminal, direction] = temperate_advect_inverted_projection(phi,v,ztemp)
fout=v-ztemp;
isterminal = zeros(size(ztemp));
direction = 0;
end

function [phi_correct, N_correct] = pressure_correct(z_grid,phi_grid,N_grid,parameters)
z_t = parameters.model.z_t;
if abs(z_grid(1)-z_t) < abs(z_grid(end)-z_t)
    N_t = N_grid(1);
    phi_t = phi_grid(1);
else
    N_t = N_grid(end);
    phi_t = phi_grid(end);
end
if parameters.model.u_0 == 0 && isfield(parameters.model,'zero_velocity_tolerance') && abs(parameters.model.u_z*parameters.model.z_t) < parameters.model.zero_velocity_tolerance*parameters.model.delta
    if ~isfield(parameters,'partition') || ~isfield(parameters.partition,'grid')
        centre_spacing = min(0.25,sqrt(parameters.model.delta));
        min_spacing = min(.05,.125*parameters.model.delta);
        x_max = .5*log(2*centre_spacing/min_spacing);
        nmax = round(max(5,x_max)/centre_spacing);
        parameters.partition.grid = .5*(1+tanh((-nmax:nmax)*centre_spacing));
        parameters.partition.grid = (parameters.partition.grid-min(parameters.partition.grid))/(abs(parameters.partition.grid(end)-parameters.partition.grid(1)));
    elseif min(parameters.partition.grid) > 0
        parameters.partition.grid = (parameters.partition.grid-min(parameters.partition.grid))/(abs(parameters.partition.grid(end)-parameters.partition.grid(1)));
    end
    N_t = parameters.model.a/phi_t;
    parameters.model.phi_t = phi_t;
    %initialize guess
    a = parameters.model.a;
    u_z = parameters.model.u_z;
    Pe_phi = parameters.model.Pe_phi;
    g = parameters.model.g;
    alpha = parameters.model.alpha;
    delta = parameters.model.delta;
    N_0 = parameters.model.N_0;
    z_ct = parameters.model.z_ct;
    if isfield(parameters.model,'z_farfield')
        z_farfield = parameters.model.z_farfield;
    else
        z_farfield = z_ct;
    end
    direction = sign(z_t-z_ct);
    if sign(z_t-z_farfield) ~= direction
        z_farfield = 2*z_t - z_farfield;
    end
    %pre-process
    u_t = u_z*(z_ct-z_t);
    phi_0 = a/N_0;
    %crude approximation, pretending u can be treated as constant as per
    %the standard case, using the length scale delta for the boundary layer
    %to estimate a velocity
    lambda = sqrt((Pe_phi*u_t+alpha*g*phi_t^(alpha-1))/(delta*Pe_phi*u_t*phi_t^(alpha-1)));
    N_temp = (N_0-N_t)*exp(-lambda*abs((z_farfield-z_t)*parameters.partition.grid(1:end-1)))+N_t;
    phi_temp = (phi_0 - phi_t)*exp(-lambda*abs((z_farfield-z_t)*parameters.partition.grid(1:end-1)))+phi_t;
    v_0 = zeros(2*length(parameters.partition.grid)-2,1);
    v_0(1:2:end) = phi_temp;
    v_0(2:2:end) = N_temp;
    v_temp = Newton_single(@zero_velocity_shoot,v_0,parameters);
    [phi_correct, N_correct] = zero_velocity_project(z_grid,v_temp,parameters);
    phi_correct = phi_correct - phi_t;
    N_correct = N_correct - N_t;
else
    %extract parameters
    u_0 = parameters.model.u_0;
    u_z = parameters.model.u_z;
    Pe_phi = parameters.model.Pe_phi;
    g = parameters.model.g;
    alpha = parameters.model.alpha;
    delta = parameters.model.delta;
    N_0 = parameters.model.N_0;
    %pre-process: set N_ct and u_ct
    u_t = u_0 + u_z*z_t;
    lambda = sqrt((Pe_phi*u_t+alpha*g*phi_t^(alpha-1))/(delta*Pe_phi*u_t*phi_t^(alpha-1)));
    N_correct = (N_0-N_t)*exp(-lambda*abs(z_grid-z_t));
    phi_correct = zeros(size(N_correct));
end
end

function [phi_correct N_correct] = pressure_correct_critical(z_grid,parameters)
%set up grid, if necessary
if ~isfield(parameters,'partition') || ~isfield(parameters.partition,'grid')
    centre_spacing = min(0.25,sqrt(parameters.model.delta));
    min_spacing = min(.05,.125*parameters.model.delta);
    x_max = .5*log(2*centre_spacing/min_spacing);
    nmax = round(max(5,x_max)/centre_spacing);
    parameters.partition.grid = .5*(1+tanh((-nmax:nmax)*centre_spacing));
    parameters.partition.grid = (parameters.partition.grid-min(parameters.partition.grid))/(abs(parameters.partition.grid(end)-parameters.partition.grid(1)));
elseif min(parameters.partition.grid) > 0
    parameters.partition.grid = (parameters.partition.grid-min(parameters.partition.grid))/(abs(parameters.partition.grid(end)-parameters.partition.grid(1)));
end
%initialize guess
a = parameters.model.a;
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
Pe_phi = parameters.model.Pe_phi;
g = parameters.model.g;
alpha = parameters.model.alpha;
delta = parameters.model.delta;
z_ct = parameters.model.z_ct;
z_t = parameters.model.z_t;
z_f = parameters.model.z_f;
phi_f = parameters.model.phi_f;
%pre-process
u_f = u_0 + u_z*(z_ct-z_f);
%Initial guess: replicate outer solution up to a distance of order
%delta^(2/5)
if isfield(parameters.model,'z_farfield')
    z_farfield = parameters.model.z_farfield;
else
    z_farfield = z_ct;
end
direction = sign(z_t-z_ct);
if sign(z_t-z_farfield) ~= direction
    z_farfield = 2*z_t - z_farfield;
end
z_0 = z_farfield + (z_t-z_farfield)*parameters.partition.grid(1:end-1);
phi_temp = -(2*a*(z_0-z_t)/(g*alpha*(alpha-1)*phi_f^(alpha-2)));
phi_temp(abs(z_0-z_t)<delta^(2/5)) = max(phi_temp(abs(z_0-z_t)>delta^(2/5)));
N_temp = -Pe_phi*u_f*a./(g*alpha*(alpha-1)*phi_f^(alpha-1)*phi_temp);

%solve for boundary layer
v_0 = zeros(2*length(z_0),1);
v_0(1:2:end) = phi_temp;
v_0(2:2:end) = N_temp;
if isfield(parameters,'srchparams')
    v_temp = Newton_single(@critical_shoot,v_0,parameters,parameters.srchparams);
else
    v_temp = Newton_single(@critical_shoot,v_0,parameters);
end
[phi_correct, N_correct] = critical_project(z_grid,v_temp,parameters);
%add on the second order correction again that was taken off from outer
%solution; this allows correct behaviour for N_correct z_ct but screws up
%the bounary condition at z_t to O(1), which is the same error as that
%incurred in keeping the boundary value N_0 to start with (at leading order
%we should simply put N_correct = 0 at the boundary)
%Similar correction for phi_correct
phi_correct = phi_correct - phi_correct(abs(z_grid-z_ct)==min(abs(z_grid-z_ct)))...
    -(2*a*(z_ct-z_f)/(alpha*(alpha-1)*g*phi_f^(alpha-2))).^(1/2);
N_correct = N_correct+2*a*(alpha-2)/(3*(alpha-1)*phi_f);
end

function [phi_correct N_correct] = flux_correct(z_grid,phi_grid,q_grid,parameters)
%extract parameters   
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
Pe_phi = parameters.model.Pe_phi;
g = parameters.model.g;
alpha = parameters.model.alpha;
delta = parameters.model.delta;
z_ct = parameters.model.z_ct;
%pre-process: set q_ct and u_ct
parameters.model.u_ct = u_0 + u_z*z_ct;
if abs(z_grid(1)-z_ct) < abs(z_grid(end)-z_ct)
    parameters.model.q_ct = q_grid(1);
    parameters.model.phi_ct = phi_grid(1);
else
    parameters.model.q_ct = q_grid(end);
    parameters.model.phi_ct = phi_grid(end);
end
%initial integration to find distance offset
%suitable initial condition on unstable manifold of saddle point
u_ct = parameters.model.u_ct; phi_ct = parameters.model.phi_ct; q_ct = parameters.model.q_ct;
if sign(Pe_phi*u_ct + alpha*g*phi_ct^(alpha-1)) ~= sign(Pe_phi*u_ct), error('Wrong Hamiltonian fixed point'), end
lambda = sqrt((Pe_phi*u_ct + alpha*g*phi_ct^(alpha-1))/(delta*phi_ct^(alpha-1)*Pe_phi*u_ct));
v_0 = [phi_ct; 0];
v_prime = (q_ct/(Pe_phi*u_ct)-phi_ct)/10*[1; -Pe_phi*abs(u_ct)*lambda/phi_ct];
options = odeset('Events',@(z,v)Darcy_zero(z,v,parameters),'RelTol',1e-12,'AbsTol',1e-12);
z_span = [0 2*(z_ct-parameters.model.z_t)];
z_f = [];
while isempty(z_f)
    [z_out,v_out,z_f,v_f,i_f] = ode45(@(z,v)Hamiltonian(z,v,parameters),z_span,v_0+v_prime,options);
    if isempty(z_f)
        z_out = z_out(abs(z_out)<abs(z_span(end)/2));
        v_prime = v_out(length(z_out),:).'-v_0;    %shift v_0 as close to half way along the integration as possible, then repeat
    elseif abs(z_f)<abs(z_span(end)/2)  %should only be relevant if first computation finds z_f in first half of integration
        %if abs(v_prime(2)) > sqrt(eps)*abs(q_ct/(Pe_phi*u_ct)-phi_ct) %limit how close to the fixed point we allow the initial conditions to come, here within a fractional distance in N of sqrt(eps
        if abs(v_prime(2)) > sqrt(eps)*abs((q_ct/phi_ct-Pe_phi*u_ct)*lambda)  %equivalent for fractional distance in N of sqrt(eps)
            v_prime = v_prime/10;
            z_f = [];
        else
            warning('truncated calculation of far field of flux boundary layer')
        end
    end
end
%z_f signifies position of zero Darcy flux boundary, integrate system
%forward to grid points relative to z_f
z_grid_prime = z_grid(z_ct-z_grid > min(0,z_f) & z_ct - z_grid < max(0,z_f));
if  sign(z_grid_prime(end)-z_grid_prime(1)) == sign(z_f)
    z_span = [0 z_grid_prime+z_f-z_grid_prime(end)];
else
    z_span = [0 z_grid_prime(end:-1:1)+z_f-z_grid_prime(1)];
end
if z_span(2) == z_span(1), z_span(1) = z_span(1)-eps; end
options = odeset('RelTol',1e-12,'AbsTol',1e-12);
[z_out, v_out] = ode45(@(z,v)Hamiltonian(z,v,parameters),z_span,v_0+v_prime,options);
phi_correct = zeros(size(z_grid));
N_correct = zeros(size(z_grid));
if sign(z_grid(end)-z_grid(1)) == sign(z_f)
    phi_correct(z_ct-z_grid > min(0,z_f) & z_ct-z_grid < max(0,z_f)) = v_out(2:end,1).'-phi_ct;
    N_correct(z_ct-z_grid > min(0,z_f) & z_ct-z_grid < max(0,z_f)) = v_out(2:end,2).';
else
     phi_correct(z_ct-z_grid > min(0,z_f) & z_ct-z_grid < max(0,z_f)) = v_out(end:-1:2,1).'-phi_ct;
     N_correct(z_ct-z_grid > min(0,z_f) & z_ct-z_grid < max(0,z_f)) = v_out(end:-1:2,2).';   
end
end

function N_correct2 = transform_correct(z,parameters)
%extract parameters
alpha = parameters.model.alpha;
a = parameters.model.a;
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
g = parameters.model.g;
Pe_phi = parameters.model.Pe_phi;
z_c = parameters.model.z_c;
z_t = parameters.model.z_t;
z_ct = parameters.model.z_ct;

%compute velocity (normal, into domain)
u = sign(z_t-z_c)*(u_0+u_z*z_ct);
g = sign(z_t-z_c)*g;

%zeta positions in grid
if isfield(parameters.transform,'grid')
    grid = parameters.transform.grid;
else
    grid = 0:-.5:-100;
end
grid = grid-max(grid);
grid = grid(1:end-1);
grid = reshape(grid,1,length(grid));

%compute boundary layer for inflow boundary in transformed coordinates
if alpha >= 3
    disp('Boundary layer not needed')
elseif alpha >= 2
    parameters.model.phi_tfm = a/(Pe_phi*u)*abs(z_t-z_ct); 
    if isfield(parameters.model,'phi_tfm_farfield')
        parameters.model.phi_tfm = max(parameters.model.phi_tfm,parameters.model.phi_tfm_farfield);
    end
    parameters.model.Psi_tfm = alpha*g*a/(Pe_phi*u)*parameters.model.phi_tfm^(2*alpha-4);
    v_0_temp(1,:) = parameters.model.phi_tfm*exp(a*grid/(Pe_phi*u));
    v_0_temp(2,:) = alpha*g*a/(Pe_phi*u)*v_0_temp(1,:).^(2*alpha-4);
    v_0_temp(3,:) = g/a*v_0_temp(1,:).^(alpha-1);
    v_0 = reshape(v_0_temp,3*length(grid),1);
    v_temp = Newton_single(@pressure_transformed_shoot,v_0,parameters);
    v_temp = reshape(v_temp,3,length(grid));
else
    error('Current code not set up for alpha < 2')
end
N_correct2 = transform_project(v_temp,z,grid,parameters);
end

function [f_out, Df_out] = zero_velocity_shoot(v_in,parameters)
grid = parameters.partition.grid;
n_pts = length(grid);
N_0 = parameters.model.N_0;
a = parameters.model.a;
phi_t = parameters.model.phi_t;
z_t = parameters.model.z_t;
z_ct = parameters.model.z_ct;
if isfield(parameters.model,'z_farfield')
    z_farfield = parameters.model.z_farfield;
else
    z_farfield = z_ct;
end
direction = sign(z_t-z_ct);
if sign(z_t-z_farfield) ~= direction
    z_farfield = 2*z_t - z_farfield;
end
grid = z_farfield + grid*(z_t-z_farfield);

parameters.functions.evolve = @zero_velocity;
ftemp = zeros(2,n_pts-1);
if nargout > 1
    Dftemp = zeros(2,2,n_pts-1);
end
for ii = 1:n_pts-1
    parameters.t_span = [grid(ii) grid(ii+1)];
    v = v_in((ii-1)*2+1:ii*2);
    if nargout == 1
        ftemp(:,ii) = shoot(v,parameters);
    else
        [ftemp(:,ii), Dftemp(:,:,ii)] = shoot(v,parameters);
    end
end 

f_out = zeros(size(v_in));
if nargout > 1
    Df_out = zeros(length(v_in));
end

if direction*grid(1) > direction*grid(end)
    f_out(1) = v_in(2) - N_0;
    f_out(2) = ftemp(1,end) - phi_t;
    if nargout > 1
        Df_out(1,2) = 1;
        Df_out(2,2*(n_pts-2)+(1:2)) = Dftemp(1,:,end);
    end
else
    f_out(1) = ftemp(2,end) - N_0;
    f_out(2) = v_in(1) - phi_t;
    if nargout > 1
        Df_out(1,2*(n_pts-2)+(1:2)) = Dftemp(2,:,end);
        Df_out(2,1) = 1;
    end
end
for ii = 2:n_pts-1
    f_out((ii-1)*2+(1:2)) = ftemp(:,ii-1) - v_in((ii-1)*2+(1:2));
    if nargout > 1
        Df_out((ii-1)*2+(1:2),(ii-2)*2+(1:2)) = Dftemp(:,:,ii-1);
        Df_out((ii-1)*2+(1:2),(ii-1)*2+(1:2)) = -eye(2);                   
    end
end
end

function [f_out Df_out] = critical_shoot(v_in,parameters)
%grid
grid = parameters.partition.grid;
n_pts = length(grid);
z_t = parameters.model.z_t;
z_ct = parameters.model.z_ct;
if isfield(parameters.model,'z_farfield')
    z_farfield = parameters.model.z_farfield;
else
    z_farfield = z_ct;
end
direction = sign(z_t-z_ct);
if sign(z_t-z_farfield) ~= direction
    z_farfield = 2*z_t - z_farfield;
end
grid = z_farfield + grid*(z_t-z_farfield);

%extract parameters
a = parameters.model.a;
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
Pe_phi = parameters.model.Pe_phi;
g = parameters.model.g;
alpha = parameters.model.alpha;
phi_f = parameters.model.phi_f;
z_f = parameters.model.z_f;

%velocity at boundary
u_f = u_0 + u_z*z_f;

%boundary values
N_0 = parameters.model.N_0;
c_inf = -Pe_phi*u_f*a/(g*alpha*(alpha-1)*phi_f^(alpha-1));

parameters.functions.evolve = @critical;
ftemp = zeros(2,n_pts-1);
if nargout > 1
    Dftemp = zeros(2,2,n_pts-1);
end
for ii = 1:n_pts-1
    parameters.t_span = [grid(ii) grid(ii+1)];
    v = v_in((ii-1)*2+1:ii*2);
    if nargout == 1
        ftemp(:,ii) = shoot(v,parameters);
    else
        [ftemp(:,ii), Dftemp(:,:,ii)] = shoot(v,parameters);
    end
end 

f_out = zeros(size(v_in));
if nargout > 1
    Df_out = zeros(length(v_in));
end
if direction*grid(1) > direction*grid(end)
    f_out(1) = v_in(2) - N_0;
    f_out(2) = ftemp(1,end)*ftemp(2,end) - c_inf;
    if nargout > 1
        Df_out(1,2) = 1;
        Df_out(2,2*(n_pts-2)+(1:2)) = ftemp(2,end).*Dftemp(1,:,end) + ftemp(1,end).*Dftemp(2,:,end);        
    end
else
    f_out(1) = ftemp(2,end) - N_0;
    f_out(2) = v_in(1)*v_in(2) - c_inf;
    if nargout > 1
        Df_out(1,2*(n_pts-2)+(1:2)) = Dftemp(2,:,end);
        Df_out(2,1) = v_in(2);        
        Df_out(2,2) = v_in(1);
    end
end
for ii = 2:n_pts-1
    f_out((ii-1)*2+(1:2)) = ftemp(:,ii-1) - v_in((ii-1)*2+(1:2));
    if nargout > 1
        Df_out((ii-1)*2+(1:2),(ii-2)*2+(1:2)) = Dftemp(:,:,ii-1);
        Df_out((ii-1)*2+(1:2),(ii-1)*2+(1:2)) = -eye(2);                   
    end
end
end

function [f_out, Df_out] = pressure_transformed_shoot(v_in,parameters)
grid = parameters.transform.grid;
n_pts = length(grid);
phi_0 = parameters.model.phi_tfm;
Psi_0 = parameters.model.Psi_tfm;

parameters.functions.evolve = @pressure_transformed;
ftemp = zeros(3,n_pts-1);
if nargout > 1
    Dftemp = zeros(3,3,n_pts-1);
end
for ii = 1:n_pts-1
    parameters.t_span = [grid(ii) grid(ii+1)]; 
    v = v_in((ii-1)*3+1:ii*3);
    if nargout == 1
        ftemp(:,ii) = shoot(v,parameters);
    else
        [ftemp(:,ii), Dftemp(:,:,ii)] = shoot(v,parameters);
    end
end 

f_out = zeros(size(v_in));
if nargout > 1
    Df_out = zeros(length(v_in));
end

if grid(1) > grid(end)
    f_out(1) = v_in(1) - phi_0;
    f_out(2) = v_in(2) - Psi_0;
    f_out(3) = ftemp(3,end)/ftemp(1,end);
    if nargout > 1
        Df_out(1,1) = 1; Df_out(2,2) = 1;
        Df_out(3,3*(n_pts-2)+(1:3)) = Dftemp(3,:,end)/ftemp(1,end) - ftemp(3,end)/ftemp(1,end)^2*Dftemp(1,:,end);
    end
else
    f_out(1) = ftemp(1,end) - phi_0;
    f_out(2) = ftemp(2,end) - Psi_0;
    f_out(3) = v_in(3)/v_in(1);
    if nargout > 1
        Df_out(1,3*(n_pts-2)+(1:3)) = Dftemp(1,:,end);
        Df_out(2,3*(n_pts-2)+(1:3)) = Dftemp(2,:,end);
        Df_out(3,3) = 1/v_in(1); Df_out(3,1) = -v_in(3)/v_in(1)^2;
    end
end
for ii = 2:n_pts-1
    f_out((ii-1)*3+(1:3)) = ftemp(:,ii-1) - v_in((ii-1)*3+(1:3));
    if nargout > 1
        Df_out((ii-1)*3+(1:3),(ii-2)*3+(1:3)) = Dftemp(:,:,ii-1);
        Df_out((ii-1)*3+(1:3),(ii-1)*3+(1:3)) = -eye(3);                   
    end
end  
end

function v_out = zero_velocity(z,v_in,n,parameters)
%extract parameters
a = parameters.a;
u_z = parameters.u_z;
Pe_phi = parameters.Pe_phi;
g = parameters.g;
alpha = parameters.alpha;
delta = parameters.delta;
N_0 = parameters.N_0;
phi_t = parameters.phi_t;
z_t = parameters.z_t;

%flux at boundary
q_t = g*phi_t^alpha;

%phi and N
phi = v_in(1);
N = v_in(2);

%output
switch n
    case 0
        v_out = zeros(2,1);
        v_out(1) = (a - phi*N)/(Pe_phi*u_z*z);
        v_out(2) = (q_t/phi^alpha - g)/delta;
    case 1
        v_out = zeros(2,2);
        v_out(1,1) = - N/(Pe_phi*u_z*z); v_out(1,2) =  - phi/(Pe_phi*u_z*z);
        v_out(2,1) = -q_t*alpha/phi^(alpha+1)/delta;
    case 2
        v_out = zeros(2,2,2);
        v_out(1,1,2) =  - 1/(Pe_phi*u_z*z);
        v_out(1,2,1) = v_out(1,1,2);
        v_out(2,1,1) = q_t*alpha*(alpha+1)/phi^(alpha+2)/delta;
end
end

function v_out = critical(z,v_in,n,parameters)
%extract parameters
a = parameters.a;
u_0 = parameters.u_0;
u_z = parameters.u_z;
Pe_phi = parameters.Pe_phi;
g = parameters.g;
alpha = parameters.alpha;
delta = parameters.delta;
phi_f = parameters.phi_f;
z_f = parameters.z_f;

%velocity at boundary
u_f = u_0 + u_z*z_f;

%test critical value being fed in
if abs(Pe_phi*u_f + alpha*g*phi_f^(alpha-1)) > 1e-7
    warning('Outer problem integration may have misidentified critical value of phi')
    disp(phi_f), disp((Pe_phi*u_f/(-alpha*g))^(1/(alpha-1)))
end

%phi and N
phi = v_in(1);
N = v_in(2);

%output
switch n
    case 0
        v_out = zeros(2,1);
        v_out(1) = -phi_f*N/(Pe_phi*u_f);
        v_out(2) = (a*(z-z_f)-g*alpha*(alpha-1)*phi_f^(alpha-2)*phi^2/2)/(delta*phi_f^alpha);
    case 1
        v_out = zeros(2,2);
        v_out(1,2) =  - phi_f/(Pe_phi*u_f);
        v_out(2,1) = -g*alpha*(alpha-1)*phi_f^(alpha-2)*phi/(delta*phi_f^alpha);
    case 2
        v_out = zeros(2,2,2);
        v_out(2,1,1) =  -g*alpha*(alpha-1)*phi_f^(alpha-2)/(delta*phi_f^alpha);
end
end

function v_out = Hamiltonian(z,v_in,parameters)
%parameters
q_ct = parameters.model.q_ct;
u_ct = parameters.model.u_ct;
Pe_phi = parameters.model.Pe_phi;
g = parameters.model.g;
alpha = parameters.model.alpha;
delta = parameters.model.delta;
%dependent variables
phi = v_in(1);
N = v_in(2);
%rates of change
v_out = zeros(2,1);
v_out(1) = -phi*N/(Pe_phi*u_ct);
v_out(2) = ( q_ct/phi^alpha - Pe_phi*u_ct/phi^(alpha-1) - g ) / delta;
end

function [v_out, isterminal, direction] = Darcy_zero(z,v_in,parameters)
q_ct = parameters.model.q_ct;
u_ct = parameters.model.u_ct;
Pe_phi = parameters.model.Pe_phi;
phi = v_in(1);    
v_out = q_ct-Pe_phi*u_ct*phi;
isterminal = true;
direction = 0;
end

function v_out = pressure_transformed(z,v_in,n,parameters)
Pe_phi = parameters.Pe_phi;
g = sign(parameters.z_t-parameters.z_c)*parameters.g;
alpha = parameters.alpha;
delta = parameters.delta;
u = sign(parameters.z_t-parameters.z_c)*(parameters.u_0 + parameters.u_z*parameters.z_ct);
a = parameters.a;

phi = v_in(1);
Psi = v_in(2);
chi_tilde = v_in(3);

switch n
    case 0
        v_out = zeros(3,1);
        v_out(1) = a/(Pe_phi*u)*phi;
        v_out(2) = a*chi_tilde/delta - g*phi^(alpha-1)/delta + (alpha-2)*a/(Pe_phi*u)*Psi;
        v_out(3) = -a/(Pe_phi*u)*chi_tilde + 1/a*phi^(3-alpha)*Psi;
    case 1
        v_out = zeros(3,3);
        v_out(1,1) = a/(Pe_phi*u);
        v_out(2,1) = -(alpha-1)*g*phi^(alpha-2)/delta;
        v_out(2,3) = a/delta;
        v_out(2,2) = (alpha-2)*a/(Pe_phi*u);
        v_out(3,1) = (3-alpha)/a*phi^(2-alpha)*Psi;
        v_out(3,3) = -a/(Pe_phi*u);
        v_out(3,2) =  1/a*phi^(3-alpha);
    case 2
        v_out = zeros(3,3,3);
        v_out(2,1,1) = -(alpha-1)*(alpha-2)*g*phi^(alpha-3)/delta;
        v_out(3,1,1) = (3-alpha)*(2-alpha)/a*phi^(1-alpha)*Psi;
        v_out(3,1,2) = (3-alpha)/a*phi^(2-alpha);
        v_out(3,2,1) = v_out(3,1,2);
end
end

function [phi_correct, N_correct] = zero_velocity_project(z,v_in,parameters)
%integration of ordinary differential equation
z_t = parameters.model.z_t;
z_ct = parameters.model.z_ct;
if isfield(parameters.model,'z_farfield')
    z_farfield = parameters.model.z_farfield;
else
    z_farfield = z_ct;
end
direction = sign(z_t-z_ct);
if sign(z_t-z_farfield) ~= direction
    z_farfield = 2*z_t - z_farfield;
    disp('bad z_farfield')
end
shoot_grid = z_farfield + (z_t-z_farfield)*parameters.partition.grid;
phi_correct = zeros(size(z));
N_correct = zeros(size(z));
for ii = 1:length(shoot_grid)-1;
    zgrid = z(z>=min(shoot_grid(ii:ii+1))&z<=max(shoot_grid(ii:ii+1)));
    if ~isempty(zgrid)
        if abs(zgrid(end)-shoot_grid(ii)) < abs(zgrid(1)-shoot_grid(ii))
            zgrid = zgrid(end:-1:1);
        end
        zgrid = reshape(zgrid,1,length(zgrid));
        if zgrid(1) == shoot_grid(ii)
            zgrid_calc = zgrid;
        else
            zgrid_calc = [shoot_grid(ii) zgrid];
        end
        [z_out,v_out] = ode45(@(zint,v)zero_velocity(zint,v,0,parameters.model),zgrid_calc,v_in(2*(ii-1)+(1:2)));
        for jj = 1:length(z)
            if z(jj)>=min(shoot_grid(ii:ii+1))&&z(jj)<=max(shoot_grid(ii:ii+1))
                kk = find(z_out == z(jj));
                if ~isempty(kk)
                    phi_correct(jj) = v_out(kk,1);
                    N_correct(jj) = v_out(kk,2);
                else
                    phi_correct(jj) = NaN; N_correct(jj) = NaN;
                end
            end
        end
    end
end
end

function [phi_correct, N_correct] = critical_project(z,v_in,parameters)
%integration of ordinary differential equation
%grid
z_t = parameters.model.z_t;
z_ct = parameters.model.z_ct;
if isfield(parameters.model,'z_farfield')
    z_farfield = parameters.model.z_farfield;
else
    z_farfield = z_ct;
end
direction = sign(z_t-z_ct);
if sign(z_t-z_farfield) ~= direction
    z_farfield = 2*z_t - z_farfield;
end
shoot_grid = z_farfield + (z_t-z_farfield)*parameters.partition.grid;
phi_correct = zeros(size(z));
N_correct = zeros(size(z));
for ii = 1:length(shoot_grid)-1;
    zgrid = z(z>=min(shoot_grid(ii:ii+1))&z<=max(shoot_grid(ii:ii+1)));
    if ~isempty(zgrid)
        if abs(zgrid(end)-shoot_grid(ii)) < abs(zgrid(1)-shoot_grid(ii))
            zgrid = zgrid(end:-1:1);
        end
        zgrid = reshape(zgrid,1,length(zgrid));
        zgrid_calc = [shoot_grid(ii) zgrid];
        [z_out,v_out] = ode45(@(zint,v)critical(zint,v,0,parameters.model),zgrid_calc,v_in(2*(ii-1)+(1:2)));
        for jj = 1:length(z)
            if z(jj)>=min(shoot_grid(ii:ii+1))&&z(jj)<=max(shoot_grid(ii:ii+1))
                kk = find(z_out == z(jj));
                if ~isempty(kk)
                    phi_correct(jj) = v_out(kk,1);
                    N_correct(jj) = v_out(kk,2);
                else
                    phi_correct(jj) = NaN; N_correct(jj) = NaN;
                end
            end
        end
    end
end
end

function N_correct3 = transform_project(v_in,z,grid,parameters)
direction = sign(parameters.model.z_t-parameters.model.z_c);
z_ct = parameters.model.z_ct;
ztemp = z(direction*(z-z_ct)>0);
options = odeset('Events',@(z_int,v)transform_grid_crossing(z_int,v,ztemp,parameters.model));
v_transform = zeros(length(ztemp),3);
z_transform = zeros(size(ztemp));
N_correct3 = zeros(size(ztemp));
counter = 0;
for ii = 1:length(grid)-1
    [zetatemp,vtemp,zetai,vi,zi] =ode45(@(zint,v)pressure_transformed(zint,v,0,parameters.model),[grid(ii) grid(ii+1)],v_in(3*(ii-1)+(1:3)),options);
    if ~isempty(zetai)
        v_transform(counter+1:counter+length(zetai),:)=vi;
        z_transform(counter+1:counter+length(zetai))=ztemp(zi);
        counter = counter+length(zetai);
    end
end
beta = parameters.model.alpha-2;
for ii = 1:length(z)
    if direction*(z(ii)-z_ct)>0
        jj = find(z_transform == z(ii),1);
        if ~isempty(jj)
            N_correct3(ii) = v_transform(jj,1)^(-beta)*v_transform(jj,2);
        else
            N_correct3(ii) = NaN;
        end
    end
end
%correction
alpha = parameters.model.alpha;
Pe_phi = parameters.model.Pe_phi;
z_ct = parameters.model.z_ct;
z_t = parameters.model.z_t;
direction = sign(z_t-z_ct);
normal = max(0,(ztemp-z_ct)*direction);
u_n = direction*(parameters.model.u_0+parameters.model.u_z*z_ct);
g_n = parameters.model.g*direction;
a = parameters.model.a;
%delta = parameters.model.delta;
N_correct3 = N_correct3 - alpha*g_n*(a/(Pe_phi*u_n))^(alpha-1)*normal.^(alpha-2);
%Remove higher order terms in far field; however, this makes the near field
%inaccurate as we should include additional terms. Leave commented out
%C_0 = (a/(Pe_phi*u_n))^(alpha-1);
%C = g_n;
%for jj = 0:floor(1/(3-alpha)-1)
%    C = C*C_0*(1+jj*(alpha-3))*((jj+1)*alpha-jj*3);
%    N_correct3 = N_correct3 - delta^jj*C*normal.^(1-(jj+1)*(3-alpha));
%end
%Alterntaive version: remove first order correction for alpha > 2.5
%if alpha >= 5/3
%    delta = parameters.model.delta;
%    N_correct3 = N_correct3 - delta*alpha*(alpha-2)*(2*alpha-3)*(a/Pe_phi*u_n)^(2*alpha-2)*g_n*normal.^(2*alpha-5);
%end
end

function [fout, isterminal, direction] = transform_grid_crossing(z,v,ztemp,parameters)
    Pe_phi = parameters.Pe_phi;
    a = parameters.a;
    u = abs(parameters.u_0 + parameters.u_z*parameters.z_ct);
    fout = parameters.z_ct + sign(parameters.z_t-parameters.z_c)*Pe_phi*u/a*v(1)-ztemp;
    isterminal = zeros(size(ztemp));
    direction = 0;
end
