function parameters_out = grid_generator(parameters)
%parameters_out = grid_generator(parameters)
%Adds grid to fields to an otherwise complete parameter structure as
%specified in steady_solver.m. The grid is designed to capture the details
%of various known boundary layers in the limit of small delta
%
%Input argument:
%   parameters:  parameter structure containing all necessary entries in
%                   the substructure 'model' for use in steady_solver.
%
%Output:
%   parameters_out: the same as parameters, but with substructures
%                   'partition' and 'transform' either newly created or
%                   overwritten, with a grid chosen based on model
%                   parameters

centre_spacing = min(0.25,sqrt(parameters.model.delta));
if parameters.model.alpha < 3
    min_spacing = min([.05,.25*parameters.model.delta,10*parameters.model.delta^(1/(3-parameters.model.alpha))]); 
else
    min_spacing = min(.05,.25*parameters.model.delta);
end
x_max = .5*log(2*centre_spacing/min_spacing);
nmax = round(max(5,x_max)/centre_spacing);
parameters.partition.grid = .5*(1+tanh((-nmax:nmax)*centre_spacing));
parameters.partition.grid = (parameters.partition.grid-min(parameters.partition.grid))/(abs(parameters.partition.grid(end)-parameters.partition.grid(1)));
if sign(parameters.model.z_t-parameters.model.z_c) == sign(parameters.model.u_0) || sign(parameters.model.z_t-parameters.model.z_c) == sign(parameters.model.u_z)
    if parameters.model.alpha > 2 && parameters.model.alpha < 3
        offset = min(.05,10*parameters.model.delta^(1/2));
        %commented-out version attempts to scale this to size of inflow
        %boundary layer, which however leads to difficulties solving the
        %untransformed problem when alpha is close to 3, as the boundary
        %layer becomes extremely narrow
        %offset = min([.05,10*parameters.model.delta^(1/(3-parameters.model.alpha)),10*parameters.model.delta^(1/2)]);
    else
        offset = min(.05,10*parameters.model.delta^(1/2));
    end
    parameters.partition.grid = offset + (1-offset)*parameters.partition.grid;
    parameters.transform.transform = true;
    zeta_scale = parameters.model.Pe_phi*max(abs(parameters.model.u_0),abs(parameters.model.u_z*(parameters.model.z_c-parameters.model.z_t)))/parameters.model.a;
    if parameters.model.alpha <= 3
        delta_zeta_scale = min(.125,parameters.model.delta^(1/2)*offset^((parameters.model.alpha-3)/2)/zeta_scale);
    else
        delta_zeta_scale =  min(.125,parameters.model.delta^(1/2)/zeta_scale); 
    end
    if parameters.model.alpha <= 3
        zeta_scaled_max = 12;
        parameters.transform.grid = zeta_scale*(0:-delta_zeta_scale:-zeta_scaled_max);
    else
        zeta_scaled_max = round(2/(parameters.model.alpha-2)/1e-4^((parameters.model.alpha-3)/2));
        parameters.transform.grid = zeta_scale*(0:-delta_zeta_scale:-zeta_scaled_max);
    end
end

parameters_out = parameters;
end