function [phi, N, T, q] = grid_projection(v_in,z_in,parameters)
%[phi, N, T, q] = grid_projection(v_in,z_in,parameters)
%grid_projection converts the raw output of steady_solver.m (see
%documentation for steady_solver.m) to values of phi, N, T and q on a user-
%specified grid.
%
%Input arguments:
%   v_in:   solution structure as output by steady_solver
%   z_in:   Vector giving grid point positions at which solution is to be
%           computed. Should be an ordered vector of values between z_t and
%           z_c
%   parameters: parameter structure as used by steady_solver. This can be
%           read in from v_in if it is not supplied separately. If the
%           'display' flag is set (parameters.dispaly = true), the
%           grid_projection.m will graphically output the solution computed
%           on figure 99. The colour scheme used is:
%               T:      magenta
%               phi:    green
%               N:      blue
%               q:      red
%
%Christian Schoof, January 2015
%
%Dependencies: temperate_ode.m, temperate_ode_transformed.m

%use parameters from v_in if not separately supplied
if nargin < 3
    parameters = v_in.parameters;
end
   
%extract parameters
Pe_phi = parameters.model.Pe_phi;
Pe = parameters.model.Pe;
g = parameters.model.g;
alpha = parameters.model.alpha;
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
a = parameters.model.a;
z_c = parameters.model.z_c;
z_t = parameters.model.z_t;
z_ct = v_in.therm(1);
A = v_in.therm(2);
B = v_in.therm(3);
%initialize
phi = zeros(size(z_in)); N=zeros(size(z_in));
z = reshape(z_in,1,length(z_in));
%temperature field
if u_0 ~= 0
    T = A/(Pe*u_0)*exp(Pe*u_0*z) + a*z/(Pe*u_0) + B;
    q = Pe*u_0*T - ( A*exp(Pe*u_0*z) + a/(Pe*u_0));
else
    if isfield(parameters,'integration')
        x_int = parameters.integration.nodes;
        w_int = parameters.integration.weights;
    else
       [x_int,w_int] =  gaussquad(64,1);
    end
    z_aux = reshape(z,1,length(z));
    z_aux = repmat(z_aux,length(x_int),1);
    x_aux = repmat(x_int,1,length(z));
    w_aux = repmat(w_int,1,length(z));
    if u_z > 0
        Pe = Pe*u_z;
        T = -2*a/Pe*sum(w_aux.*dawson_minus(x_aux*sqrt(Pe/2).*z_aux)*sqrt(Pe/2).*z_aux,1)...
            + A*sqrt(2/Pe)*exp(Pe*z.^2/2).*dawson_plus(sqrt(Pe/2)*z) + B;
        q = Pe*u_z*z.*T -( A*exp(Pe*z.^2/2) - a*sqrt(2/Pe)*dawson_minus(sqrt(Pe/2)*z) );
    else
        Pe = Pe*abs(u_z);
        T = -2*a/Pe*sum(w_aux.*dawson_plus(x_aux*sqrt(Pe/2).*z_aux)*sqrt(Pe/2).*z_aux,1)...
           + A*sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z)+ B;
        q = Pe*sign(u_z)*z.*T -( A*exp(-Pe*z.^2/2) - a*sqrt(2/Pe).*dawson_plus(sqrt(Pe/2)*z) );
    end
    T = reshape(T,size(z));
    q = reshape(q,size(q));
end
T(z<min(z_c,z_ct)|z>max(z_c,z_ct)) = 0;
q(z<min(z_c,z_ct)|z>max(z_c,z_ct)) = 0;

%integration of ordinary differential equation
direction = sign(z_t - z_c);
shoot_grid = z_ct + (z_t-z_ct)*parameters.partition.grid;
n_pts = length(shoot_grid);
if (direction == sign(u_0) || direction == sign(u_z)) && parameters.transform.transform
    if direction*shoot_grid(1) < direction*shoot_grid(end)
        shoot_grid = shoot_grid(2:end);
    else
        shoot_grid = shoot_grid(1:end-1);
    end
    parameters.model.direction = direction;
    parameters.model.z_ct = z_ct;
    transform_grid = parameters.transform.grid;
    ztemp = z(direction*(z-z_ct)>0&direction*(z-z_ct)<min(direction*(shoot_grid-z_ct)));
    options = odeset('Events',@(z_int,v)transform_grid_crossing(z_int,v,z_ct,ztemp,parameters.model));
    v_transform = zeros(length(ztemp),4);
    z_transform = zeros(size(ztemp));
    counter = 0;
    for ii = 1:length(transform_grid)-1
           [zetatemp,vtemp,zetai,vi,zi] =ode45(@(zint,v)temperate_ode_transformed(zint,v,0,parameters.model),[transform_grid(ii) transform_grid(ii+1)],v_in.temp(3*(n_pts-2)+4*(ii-1)+(1:4)),options);
           if ~isempty(zetai)
               v_transform(counter+1:counter+length(zetai),:)=vi;
               z_transform(counter+1:counter+length(zetai))=ztemp(zi);
               counter = counter+length(zetai);
           end
    end
        if alpha > 3
            beta = (alpha-1)/2;
        else
            beta = alpha-2;
        end
        for ii = 1:length(z)
           if direction*(z(ii)-z_ct)>0
               jj = find(z_transform == z(ii),1);
               if ~isempty(jj)
                   phi(ii) = v_transform(jj,1);
                   N(ii) = v_transform(jj,1)^(-beta)*v_transform(jj,2);
                   q(ii) = a*(z(ii)-z_ct) + direction*v_transform(jj,1)*v_transform(jj,3);
               else
                   phi(ii) = NaN; N(ii) = NaN; q(ii) = NaN;
               end
           end
       end
elseif min(shoot_grid) > 5e-3 || max(shoot_grid) < 1-5e-3
    warning('grid may not cover temperate domain')
end
for ii = 1:length(shoot_grid)-1;
    zgrid = z(z>=min(shoot_grid(ii:ii+1))&z<=max(shoot_grid(ii:ii+1)));
    if ~isempty(zgrid)
        if abs(zgrid(end)-shoot_grid(ii)) < abs(zgrid(1)-shoot_grid(ii))
            zgrid = zgrid(end:-1:1);
        end
        zgrid = reshape(zgrid,1,length(zgrid));
        zgrid_calc = [shoot_grid(ii) zgrid];
        [z_out,v_out] = ode45(@(zint,v)temperate_ode(zint,v,0,parameters.model),zgrid_calc,v_in.temp(3*(ii-1)+(1:3)));
        for jj = 1:length(z)
            if z(jj)>=min(shoot_grid(ii:ii+1))&&z(jj)<=max(shoot_grid(ii:ii+1))
                kk = find(z_out == z(jj));
                if ~isempty(kk)
                    phi(jj) = v_out(kk,1);
                    N(jj) = v_out(kk,2);
                    q(jj) = v_out(kk,3);
                else
                    phi(jj) = NaN; N(jj) = NaN; q(jj) = NaN;
                end
            end
        end
    end
end
N(z<min(z_t,z_ct)|z>max(z_t,z_ct)) = NaN;


if isfield(parameters,'display')
    figure(99), hold on
    plot(z,T,'m')
    plot(z,phi,'g')
    plot(z,N,'b')
    plot(z,q,'r')
end
end

function [fout, isterminal, direction] = transform_grid_crossing(z,v,z_ct,ztemp,parameters)
fout = z_ct + sign(parameters.z_t-parameters.z_c)*v(4)*v(1)-ztemp;
isterminal = zeros(size(ztemp));
direction = 0;
end