function v_out = steady_solver(v_in,parameters,grid)
%v_out = steady_solver(v_in,parameters,grid)
%Solves steady state temperate ice problem in one dimension
%The problem can be stated in the form
%   dphi/dz = (a - phi*N)/(Pe_phi*u)
%   dN/dz = (q - Pe_phi*u*phi)/(delta*phi^alpha) - g/delta
%   dq/dz = a + Pe_phi*du/dz*phi
%in the temperate part of the domain, z \in (min(z_t,z_ct),max(z_t,z_ct)),
%and
%   Pe*u*T_z - T_zz = a
%in the cold part of the domain, z \in (min(z_c,z_ct),max(z_c,z_ct))
%At the free boundary z = z_ct, boundary conditions are
%   T = 0
%   Pe*u*T - T_z = Pe_phi*u*phi + phi^alpga*(g+delta dN/dz)
%   phi^alpga*(g+delta dN/dz) = 0
%At the fixed cold surface z = z_c, a Dirichlet condition is applied on
%temperature
%   T = T_0 < 0
%while at z = z_t, a Dirichlet condition is likewise applied to N
%   N = N_0
%In addition, if u*sign(z_c-z_t) > 0 at z=z_t, we need a Cauchy condition
%on phi,
%   phi = phi_0 > 0
%Note that we call this situation an "outflow problem" because it implies
%ice flows from the temperate subdomain into the cold subdomain. The
%reverse case is called an "inflow problem".
%
%The velocity field u can take either the form u = u_0 = constant or u =
%u_z*z with u_z = constant. In the latter case, u must not change sign
%between z = z_t and z = z_ct; in fact, it must not vanish either but may
%become small near z = z_t.
%There are situations in which no solutions exist, primarily when
%gravity counteracts advection of moisture in the temperate region and
%gravity becomes dominant somewhere in the temperate region. Alternatively,
%for sufficiently cold temperatures at z=z_c, no temperate ice may form, so
%there is no z_ct between z_t and z_c. In either case, steady_solver.m will
%fail to converge.
%
%steady_solver solves these equations using a generalized multiple shooting
%method, employing a coordinate transform detailed in the documentation of
%temperate_ode_transform to deal with the singular nature of the equation
%for N in the vicinity of a boundary where phi goes to zero.
%
%The shooting method is in turn based on Newton's method for rootfinding,
%and convergence is not guaranteed even when a solution exists. The routine
%temperate_initialize_v3 is called as a subroutine if no initial guess is
%supplied. This should be done whenever a high quality initial guess  for a
%solution is not available. temperate_initialize_v3 uses the solution to a
%leading-order problem in small delta to provide an initial guess.
%
%Input arguments:
%   v_in:   an initial guess for the solution. This is optional, if an
%           empty vector is supplied instead, steady_solver calls
%           temperate_initialize_v3 to create the initial guess instead
%           (recommended). The structure of the initial guess depends on
%           the partition of the temperate interval and other parameter
%           settings as described below.
%   parameters: An optional parameter structure with the following fields
%       model:  A substructure containing all necessary model parameters
%           Pe:     temperature Peclet number (defaults to 2)
%           Pe_phi: moisture Peclet number (defaults to 1)
%           a:      dissipation rate (defaults to 1)
%           g:      signed gravitational acceleration (defaults to -1)
%           delta:  compaction length scale parameter (defaults to 1)
%           alpha:  permeability exponent (defaults to 3)
%           u_0:    constant part of velocity field (defaults to 1)
%           u_z:    strain rate (defaults to 0)
%                   Note that only one of u_0, u_z can be non-zero at any
%                   one time; one mof them must also be non-zero
%           z_c:    location of cold boundary (defaults to 1)
%           z_t;    location of warm boundary (defaults to sqrt(eps))
%           N_0:    boundary condition on N at z=z_t (defaults to 1)
%           phi_0:  boundary condition on phi at z=z_t (defaults to 1,
%                   applied only when required)
%           T_0:    boundary condition on T at z=z_c (defaults to -1)
%       partition:  substructure specifying the partitioning of the interval
%                   from z_t to z_ct, containing the single field
%           grid:   ordered row vector giving partition points between 0
%                   and 1, with correct computations requiring grid(1)=0,
%                   grid(end)=1 or vice versa. This partition is used in
%                   the shooting solution of the temperate ice equations,
%                   using partition points at z = z_ct
%                   +partition.grid(ii)*(z_t-z_ct), shooting from ii=1 to
%                   ii = length(partition.grid)
%       transform:  optional substructure specifying how the coordinate
%                   transformation described in the documentation for
%                   temperate_ode_transform is applied, with fields
%           transform:  Boolean, specfiyng whether transformation is
%                   applied to the partition closest to z_ct (true) or not
%                   (false); used only for inflow problems, where ice
%                   velocity is from the cold subdomain into the temperate
%                   subdomain. Using the transformation appears to be more
%                   or less essential for inflow problems; an alternative
%                   of having a vary finely divided partition.grid near
%                   values of zero while not allowing zero to be reached
%                   could also work, but precisely how to devise that
%                   partition is unclear.
%           grid:   a row vector specifying the integration interval with
%                   respect to the transformed independent variable zeta
%                   and interior partition points. As with partition.grid,
%                   transform.grid must be ordered. Integration in zeta is
%                   from transform.grid(1) to transform.grid(end). When the
%                   transformation is applied, the partition interval
%                   closest to zero (so either from partition.grid(1) to
%                   partition.grid(2) or from partition.grid(end-1) to
%                   partition.grid(end) is omitted from the shooting method
%                   in the original variables. Instead, original at the end
%                   point of the next interval (at partition.grid(2) or
%                   partition.grid(end-1)) are matched with the transformed
%                   variables at the end point in transform.grid
%                   corresponding to the largest (least negative) value of
%                   zeta (so transform.grid(1) or transform.grid(end)). The
%                   transformed variables at the most negative value of
%                   zeta are then matched with the required asymptotic
%                   behaviour of an orbit on the stable or centre manifold
%                   connecting to the fixed point of the transformed
%                   system
%       weights:    optional substructure defining weights to apply to the
%                   boundary conditions when assembling the function whose
%                   zero the shooting method aims to compute. This is
%                   really a legacy item and the default values of 1 should
%                   be used. weights has fields N_0, phi_0, T_0.
%       integration: substructure defining integration weights and nodes
%                   used in the computation of the semianalytical
%                   temperature solution for the case of non-zero u_z. It
%                   has fields
%           nodes:  vector containing nodes for integration over the
%                   interval [0,1]. If compatibility with
%                   asymptotic_solution is desired, this needs to be a
%                   column vector
%           weights: the corresponding weights, same size vector as nodes.
%       srchparams: optional substructure, passed onto Newton solver
%                   Newton_single.m in place of srchparams. See the
%                   documentation for Newton_single.m
%       temp_guess: optional field containing a 3-by-1 vector as an initial
%                   guess for the argument [z_ct; A; B] taken by tempsol.
%                   Used only if the actual initial guess v_0 is left empty
%                   and temperate_intialize_v2 is used to compute the
%                   initial guess. This can be used to improve computation of
%                   that initial guess by temperate_initialize_v3 (which
%                   itself needs an initial guess for these three
%                   argumentts in order to compute the relevant small-delta
%                   asymptotic solution; if temp_guess is not supplied,
%                   temperate_initialize_v3 uses a small Peclet number Pe
%                   approximation combined with (for an outflow problem) a
%                   large moisture Peclet number approximation. Improved
%                   guesses of [z_ct; A; B] for outflow problems can often
%                   be obtained by using advect_solution.m
%       display:    boolean, if set to true, grid_projection (called if an
%                   input variable grid is provided) will plot the
%                   final numerical solution in figure 99. The colour
%                   scheme used is:
%                       T:      magenta
%                       phi:    green
%                       N:      blue
%                       q:      red
%   grid: an optional vector of ordered grid points at which the solution for
%       T,N,phi,q are to be evaluated; as the basic solution method used by
%       steady_solver.m is a multiple shooting method, the basic solution
%       consists of values of dependent variables at the partition points
%       used by the multiple shooting method. Computing dependent variables
%       at predefined (generally, more closely spaced) positions requires
%       an additional "projection" step using the original ode solvers;
%       this projection step is slightly non-trivial for any intervals in
%       which the transformed system of temperate ice equations is used,
%       where it requires the use of event functions. The routine used to
%       handle all of this is grid_projection.m
%   v_in recap: having described the grid, we can now define the format of
%   the input vector v_in. v_in is a column vector with the following
%   entries
%       v_in(1:3):  guesses for [z_ct; A; B], these being the
%                   cold-temperate transition boundary and the
%                   constants of integration A and B as they appear in the
%                   documentation to tempsol.m
%       v_in(3+ii*(1:3)): here ii goes from 1 to length(partition.grid)-1
%                   if the transformed equations are not used, and from 1
%                   to length(partition.grid)-2 if the transformed
%                   equations are used. For each ii, the vector
%                   v_in(3+(ii)*(1:3)) contains guesses for the values of
%                   [phi(ii), N(ii), q(ii)] at location z_ct +
%                   partition.grid(ii)*(z_t-z_ct) if either the transformed
%                   equations are not used or if partition.grid(1) >
%                   partition.grid(end), and at location z_ct +
%                   partition.grid(ii-1)*(z_t-z_ct) if the transformed
%                   equations are used and
%                   partition.grid(1)<partition.grid(end) (so the first
%                   partition is solved in transformed coordinates)
%       v_in(3+(n_pts-1)*3+ii*(1:4)): here n_pts = length(.partitongrid) if
%                   the transformed equations are not used, and
%                   length(partition.grid)-1 if they are, while ii gues
%                   from 1 to length(transform.grid)-1. For each ii, the
%                   vector v_in(3+n_pts*3+ii*(1:4)) contains a guess for
%                   [phi;Psi;omega;chi] at location zeta=transform.grid(ii)
%
%Output:
%   v_out:  output structure with fields
%       parameters: the parameters structure used, including any internally
%                   generated default settings
%                   equations are not used or if partition.grid(1)>partition.grid(end), and at location 
%       therm:      the vector [z_ct;A;B] at the solution, where z_ct is
%                   the cold-temperate boundary location and A and B are
%                   the constants of integration specified in the
%                   documentation to tempsol.m
%       temp:       the equivalent values to v_in(4:end) but evaluated at
%                    the solution rather than given as guessse
%       grid:       locations of the grid points specified at input, if
%                   they were
%       grid_T, grid_N, grid_phi, grid_q: the corresponding values of the
%                   variables T, N, phi, q at these grid points
%
%Dependencies: temperate_intialize_v2, temperate_steady_v3, tempsol,
%temperate_ode, temperate_ode_transformed, grid_projection, shoot,
%dawson_plus, dawson_minus, faddeeva, gaussquad
%
%Christian Schoof, January 2015

%% Default settings, if necessary

%identify partition of temperate region
if nargin == 1 || isempty(parameters) || ~isfield(parameters,'partition')
    parameters.partition.grid = 1:-.1:0;
end
if min(parameters.partition.grid) < 0 || max(parameters.partition.grid) > 1
    error('The values in parameters.partition.grid need to lie between 0 and 1')
end
if ~issorted(parameters.partition.grid) && ~issorted(-parameters.partition.grid)
    error('The values in parameters.partition.grid need to be in ascending or descending order')
end
n_pts = length(parameters.partition.grid);

%transformation parameters
if isfield(parameters,'transform')  && (~isfield(parameters.transform,'transform') || parameters.transform.transform)
    transform = true;
    if ~isfield(parameters.transform,'grid')
        parameters.transform.grid = 0:-.2:-50;
    end
    if ~issorted(parameters.transform.grid) &&  ~issorted(-parameters.transform.grid)
        error('The values in parameters.transform.grid must be in ascending or descending order')
    end
else
    transform = false;
end
if transform
    n_pts_transform = length(parameters.transform.grid);
else
    n_pts_transform = 1;
end
    
%weights for compliance with boundary conditions
if ~isfield(parameters,'weights')
    parameters.weights.N_0 = 1;
    parameters.weights.phi_0 = 1;
    parameters.weights.q_0 = 1;
end
%integration nodes and weights
if ~isfield(parameters,'integration')
    [parameters.integration.nodes, parameters.integration.weights] = gaussquad(64,1);
end

%extract or create physical model parameters
if isfield(parameters,'model')
    if ~isfield(parameters.model,'a')
        parameters.model.a = 1;
    end
    if ~isfield(parameters.model,'Pe')
        parameters.model.Pe = 2;
    end
    if ~isfield(parameters.model,'Pe_phi')
        parameters.model.Pe_phi = 1;
    end
    if ~isfield(parameters.model,'g')
        parameters.model.g = -1;
    end
    if ~isfield(parameters.model,'delta')
        parameters.model.delta = 1;
    end
    if ~isfield(parameters.model,'alpha')
        parameters.model.alpha = 3;
    end
    if ~isfield(parameters.model,'u_0')
        parameters.model.u_0 = 1;
    end
    if ~isfield(parameters.model,'u_z')
        if parameters.model.u_0 == 0
            parameters.model.u_z = 1;
        else
            parameters.model.u_z = 0;
        end
    end
    if parameters.model.u_0 ~= 0 && parameters.model.u_z ~= 0
        error('either base velocity u_0 or strain rate u_z must be zero; to have a non-zero velocity and non-zero strain rate at the boundaries, choose both boundaries z_c and z_t to be non-zero instead')
    end

    if ~isfield(parameters.model,'z_c')
        parameters.model.z_c = 1+sqrt(eps);
    end
    if ~isfield(parameters.model,'z_ct')
        parameters.model.z_ct = .5; %for debugging and testing purposes only
    end
    if ~isfield(parameters.model,'z_t')
        parameters.model.z_t = sqrt(eps);
    end
    if ~isfield(parameters.model,'T_0')
        parameters.model.T_0 = -1;
    end
    if ~isfield(parameters.model,'N_0')
        parameters.model.N_0 = 1;
    end
    if ~isfield(parameters.model,'phi_0')
        if abs(parameters.model.z_t) < 100*sqrt(eps) && parameters.model.u_0 == 0
            parameters.model.phi_0 = parameters.model.a/parameters.model.N_0;
        else
            parameters.model.phi_0 = 1;
        end
    end
else
    parameters.model.a = 1;
    parameters.model.Pe = 2;
    parameters.model.Pe_phi = 1;
    parameters.model.g = -1;
    parameters.model.delta = 1;
    parameters.model.alpha = 3;
    parameters.model.u_0 = 1;
    parameters.model.u_z = 0;
    parameters.model.T_0 = -1;
    parameters.model.N_0 = 1;
    parameters.model.phi_0 = parameters.model.a/parameters.model.N_0;
    parameters.model.z_c = 1;
    parameters.model.z_t = sqrt(eps);
    parameters.model.z_ct = 0.5; %for debugging and testing purposes only
end

%% Set up for particular problem, depending on z_t, z_c, u_0 and u_z
if ~isempty(v_in)
    if ~(isfield(parameters,'transform') && parameters.transform.transform && ...
            (sign(parameters.model.z_t-parameters.model.z_ct) == sign(parameters.model.u_0) ||...
            sign(parameters.model.z_t-parameters.model.z_ct) == sign(parameters.model.u_z)) &&...
            length(v_in) == 3 + 3*(n_pts-2) + 4*(n_pts_transform-1)) && ...
            (length(v_in) ~= 3 + 3*(n_pts-1))
        error('Input argument v_in has the wrong length')       
    end
else
    if isfield(parameters,'temp_guess') && ~isempty(parameters.temp_guess)
        v_in = temperate_initialize_v3(parameters.temp_guess,parameters);
    else
        v_in = temperate_initialize_v3([],parameters);
    end
    %v_out = v_in; return
end

%% Jacobian test
if isfield(parameters,'flags') && isfield(parameters.flags,'test') && parameters.flags.test
    parameters.flags.test = false; %steps test flag being passed onto shooting methods
    v_out.parameters = parameters;
    v_out.v_in = v_in;
    if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
        [v_out.f v_out.Df] = temperate_steady_v3(v_in(4:end),parameters);
    else
        [v_out.f v_out.Df] = temperate_steady_v3(v_in,parameters);       
    end
    if isfield(parameters.flags,'identity') && parameters.flags.identity
        if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
            v_out.tempsol = Jacobian_test_single(@tempsol,ones(3,1),parameters,sqrt(eps));
            v_out.temperate_steady = Jacobian_test_single(@temperate_steady_v3,ones(length(v_in)-3,1),parameters,100*sqrt(eps));
        else
            v_out.temperate_steady = Jacobian_test_single(@temperate_steady_v3,ones(length(v_in),1),parameters,100*sqrt(eps));
        end
    elseif isfield(parameters.flags,'local') && parameters.flags.local
        if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
            v_out.tempsol = Jacobian_test_single(@tempsol,v_in(1:3),parameters,sqrt(eps));
            v_out.temperate_steady = Jacobian_test_single(@temperate_steady_v3,v_in(4:end),parameters,100*sqrt(eps));
        else
            v_out.temperate_steady = Jacobian_test_single(@temperate_steady_v3,v_in,parameters,100*sqrt(eps));
        end
    else
        if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
            v_out.tempsol = Jacobian_test_single(@tempsol,rand(3,1),parameters,sqrt(eps));
            v_out.temperate_steady = Jacobian_test_single(@temperate_steady_v3,rand(length(v_in)-3,1),parameters,100*sqrt(eps));
        else
            v_out.temperate_steady = Jacobian_test_single(@temperate_steady_v3,rand(length(v_in),1),parameters,100*sqrt(eps));
        end
    end
    return        
end

%% run solver

%solve for temperature structure separately for inflow-type temperate
%problem
if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
    v_therm = v_in(1:3);
    if isfield(parameters,'srchparams')
        v_therm = Newton_single(@tempsol,v_therm,parameters,parameters.srchparams);
    else
        v_therm = Newton_single(@tempsol,v_therm,parameters);
    end
    v_in = v_in(4:end);
    parameters.model.z_ct = v_therm(1);
end

%run temperate ice solver
if isfield(parameters,'srchparams')
    v_temp = Newton_single(@temperate_steady_v3,v_in,parameters,parameters.srchparams);
else
    v_temp = Newton_single(@temperate_steady_v3,v_in,parameters);
end

%output
v_out.parameters = parameters;
if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
    v_out.therm = v_therm;
    v_out.temp = v_temp;
else
    v_out.therm = v_temp(1:3);
    v_out.temp = v_temp(4:end);
end


%compute fields at grid points
if nargin > 2
    v_out.grid = grid;
    [v_out.grid_T, v_out.grid_N, v_out.grid_phi, v_out.grid_q] = grid_projection(v_out,grid,parameters);
end

end