function v_out = temperate_initialize_v3(v_0,parameters)
%v_out = temperate_initialize_v3(parameters)
%computes an initial guess for use in steady_solver based on a hyperbolic
%approximation for small compaction lengths. This asymptotic solution is
%almost identical to that calculated in asymptotic_solution.m, see the
%documentation for the latter, and the two routines share many elements of
%code.
%
%The main difference between the two codes (other than the output format)
%arises for inflow problems, where ice flows from the cold subdomain into
%the temperate subdomain. asymptotic_solution.m computes a genuine
%composite solution that accounts for the inflow boundary layer formed when
%2 < alpha < 3. temperate_initialize_v3 only captures the boundary layer if
%initalizing for a set-up in steady_solver that incorporates a transformed
%segment in the temperate ice domain, that is, if
%parameters.transform.transform is set to true. In that case,
%temperate_initialize_v3 uses the same transformation as described in the
%documentation for asymptotic_solution.m for the boundary layer,
%identifying phi and Psi with the variables of the same name used in
%temperate_ode_transformed for initialization purposes, and setting chi =
%chi_tilde + Pe_phi*u_n/a. The second difference is in the treatment of the
%far field for the boundary layer. This is handled appropriately in
%asymptotic_solution.m by allowing the far field conditions to be met at an
%externally chosen value of phi = parameters.model.phi_tfm_farfield, which
%can be set to a large enough number to suppress errors. In
%temperate_initialize_v3.m, the transformed solution is instead matched
%directly to the asymptotic solution without boundary layer at the end of
%the transformed interval in the partition of the temperate subdomain,
%which can give rise to slightly odd-looking solutions if the boundary
%layer correction has not become sufficiently small at that point.
%
%Input parameters:
%   v_0:    is an optional guess for [z_ct; A; B] as used in tempsol; this
%           guess is mostly relevant if the cold-temperate boundary needs a
%           good initial guess. If [] is supplied, an initial guess is made
%           based on a low thermal Peclet number Pe and a high moisture
%           Peclet number Pe_phi
%   parameters: same parameter structure as in steady_solver
%
%Output:
%   v_out:  Initial guess for solution in format specified for v_in by
%           steady_solver documentation
%
%Dependencies: shoot.m
%
%Christian Schoof, December 2014

%% first, compute temperature parameters and free boundary location
if isempty(v_0) 
    if (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
        %make initial guess based on a small T_0*(Pe*u)^2/a approximation for the
        %obstacle problem version of the temperature problem
        v_0 = zeros(3,1);
        u_c = (parameters.model.u_0+parameters.model.u_z*parameters.model.z_c);
        v_0(1) = parameters.model.z_c + sign(parameters.model.z_t-parameters.model.z_c)*(-2*parameters.model.T_0/parameters.model.a)^(1/2);
        v_0(2) = -parameters.model.a*exp(-parameters.model.Pe*u_c*v_0(1))/(parameters.model.Pe*u_c);
        v_0(3) = -parameters.model.a/(parameters.model.Pe*u_c)*(v_0(1) - 1/(parameters.model.Pe*u_c) );
    else
        %Stefan condition version
        v_0 = zeros(3,1);
        u_c = (parameters.model.u_0+parameters.model.u_z*parameters.model.z_c);
        q_c =  parameters.model.Pe_phi*parameters.model.u_0*parameters.model.phi_0 + parameters.model.g*parameters.model.phi_0^parameters.model.alpha...
            + (parameters.model.a+parameters.model.Pe_phi*parameters.model.u_z*parameters.model.phi_0)*(parameters.model.z_c-parameters.model.z_t);
        v_0(1) = parameters.model.z_c + parameters.model.T_0/(q_c*(1+parameters.model.a*parameters.model.T_0/q_c));
        v_0(2) = q_c*exp(-parameters.model.Pe*u_c*v_0(1));
        v_0(3) =  -parameters.model.a/(parameters.model.Pe*u_c)*(v_0(1) + q_c);
    end
end
if isfield(parameters,'srchparams')
    v_temp = Newton_single(@steady_advect,v_0,parameters,parameters.srchparams);
else
    v_temp = Newton_single(@steady_advect,v_0,parameters);
end
z_ct = v_temp(1);


%% integrate hyperbolic model for phi from appropriate boundary, taking
%% account of grid ordering

shoot_grid = parameters.partition.grid;
func = @(z,v)temperate_advect(z,v,parameters.model);
if (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
    if shoot_grid(1) < shoot_grid(end)
        z_span = z_ct+shoot_grid(1:end)*(parameters.model.z_t-z_ct);
        [z_temp phi_temp] = ode45(func,z_span(end:-1:1),parameters.model.phi_0);
        phi_grid = phi_temp(end:-1:1);
    else
        z_span = z_ct+shoot_grid(end:-1:1)*(parameters.model.z_t-z_ct);
        [z_temp phi_temp] = ode45(func,z_span(end:-1:1),parameters.model.phi_0);
        phi_grid = phi_temp;
    end
else
    if shoot_grid(1) < shoot_grid(end)
        if min(shoot_grid) == 0
            z_span = z_ct+shoot_grid*(parameters.model.z_t-z_ct);
        else
            z_span = [z_ct z_ct+shoot_grid*(parameters.model.z_t-z_ct)];
        end
        [z_temp phi_temp] = ode45(func,z_span,0);
        if min(shoot_grid) == 0
            phi_grid = phi_temp;
        else
            phi_grid = phi_temp(2:end);
        end
    else
        if min(shoot_grid) == 0
            z_span = z_ct+shoot_grid(end:-1:1)*(parameters.model.z_t-z_ct);
        else
            z_span =  [z_ct z_ct+shoot_grid(end:-1:1)*(parameters.model.z_t-z_ct)];
        end
        [z_temp phi_temp] = ode45(func,z_span,0);
        if min(shoot_grid) == 0
            phi_grid = phi_temp(end:-1:1);
        else
            phi_grid = phi_temp(end:-1:2);
        end
    end
end
phi_grid = reshape(phi_grid,1,length(phi_grid)); %make phi_grid a row vector;

%% post-process: compute locations, effective pressures and net flux
z_grid = z_ct+shoot_grid*(parameters.model.z_t-z_ct);
N_grid = (parameters.model.a - parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_grid).*temperate_advect(z_grid,phi_grid,parameters.model))./phi_grid;
q_grid = parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_grid).*phi_grid + parameters.model.g*phi_grid.^parameters.model.alpha;

%% Add boundary layers
if (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
    %correct for zero Darcy flux condition at outflow boundary
    [phi_correct, N_correct] = flux_correct(z_ct,z_grid,phi_grid,q_grid,parameters);
    N_grid = N_grid+N_correct;
    phi_grid = phi_grid+phi_correct;
end

if ~(abs(parameters.model.z_t) <  100*sqrt(eps) && parameters.model.u_0 == 0)
    %effective pressure boundary condition boundary layer
    N_correct = pressure_correct(parameters.model.z_t,z_grid,phi_grid,N_grid,parameters);
    N_grid = N_grid + N_correct;
end


%% Transformed region
if isfield(parameters,'transform') && (~isfield(parameters.transform,'transform') || parameters.transform.transform) ...
        && (sign(parameters.model.z_t-parameters.model.z_c) == sign(parameters.model.u_0) || sign(parameters.model.z_t-parameters.model.z_c) == sign(parameters.model.u_z))
    if shoot_grid(1) < shoot_grid(end)
        phi_grid = phi_grid(2:end); N_grid = N_grid(2:end); q_grid = q_grid(2:end);
        z_transform = z_grid(1); phi_transform = phi_grid(1); N_transform = N_grid(1); q_transform = q_grid(1);
    else
        phi_grid = phi_grid(1:end-1); N_grid = N_grid(1:end-1); q_grid = q_grid(1:end-1);
        z_transform = z_grid(end-1); phi_transform = phi_grid(end); N_transform = N_grid(end); q_transform = q_grid(end);
    end
    v_transform = transform_initialize(z_ct,z_transform,phi_transform,N_transform,q_transform,parameters);
end


%% produce output
if isfield(parameters,'transform') && (~isfield(parameters.transform,'transform') || parameters.transform.transform) ...
        && (sign(parameters.model.z_t-parameters.model.z_c) == sign(parameters.model.u_0) || sign(parameters.model.z_t-parameters.model.z_c) == sign(parameters.model.u_z))
    v_out = zeros(3*(length(shoot_grid)-1)+4*(length(parameters.transform.grid)-1),1);
    v_out(1:3) = v_temp;
    v_out(4:3*(length(shoot_grid)-1)) =  reshape([phi_grid(1:end-1);N_grid(1:end-1);q_grid(1:end-1)],3*(length(shoot_grid)-2),1);
    v_out(3*(length(shoot_grid)-1)+1:end) = reshape(v_transform,4*(length(parameters.transform.grid)-1),1);
else
    v_out = zeros(3*length(shoot_grid),1);
    v_out(1:3) = v_temp;
    v_out(4:end) = reshape([phi_grid(1:end-1);N_grid(1:end-1);q_grid(1:end-1)],3*(length(shoot_grid)-1),1);
end

end

function [v_out Dv_out] = steady_advect(v_in,parameters)
if nargout == 1
    ftemp1 = tempsol(v_in,parameters);
    v_out = ftemp1;
else
    [ftemp1 Dftemp1] = tempsol(v_in,parameters);
    v_out = ftemp1;
    Dv_out = Dftemp1;
end
if (parameters.model.z_t-parameters.model.z_c < 0 && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) || (parameters.model.z_t-parameters.model.z_c > 0 && (parameters.model.u_0 < 0 || parameters.model.u_z < 0))
    phi_0 = parameters.model.phi_0;
    z_ct = v_in(1);
    z_span = [parameters.model.z_t z_ct];
    func = @(z,v)temperate_advect(z,v,parameters.model);
    options = odeset('NonNegative',1);
    [zsol phisol] = ode45(func,z_span,phi_0,options);
    phi_ct = phisol(end);
    ftemp2 = parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct)*phi_ct + parameters.model.g*phi_ct^parameters.model.alpha;   %enthalpy flux in positive z-direction
    v_out(3) = v_out(3) - ftemp2;
    if nargout > 1
        Dphi_ct = temperate_advect(z_ct,phi_ct,parameters.model);
        Dftemp2 = parameters.model.Pe_phi*parameters.model.u_z*phi_ct + ...
            (parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct) + parameters.model.g*parameters.model.alpha*phi_ct^(parameters.model.alpha-1))*Dphi_ct;
        Dv_out(3,1) = Dv_out(3,1) - Dftemp2;
    end
end
end

function v_out = temperate_advect(z,phi,params)
%height-dependence equation for the hyperbolic version of temperate ice in steady
%state
Pe_phi = params.Pe_phi;
g = params.g;
alpha = params.alpha;
u_0 = params.u_0;
u_z = params.u_z;
a = params.a;
v_out = a./(Pe_phi*(u_0+u_z*z) + alpha*phi.^(alpha-1)*g);
end

function N_correct = pressure_correct(z_t,z_grid,phi_grid,N_grid,parameters)
%extract parameters
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
Pe_phi = parameters.model.Pe_phi;
g = parameters.model.g;
alpha = parameters.model.alpha;
delta = parameters.model.delta;
N_0 = parameters.model.N_0;
%pre-process: set N_ct and u_ct
u_t = u_0 + u_z*z_t;
if abs(z_grid(1)-z_t) < abs(z_grid(end)-z_t)
    N_t = N_grid(1);
    phi_t = phi_grid(1);
else
    N_t = N_grid(end);
    phi_t = phi_grid(end);
end
lambda = sqrt((Pe_phi*u_t+alpha*g*phi_t^(alpha-1))/(delta*Pe_phi*u_t*phi_t^(alpha-1)));
N_correct = (N_0-N_t)*exp(-lambda*abs(z_grid-z_t));
end

function [phi_correct N_correct] = flux_correct(z_ct,z_grid,phi_grid,q_grid,parameters)
%extract parameters   
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
Pe_phi = parameters.model.Pe_phi;
g = parameters.model.g;
alpha = parameters.model.alpha;
delta = parameters.model.delta;
%pre-process: set q_ct and u_ct
parameters.model.u_ct = u_0 + u_z*z_ct;
if abs(z_grid(1)-z_ct) < abs(z_grid(end)-z_ct)
    parameters.model.q_ct = q_grid(1);
    parameters.model.phi_ct = phi_grid(1);
else
    parameters.model.q_ct = q_grid(end);
    parameters.model.phi_ct = phi_grid(end);
end
%initial integration to find distance offset
%suitable initial condition on unstable manifold of saddle point
u_ct = parameters.model.u_ct; phi_ct = parameters.model.phi_ct; q_ct = parameters.model.q_ct;
if sign(Pe_phi*u_ct + alpha*g*phi_ct^(alpha-1)) ~= sign(Pe_phi*u_ct), error('Wrong Hamiltonian fixed point'), end
lambda = sqrt((Pe_phi*u_ct + alpha*g*phi_ct^(alpha-1))/(delta*phi_ct^(alpha-1)*Pe_phi*u_ct));
v_0 = [phi_ct; 0];
v_prime = (q_ct/(Pe_phi*u_ct)-phi_ct)/10*[1; -Pe_phi*abs(u_ct)*lambda/phi_ct];
options = odeset('Events',@(z,v)Darcy_zero(z,v,parameters));
z_span = [0 2*(z_ct-parameters.model.z_t)];
z_f = [];
while isempty(z_f)
    [z_out,v_out,z_f,v_f,i_f] = ode45(@(z,v)Hamiltonian(z,v,parameters),z_span,v_0+v_prime,options);
    if isempty(z_f)
        z_out = z_out(abs(z_out)<abs(z_span(end)/2));
        v_prime = v_out(length(z_out),:).'-v_0;    %shift v_0 as close to half way along the integration as possible, then repeat
    elseif abs(z_f)<abs(z_span(end)/2)  %should only be relevant if first computation finds z_f in first half of integration
        %if abs(v_prime(2)) > sqrt(eps)*abs(q_ct/(Pe_phi*u_ct)-phi_ct) %limit how close to the fixed point we allow the initial conditions to come, here within a fractional distance in N of sqrt(eps
        if abs(v_prime(2)) > sqrt(eps)*abs((q_ct/phi_ct-Pe_phi*u_ct)*lambda)  %equivalent for fractional distance in N of sqrt(eps)
            v_prime = v_prime/10;
            z_f = [];
        else
            warning('truncated calculation of far field of flux boundary layer')
        end
    end
end
%z_f signifies position of zero Darcy flux boundary, integrate system
%forward to grid points relative to z_f
z_grid_prime = z_grid(z_ct-z_grid > min(0,z_f) & z_ct - z_grid < max(0,z_f));
if  sign(z_grid_prime(end)-z_grid_prime(1)) == sign(z_f)
    z_span = [0 z_grid_prime+z_f-z_grid_prime(end)];
else
    z_span = [0 z_grid_prime(end:-1:1)+z_f-z_grid_prime(1)];
end
if z_span(2) == z_span(1), z_span(1) = z_span(1)-eps; end
options = odeset('RelTol',1e-12,'AbsTol',1e-12);
[z_out, v_out] = ode45(@(z,v)Hamiltonian(z,v,parameters),z_span,v_0+v_prime,options);
phi_correct = zeros(size(z_grid));
N_correct = zeros(size(z_grid));
if sign(z_grid(end)-z_grid(1)) == sign(z_f)
    phi_correct(z_ct-z_grid > min(0,z_f) & z_ct-z_grid < max(0,z_f)) = v_out(2:end,1).'-phi_ct;
    N_correct(z_ct-z_grid > min(0,z_f) & z_ct-z_grid < max(0,z_f)) = v_out(2:end,2).';
else
     phi_correct(z_ct-z_grid > min(0,z_f) & z_ct-z_grid < max(0,z_f)) = v_out(end:-1:2,1).'-phi_ct;
     N_correct(z_ct-z_grid > min(0,z_f) & z_ct-z_grid < max(0,z_f)) = v_out(end:-1:2,2).';   
end
end


function v_out = transform_initialize(z_ct,z,phi,N,q,parameters)

%extract parameters
alpha = parameters.model.alpha;
a = parameters.model.a;
u_0 = parameters.model.u_0;
u_z = parameters.model.u_z;
g = parameters.model.g;
Pe_phi = parameters.model.Pe_phi;
z_c = parameters.model.z_c;
z_t = parameters.model.z_t;

%compute velocity (normal, into domain)
u = sign(z_t-z_c)*(u_0+u_z*z_ct);
g = sign(z_t-z_c)*g;

%zeta positions in grid
grid = parameters.transform.grid;
grid = grid-max(grid);
grid = grid(1:end-1);
grid = reshape(grid,1,length(grid));

%initialize output
v_out = zeros(4,length(grid));

%emulate shape and zeta-dependence of orbits on relevant invariant manifolds to match
%imposed z, phi, N, q at position z
if alpha > 3
    C = phi^((3-alpha)/2);
    v_out(1,:) = 1./(C-(alpha-3)*a/(2*Pe_phi*u)*grid).^(2/(alpha-3));
    v_out(2,:) = alpha*g*a/(Pe_phi*u)*v_out(1,:).^((3*alpha-5)/2);
    v_out(3,:) = Pe_phi^2*u*u_z/(2*a)*v_out(1,:);
    v_out(4,:) = Pe_phi*u/a +Pe_phi^2*u*u_z/(2*a^2)*v_out(1,:) +  (g/a)*v_out(1,:).^(alpha-1);
elseif alpha == 3
    C = phi;
    v_out(1,:) = C*exp(a/(Pe_phi*u)*grid);
    v_out(2,:) = (3*a*g/(Pe_phi*u) + Pe_phi^2*u*u_z^2/a)*v_out(1,:).^2;
    v_out(3,:) = Pe_phi^2*u*u_z/(2*a)*v_out(1,:) + Pe_phi^3*u*u_z^2/(3*a^2)*v_out(1,:).^2;
    v_out(4,:) = Pe_phi*u/a +Pe_phi^2*u*u_z/(2*a^2)*v_out(1,:) + (g/a + 2*Pe_phi^3*u*u_z^2/(3*a^2))*v_out(1,:).^2;
elseif alpha >= 2
    parameters.model.phi_tfm = phi; parameters.model.Psi_tfm = phi^(alpha-2)*N;
    v_0_temp(1,:) = phi*exp(a*grid/(Pe_phi*u));
    v_0_temp(2,:) = alpha*g*a/(Pe_phi*u)*v_0_temp(1,:).^(2*alpha-4);
    v_0_temp(3,:) = g/a*v_0_temp(1,:).^(alpha-1);
    v_0 = reshape(v_0_temp,3*length(grid),1);
    v_temp = Newton_single(@pressure_transformed_shoot,v_0,parameters);
    v_temp = reshape(v_temp,3,length(grid));
    v_out(1,:) = v_temp(1,:)';
    v_out(2,:) = v_temp(2,:)';
    v_out(3,:) = Pe_phi^2*u*u_z/(2*a)*v_out(1,:);
    v_out(4,:) = Pe_phi*u/a +Pe_phi^2*u*u_z/(2*a^2)*v_out(1,:) + v_temp(3,:);
else
    error('Current code not set up for alpha < 2')
end

end

function [f_out, Df_out] = pressure_transformed_shoot(v_in,parameters)
grid = parameters.transform.grid;
n_pts = length(grid);
phi_0 = parameters.model.phi_tfm;
Psi_0 = parameters.model.Psi_tfm;

parameters.functions.evolve = @pressure_transformed;
ftemp = zeros(3,n_pts-1);
if nargout > 1
    Dftemp = zeros(3,3,n_pts-1);
end
for ii = 1:n_pts-1
    parameters.t_span = [grid(ii) grid(ii+1)]; 
    v = v_in((ii-1)*3+1:ii*3);
    if nargout == 1
        ftemp(:,ii) = shoot(v,parameters);
    else
        [ftemp(:,ii), Dftemp(:,:,ii)] = shoot(v,parameters);
    end
end 

f_out = zeros(size(v_in));
if nargout > 1
    Df_out = zeros(length(v_in));
end

if grid(1) > grid(end)
    f_out(1) = v_in(1) - phi_0;
    f_out(2) = v_in(2) - Psi_0;
    f_out(3) = ftemp(3,end)/ftemp(1,end);
    if nargout > 1
        Df_out(1,1) = 1; Df_out(2,2) = 1;
        Df_out(3,3*(n_pts-2)+(1:3)) = Dftemp(3,:,end)/ftemp(1,end) - ftemp(3,end)/ftemp(1,end)^2*Dftemp(1,:,end);
    end
else
    f_out(1) = ftemp(1,end) - phi_0;
    f_out(2) = ftemp(2,end) - Psi_0;
    f_out(3) = v_in(3)/v_in(1);
    if nargout > 1
        Df_out(1,3*(n_pts-2)+(1:3)) = Dftemp(1,:,end);
        Df_out(2,3*(n_pts-2)+(1:3)) = Dftemp(2,:,end);
        Df_out(3,3) = 1/v_in(1); Df_out(3,1) = -v_in(3)/v_in(1)^2;
    end
end
for ii = 2:n_pts-1
    f_out((ii-1)*3+(1:3)) = ftemp(:,ii-1) - v_in((ii-1)*3+(1:3));
    if nargout > 1
        Df_out((ii-1)*3+(1:3),(ii-2)*3+(1:3)) = Dftemp(:,:,ii-1);
        Df_out((ii-1)*3+(1:3),(ii-1)*3+(1:3)) = -eye(3);                   
    end
end  
end



function v_out = Hamiltonian(z,v_in,parameters)
%parameters
q_ct = parameters.model.q_ct;
u_ct = parameters.model.u_ct;
Pe_phi = parameters.model.Pe_phi;
g = parameters.model.g;
alpha = parameters.model.alpha;
delta = parameters.model.delta;
%dependent variables
phi = v_in(1);
N = v_in(2);
%rates of change
v_out = zeros(2,1);
v_out(1) = -phi*N/(Pe_phi*u_ct);
v_out(2) = ( q_ct/phi^alpha - Pe_phi*u_ct/phi^(alpha-1) - g ) / delta;
end

function [v_out, isterminal, direction] = Darcy_zero(z,v_in,parameters)
q_ct = parameters.model.q_ct;
u_ct = parameters.model.u_ct;
Pe_phi = parameters.model.Pe_phi;
phi = v_in(1);    
v_out = q_ct-Pe_phi*u_ct*phi;
isterminal = true;
direction = 0;
end

function v_out = pressure_transformed(z,v_in,n,parameters)
Pe_phi = parameters.Pe_phi;
g = sign(parameters.z_t-parameters.z_c)*parameters.g;
alpha = parameters.alpha;
delta = parameters.delta;
u = sign(parameters.z_t-parameters.z_c)*(parameters.u_0 + parameters.u_z*parameters.z_ct);
a = parameters.a;

phi = v_in(1);
Psi = v_in(2);
chi_tilde = v_in(3);

switch n
    case 0
        v_out(1) = a/(Pe_phi*u)*phi;
        v_out(2) = a*chi_tilde/delta - g*phi^(alpha-1)/delta + (alpha-2)*a/(Pe_phi*u)*Psi;
        v_out(3) = -a/(Pe_phi*u)*chi_tilde + 1/a*phi^(3-alpha)*Psi;
    case 1
        v_out = zeros(3,3);
        v_out(1,1) = a/(Pe_phi*u);
        v_out(2,1) = -(alpha-1)*g*phi^(alpha-2)/delta;
        v_out(2,3) = a/delta;
        v_out(2,2) = (alpha-2)*a/(Pe_phi*u);
        v_out(3,1) = (3-alpha)/a*phi^(2-alpha)*Psi;
        v_out(3,3) = -a/(Pe_phi*u);
        v_out(3,2) =  1/a*phi^(3-alpha);
    case 2
        v_out = zeros(3,3,3);
        v_out(2,1,1) = -(alpha-1)*(alpha-2)*g*phi^(alpha-3)/delta;
        v_out(3,1,1) = (3-alpha)*(2-alpha)/a*phi^(1-alpha)*Psi;
        v_out(3,1,2) = (3-alpha)/a*phi^(2-alpha);
        v_out(3,2,1) = v_out(3,1,2);
end
end