function fout = temperate_ode(z,v,n,params)
%fout = temperate_ode_transformed(zeta,v,n,params)
%Computes right-hand sides of one-dimensional steady temperate ice equations
%   dphi/dz = (a - phi*N)/(Pe_phi*u)
%   dN/dz = (q - Pe_phi*u*phi)/(delta*phi^alpha) - g/delta
%   dq/dz = a + Pe_phi*du/dz*phi
%
%The function temperate_ode is intended for use with o.d.e.
%solvers, and in particular, is compatible with the syntax of the shooting
%method solver shoot.m
%
%Input arguments:
%   z:   current value of z
%   v:      3-by-1 column vector [phi; N; q]
%   n:      optional differentiation level, see output below. Defaults to
%           zero
%   params: optional parameter structure containing the optional fields
%       Pe_phi: porosity Peclet number(defaults to 1)
%       g:      signed gravity (defaults to -1)
%       a:      dissipation rate (defaults to 1)
%       delta:  compaction length scale parameter (defaults to 1)
%       alpha:  permeability exponent (defaults to 3)
%       u_0:    constant part of velocity field (defaults to 1)
%       u_z:    strain rate (defaults to 0)
%
%Output:
%   fout:   depends on differentiation level set
%       if n = 0, fout gives f =
%           [dphi/dz;dN/dz;dq/dz]
%       if n = 1, fout gives Jacobian matrix of f in the previous line with
%           respect to transpose of input vector v'
%       if n = 2, fout gives second derivative array of f with respect to
%       v'
%
%Christian Schoof, January 2015

%check input
if nargin < 2, error('position and state vector must be specified'), end

if size(v) ~= [3 1], error('state vector must be 3-by-1'), end


%set differentiation level if necessary
if nargin < 3
    n = 0;
end

%extract parameters
if nargin < 4 || ~isfield(params,'Pe_phi')
    Pe_phi = 1;
else
    Pe_phi = params.Pe_phi;
end
if nargin < 4 || ~isfield(params,'g')
    g = -1;
else
    g = params.g;
end
if nargin < 4 || ~isfield(params,'delta')
    delta = 1;
else
    delta = params.delta;
end
if nargin < 4 || ~isfield(params,'alpha')
    alpha = 3;
else
    alpha = params.alpha;
end
if nargin < 4 || ~isfield(params,'u_0')
    u_0 = 1;
else
    u_0 = params.u_0;
end
if nargin < 4 || ~isfield(params,'u_z')
    u_z = 0;
else
    u_z = params.u_z;
end
if nargin < 4 || ~isfield(params,'a')
    a = 1;
else
    a = params.a;
end

phi = v(1);
N = v(2);
q = v(3);
u = u_0 + u_z*z;

switch n
    case 0
        fout = zeros(3,1);
        fout(1) = (a-phi*N)/(Pe_phi*u);
        fout(2) = (q/phi^alpha - Pe_phi*u/phi^(alpha-1) - g)/delta;
        fout(3) = a + Pe_phi*u_z*phi;
    case 1
        fout = zeros(3,3);
        fout(1,1) = -N/(Pe_phi*u); fout(1,2) = -phi/(Pe_phi*u); fout(1,3) = 0;
        fout(2,1) = (-alpha*q/phi^(alpha+1) + (alpha-1)*Pe_phi*u/phi^alpha)/delta; fout(2,2) = 0; fout(2,3) = 1/(phi^alpha*delta);
        fout(3,1) = Pe_phi*u_z; fout(3,2) = 0; fout(3,3) = 0;
    case 2
        fout = zeros(3,3,3);
        fout(1,1,2) = -1/(Pe_phi*u); fout(1,2,1) = fout(1,1,2);
        fout(2,1,1) =  (alpha*(alpha+1)*q/phi^(alpha+2) - (alpha-1)*alpha*Pe_phi*u/phi^(alpha+1))/delta; fout(2,1,3) = -alpha/(phi^(alpha+1)*delta); fout(2,3,1) = fout(2,1,3);
end

end

