function fout = temperate_ode_transformed(zeta,v,n,params)
%fout = temperate_ode_transformed(zeta,v,n,params)
%Transformed version of the one-dimensional steady temperate ice equations
%   dphi/dz = (a - phi*N)/(Pe_phi*u)
%   dN/dz = (q - Pe_phi*u*phi)/(delta*phi^alpha) - g/delta
%   dq/dz = a + Pe_phi*du/dz*phi
%The transformation is intended to deal with the singularity in these
%equations that occurs when phi -> 0, as it must near an inflow boundary
%
%Let n = (z-z_ct)*sign(z_t-z_c), where z_t and z_c are temperate and cold
%boundaries in the original problem and z_ct (with min(z_t,z_c) < z_ct <
%max(z_t,z_ct)) is the location of the cold-temperate boundary on which phi
%= 0. Define u_n = u(z)*sign(z_t-z_c), g_n = g*sign(z_t-z_t), q_n = q*sign(z_t-z_c)
%
%The form of the transformation depends on the exponent alpha.
%For 2 <= alpha < = 3, define a new independent variable zeta through
%   dn/dzeta = phi
%and let
%   chi = n/phi,
%   Psi = phi^(alpha-2)N
%   omega = (q_n-a*n)/phi
%to obtain the dynamical system
%   dphi/dzeta = a/Pe_phi*u_n)*phu - 1/(Pe_phi*u_n)*phi^(4-alpha)*Psi
%   dPsi/dzeta = a/delta*chi +1/delta*omega -Pe_phi*u_n/delta -
%           g/delta*phi^alpha-1) - (alpha-2)*a/(Pe_phi*u_n)*Psi
%   dchi/dzeta = 1 - a/(Pe_phi*u_n)*chi +
%           1/(Peh_hi*u_n)*phi^(3-alpha)*chi*Psi
%   domega/dzeta = Pe_phi*u_n'*phi - a/(Pe_phi*u_n)*omega +
%           1/(Pe_phi*u_n)*phi^(3-alpha*Psi*omega)
%where u_n' = du_n/dn. This dynamical system has a fixed point at
%(0,0,Pe_phi u_n/a,0) except for alpha = 2, when
%
%For 3 <= alpha, we have the following (which agrees with the above at
%alpha = 3)
%   dn/dzeta = phi^((alpha-1)/2)
%and
%   chi = n/phi,
%   Psi = phi^((alpha-1)/2)N
%   omega = (q_n-a*n)/phi
%leading to
%   dphi/dzeta = a/)Pe_phi*u_n)*phi^((alpha-1)/2) - 1/(Pe_phi*u_n)*phi*Psi
%   dPsi/dzeta = a/delta*chi + 1/de;ta*omega - Pe_phi*u_n/delta -
%       g/delta*phi^(alpha-1) - (alpha-1)/(2*Pe_phi*u_n)*si^2 +
%       (alpha-1)*a/(2*Pe_phi*u_n)*Phi^((alpha-3)/2)*Psi
%   dchi/dzeta = phi^((alpha-3)/2)*(1-a/(Pe_phi*u_n)*chi) +
%       1/(Pe_phi*u)n)*chi*Psi
%   domega/dzeta = Pe_phi*u_n'*phi^((alpha-1)/2) -
%       a/(Pe_phi*u_n)*phi^((alpha-3)/2)*omega + 1/(Pe_phi*u_n)*Psi*omega
%which likewise has a fixed point at (0,0,Pe_phi u_n/a,0) when alpha > 2;
%for alpha = 2, it has a one parameter-family of fixed points at
%(0,Psi_0,Pe_phi u_n/a,0) with Psi_0 arbitrary.
%
%The transformed equations turn the boundary at a finite position z into a
%fixed point of a dynamical system, approached as a transformed variable
%zeta -> -infinity. The local behaviour of orbits near that fixed point can
%be determined by from the invariant manifolds of the fixed point, which is
%used in temperate_steady_v3 to determine the form of solutions. In
%particular, to ensure that the orbit goes into the fixed point, it must
%lie on a stable or a stable centre manifold. For the transformed dynamical
%systems given below, this translates into the transformed variables
%(phi,Psi,omega,chi) being confined to a two-dimensional surface as zeta ->
%-infinity. temperate_steady_v3 imposes these constraints at the most
%negative value of zeta that the dynmical system is integrated to by
%ensuring that chi and omega are given by the linear approximation to that
%surface. For alpha >= 2, there are in principle four distinct cases with
%different flavours of fixed point and invariant manifolds, namely alpha =
%2 (one-parameter family of fixed points with a one-dimensional stable
%manifold), 2 < alpha < 3 (a single fixed point with a two-dimensional
%stable manifold), alpha = 3 (a single fixed point with one-dimensional
%stable and stable centre manifolds), alpha > 3 (again a single fixed point
%with one-dimensional stable and stable centre manifolds, but with
%different properties from the alpha = 3 case). However, in each case the
%linear approximation to the form of omega and chi on the relevant
%two-dimesional surface is the same, of the form
%   omega ~ Pe_phi^2*u_n*u_n'/(2*a)*phi + o(phi);
%   (chi - Pe_phi*u_n/a) ~ Pe_phi^2*u_n*u_n'/(2*a^2)*phi + o(phi);
%These are implemented in temperate_steady_v3 in the form
%   omega/phi - Pe_phi^2*u_n*u_n'/(2*a)*phi = 0
%   (chi - Pe_phi*u_n/a)/phi - Pe_phi^2*u_n*u_n'/(2*a^2)
%at the point with the most negative value of zeta in the integration of
%the transformed dynamical system. Because the limit zeta -> infinity
%cannot be attained numerically, it is therefore key that this value of
%zeta is sufficiently large; values that are too large will however make
%the shooting method intractably stiff (partly on account of the division
%by phi above, which is intended to be a more rigorous constraint on omega
%and chi than for instance
%   omega - Pe_phi^2*u_n*u_n'/(2*a)*phi = 0
%   chi - Pe_phi*u_n/a -Pe_phi^2*u_n*u_n'/(2*a^2)*phi = 0
%would be; the latter would simply turn into omega  ~ 0, chi ~ Pe_phi*u_n/a
%in the limit, as phi -> 0
%
%The function temperate_ode_transformed is intended for use with o.d.e.
%solvers, and in particular, is compatible with the syntax of the shooting
%method solver shoot.m
%
%Input arguments:
%   zeta:   current value of zeta
%   v:      4-by-1 column vector [phi; Psi; omega; chi]; note that omega
%           and chi are interchanged from the order in which they are given above
%   n:      optional differentiation level, see output below. Defaults to
%           zero
%   params: optional parameter structure containing the optional fields
%       Pe_phi: porosity Peclet number(defaults to 1)
%       g:      signed gravity (defaults to -1)
%       a:      dissipation rate (defaults to 1)
%       delta:  compaction length scale parameter (defaults to 1)
%       alpha:  permeability exponent (defaults to 3)
%       u_0:    constant part of velocity field (defaults to 1)
%       u_z:    strain rate (defaults to 0)
%       z_ct:   cold-temperate boundary position for evaluating velocity
%               field (defaults to 1, but will generate warning if not set)
%       direction: sign(z_t-z_c), giving direction into temperate subdomain
%               from cold-temperate boundary (defaults to -1, but will
%               generate warning if not set)
%
%Output:
%   fout:   depends on differentiation level set
%       if n = 0, fout gives f =
%           [dphi/dzeta;dPsi/dzeta;domega/dzeta;dchi/dzeta]
%       if n = 1, fout gives Jacobian matrix of f in the previous line with
%           respect to transpose of input vector v'
%       if n = 2, fout gives second derivative array of f with respect to
%       v'
%
%Christian Schoof, January 2015

%check input
if nargin < 2, error('position and state vector must be specified'), end

if size(v)~=[4 1], error('state vector for the transformed system must be 4-by-1'), end


%set differentiation level if necessary
if nargin < 3
    n = 0;
end

%extract parameters
if nargin < 4 || ~isfield(params,'Pe_phi')
    Pe_phi = 1;
else
    Pe_phi = params.Pe_phi;
end
if nargin < 4 || ~isfield(params,'g')
    g = -1;
else
    g = params.g;
end
if nargin < 4 || ~isfield(params,'delta')
    delta = 1;
else
    delta = params.delta;
end
if nargin < 4 || ~isfield(params,'alpha')
    alpha = 3;
else
    alpha = params.alpha;
end
if alpha <= 1, error('transformation assumes alpha > 1'), end
if nargin < 4 || ~isfield(params,'u_0')
    u_0 = 1;
else
    u_0 = params.u_0;
end
if nargin < 4 || ~isfield(params,'u_z')
    u_z = 0;
else
    u_z = params.u_z;
end
if nargin < 4 || ~isfield(params,'a')
    a = 1;
else
    a = params.a;
end
if nargin < 4 || ~isfield(params,'z_ct')
    z_ct = 1;
    if nargin > 3, warning('interface elevation not set as input'), end
else
    z_ct = params.z_ct;
end
if nargin < 4 || ~isfield(params,'direction')
    direction = -1;
    if nargin > 3, warning('direction not set as input'), end
else
    direction = params.direction;
end


%extract variables
phi = v(1);
Psi = v(2);
omega = v(3);
chi = v(4);


%define coefficients for test calculation
if n == -1
    if alpha < 3
        nu = 1;
        beta = alpha-2;
        gamma = 1;
    else
        nu = 1;
        beta = (alpha-1)/2;
        gamma = beta;
    end
end
    

%compute position
z = direction*phi*chi + z_ct;
%compute velocity
u = u_0 + u_z*z;

%transform for the case of direction = -1; CHECK THAT THIS WORKS! (u_z
%doesn't change sign, which is encouraging)
if direction < 0
    u = -u; g = -g;
end


switch n
    case -1 %test specific implemented exponent choices
        fout = zeros(4,1);
        fout(1) = a*phi^gamma/(Pe_phi*u) - phi^(1+gamma-beta)*Psi/(Pe_phi*u);
        fout(2) = a*phi^(beta+gamma+nu-alpha)*chi/delta + omega*phi^(1+beta+gamma-alpha)/delta - u*phi^(1+beta+gamma-alpha)/delta ...
            - g*phi^(beta+gamma)/delta - beta*phi^(gamma-beta)*Psi^2/(Pe_phi*u) + beta*a*phi^(gamma-1)*Psi/(Pe_phi*u);
        fout(3) = phi^gamma*Pe_phi*u_z - a*phi^(gamma-1)*omega/(Pe_phi*u) + phi^(gamma-beta)*Psi*omega/(Pe_phi*u);
        fout(4) = phi^(gamma-nu) - nu*a*phi^(gamma-1)*chi/(Pe_phi*u) + nu*phi^(gamma-beta)*chi*Psi/(Pe_phi*u);
        if alpha < 3
            fout(1) = a*phi/(Pe_phi*u) - phi^(4-alpha)*Psi/(Pe_phi*u) - fout(1);
            fout(2) = a*chi/delta + omega/delta - Pe_phi*u/delta - g*phi^(alpha-1)/delta ...
                -(alpha-2)*phi^(3-alpha)*Psi^2/(Pe_phi*u) + (alpha-2)*a*Psi/(Pe_phi*u) - fout(2);
            fout(3) = Pe_phi*u_z*phi - a*omega/(Pe_phi*u) + phi^(3-alpha)*Psi*omega/(Pe_phi*u) - fout(3);
            fout(4) = 1 - a*chi/(Pe_phi*u) + phi^(3-alpha)*chi*Psi/(Pe_phi*u) - fout(4);
        else
            fout(1) = a*phi^((alpha-1)/2)/(Pe_phi*u) - phi*Psi/(Pe_phi*u) - fout(1);
            fout(2) = a*chi/delta + omega/delta - Pe_phi*u/delta - g*phi^(alpha-1)/delta ...
                - (alpha-1)/(2*Pe_phi*u)*Psi^2 + (alpha-1)*a/(2*Pe_phi*u)*phi^((alpha-3)/2)*Psi - fout(2);
            fout(3) = Pe_phi*u_z*phi^((alpha-1)/2) - a*phi^((alpha-3)/2)*omega/(Pe_phi*u) + Psi*omega/(Pe_phi*u) - fout(3);
            fout(4) = phi^((alpha-3)/2)*(1-a*chi/(Pe_phi*u)) + chi*Psi/(Pe_phi*u) - fout(4);
        end
        disp(fout)
    case 0
        fout = zeros(4,1);
        if alpha < 3
            fout(1) = a*phi/(Pe_phi*u) - phi^(4-alpha)*Psi/(Pe_phi*u);
            fout(2) = a*chi/delta +omega/delta - Pe_phi*u/delta - g*phi^(alpha-1)/delta ...
                -(alpha-2)*phi^(3-alpha)*Psi^2/(Pe_phi*u) + (alpha-2)*a*Psi/(Pe_phi*u);
            fout(3) = Pe_phi*u_z*phi - a*omega/(Pe_phi*u) + phi^(3-alpha)*Psi*omega/(Pe_phi*u);
            fout(4) = 1 - a*chi/(Pe_phi*u) + phi^(3-alpha)*chi*Psi/(Pe_phi*u);            
        else
            fout(1) = a*phi^((alpha-1)/2)/(Pe_phi*u) - phi*Psi/(Pe_phi*u);
            fout(2) = a*chi/delta + omega/delta - Pe_phi*u/delta - g*phi^(alpha-1)/delta ...
                - (alpha-1)/(2*Pe_phi*u)*Psi^2 + (alpha-1)*a/(2*Pe_phi*u)*phi^((alpha-3)/2)*Psi;
            fout(3) = Pe_phi*u_z*phi^((alpha-1)/2) - a*phi^((alpha-3)/2)*omega/(Pe_phi*u) + Psi*omega/(Pe_phi*u);
            fout(4) = phi^((alpha-3)/2)*(1-a*chi/(Pe_phi*u)) + chi*Psi/(Pe_phi*u);            
        end
    case 1
        fout = zeros(4,4);
        if alpha < 3
            fout(1,1) = a/(Pe_phi*u) - (4-alpha)*phi^(3-alpha)*Psi/(Pe_phi*u);
            fout(1,2) = -phi^(4-alpha)/(Pe_phi*u);
            fout(2,1) = -g/delta*(alpha-1)*phi^(alpha-2)-(alpha-2)*(3-alpha)*phi^(2-alpha)*Psi^2/(Pe_phi*u);
            fout(2,2) = -2*(alpha-2)*phi^(3-alpha)*Psi/(Pe_phi*u) + (alpha-2)*a/(Pe_phi*u);
            fout(2,3) = 1/delta;
            fout(2,4) = a/delta;
            fout(3,1) = Pe_phi*u_z + (3-alpha)*phi^(2-alpha)*Psi*omega/(Pe_phi*u);
            fout(3,2) = phi^(3-alpha)*omega/(Pe_phi*u);
            fout(3,3) = -a/(Pe_phi*u) + phi^(3-alpha)*Psi/(Pe_phi*u);
            fout(4,1) = (3-alpha)*phi^(2-alpha)*chi*Psi/(Pe_phi*u);
            fout(4,2) = phi^(3-alpha)*chi/(Pe_phi*u);
            fout(4,4) = -a/(Pe_phi*u) + phi^(3-alpha)*Psi/(Pe_phi*u);
            if u_z ~= 0
                fout(1,1) = fout(1,1) - (a*phi - phi^(4-alpha)*Psi)/(Pe_phi*u^2)*u_z*chi;
                fout(1,4) = fout(1,4) - (a*phi - phi^(4-alpha)*Psi)/(Pe_phi*u^2)*u_z*phi;
                fout(2,1) = fout(2,1) - Pe_phi*u_z*chi/delta - ( -(alpha-2)*phi^(3-alpha)*Psi^2 + (alpha-2)*a*Psi)/(Pe_phi*u^2)*u_z*chi;
                fout(2,4) = fout(2,4) - Pe_phi*u_z*phi/delta - ( -(alpha-2)*phi^(3-alpha)*Psi^2 + (alpha-2)*a*Psi)/(Pe_phi*u^2)*u_z*phi;
                fout(3,1) = fout(3,1) - (- a*omega + phi^(3-alpha)*Psi*omega)/(Pe_phi*u^2)*u_z*chi;
                fout(3,4) = fout(3,4) - (- a*omega + phi^(3-alpha)*Psi*omega)/(Pe_phi*u^2)*u_z*phi;
                fout(4,1) = fout(4,1) - (- a*chi + phi^(3-alpha)*chi*Psi)/(Pe_phi*u^2)*u_z*chi;
                fout(4,4) = fout(4,4) - (- a*chi + phi^(3-alpha)*chi*Psi)/(Pe_phi*u^2)*u_z*phi;
            end
        else
            fout(1,1) = (alpha-1)*a*phi^((alpha-3)/2)/(2*Pe_phi*u) - Psi/(Pe_phi*u);
            fout(1,2) = - phi/(Pe_phi*u);
            fout(2,1) = -g*(alpha-1)*phi^(alpha-2)/delta ...
                + (alpha-1)*(alpha-3)*a*phi^((alpha-5)/2)*Psi/(4*Pe_phi*u);
            fout(2,2) = -(alpha-1)*Psi/(Pe_phi*u) + (alpha-1)*a*phi^((alpha-3)/2)/(2*Pe_phi*u);
            fout(2,3) = 1/delta;
            fout(2,4) = a/delta;
            fout(3,1) = (alpha-1)*Pe_phi*u_z*phi^((alpha-3)/2)/2 ...
                - (alpha-3)*a*phi^((alpha-5)/2)*omega/(2*Pe_phi*u);
            fout(3,2) = omega/(Pe_phi*u);
            fout(3,3) = -a*phi^((alpha-3)/2)/(Pe_phi*u)+ Psi/(Pe_phi*u);
            fout(4,1) = (alpha-3)*phi^((alpha-5)/2)*(1-a*chi/(Pe_phi*u))/2;
            fout(4,2) = chi/(Pe_phi*u);
            fout(4,4) = -a*phi^((alpha-3)/2)/(Pe_phi*u) + Psi/(Pe_phi*u);
            if u_z ~= 0
                fout(1,1) = fout(1,1) - (a*phi^((alpha-1)/2) - phi*Psi)/(Pe_phi*u^2)*u_z*chi;
                fout(1,4) = fout(1,4) - (a*phi^((alpha-1)/2) - phi*Psi)/(Pe_phi*u^2)*u_z*phi; 
                fout(2,1) = fout(2,1) - Pe_phi*u_z*chi/delta - ( -(alpha-1)*Psi^2 + (alpha-1)*a*phi^((alpha-3)/2)*Psi)/(2*Pe_phi*u^2)*u_z*chi;
                fout(2,4) = fout(2,4) - Pe_phi*u_z*phi/delta - ( -(alpha-1)*Psi^2 + (alpha-1)*a*phi^((alpha-3)/2)*Psi)/(2*Pe_phi*u^2)*u_z*phi;
                fout(3,1) = fout(3,1) - ( - a*phi^((alpha-3)/2)*omega + Psi*omega)/(Pe_phi*u^2)*u_z*chi;
                fout(3,4) = fout(3,4) - ( - a*phi^((alpha-3)/2)*omega + Psi*omega)/(Pe_phi*u^2)*u_z*phi;
                fout(4,1) = fout(4,1) - ( -a*phi^((alpha-3)/2)*chi + chi*Psi)/(Pe_phi*u^2)*u_z*chi; 
                fout(4,4) = fout(4,4) - ( -a*phi^((alpha-3)/2)*chi + chi*Psi)/(Pe_phi*u^2)*u_z*phi;       
            end
        end
    case 2
        fout = zeros(4,4,4);
        if alpha < 3
            fout(1,1,1) = -(4-alpha)*(3-alpha)*phi^(2-alpha)*Psi/(Pe_phi*u);
            fout(1,1,2) = - (4-alpha)*phi^(3-alpha)/(Pe_phi*u);
            fout(1,2,1) = fout(1,1,2);
            fout(2,1,1) = -g/delta*(alpha-1)*(alpha-2)*phi^(alpha-3) ...
                +(alpha-2)^2*(3-alpha)*phi^(1-alpha)*Psi^2/(Pe_phi*u);
            fout(2,1,2) = -2*(alpha-2)*(3-alpha)*phi^(2-alpha)*Psi/(Pe_phi*u);
            fout(2,2,1) = fout(2,1,2);
            fout(2,2,2) =  -2*(alpha-2)*phi^(3-alpha)/(Pe_phi*u);
            fout(3,1,1) = (3-alpha)*(2-alpha)*phi^(1-alpha)*Psi*omega/(Pe_phi*u);
            fout(3,1,2) = (3-alpha)*phi^(2-alpha)*omega/(Pe_phi*u);
            fout(3,2,1) = fout(3,1,2);
            fout(3,1,3) = (3-alpha)*phi^(2-alpha)*Psi/(Pe_phi*u);
            fout(3,3,1) = fout(3,1,3);
            fout(3,2,3) = phi^(3-alpha)/(Pe_phi*u);
            fout(3,3,2) = fout(3,2,3);
            fout(4,1,1) = (3-alpha)*(2-alpha)*phi^(1-alpha)*chi*Psi/(Pe_phi*u);
            fout(4,1,2) = (3-alpha)*phi^(2-alpha)*chi/(Pe_phi*u);
            fout(4,2,1) = fout(4,1,2);
            fout(4,4,1) = (3-alpha)*phi^(2-alpha)*Psi/(Pe_phi*u);
            fout(4,1,4) = fout(4,4,1);
            fout(4,4,2) = phi^(3-alpha)/(Pe_phi*u);
            fout(4,2,4) = fout(4,4,2);
            if u_z ~= 0
                fout(1,1,1) = fout(1,1,1) - (a - (4-alpha)*phi^(3-alpha)*Psi)/(Pe_phi*u^2)*u_z*chi ...
                    - (a - (4-alpha)*phi^(3-alpha)*Psi)/(Pe_phi*u^2)*u_z*chi ...
                    + 2*(a*phi - phi^(4-alpha)*Psi)/(Pe_phi*u^3)*u_z^2*chi^2;
                fout(1,1,2) = fout(1,1,2) + phi^(4-alpha)/(Pe_phi*u^2)*u_z*chi;
                fout(1,2,1) = fout(1,1,2);
                fout(1,1,4) = fout(1,1,4) - (a - (4-alpha)*phi^(3-alpha)*Psi)/(Pe_phi*u^2)*u_z*phi ...
                    - (a*phi - phi^(4-alpha)*Psi)/(Pe_phi*u^2)*u_z ...
                    + 2*(a*phi - phi^(4-alpha)*Psi)/(Pe_phi*u^3)*u_z^2*phi*chi;
                fout(1,4,1) = fout(1,1,4);
                fout(1,2,4) = fout(1,2,4) + phi^(4-alpha)/(Pe_phi*u^2)*u_z*phi;
                fout(1,4,2) = fout(1,2,4);
                fout(1,4,4) = fout(1,4,4)  + 2*(a*phi - phi^(4-alpha)*Psi)/(Pe_phi*u^3)*u_z^2*phi^2;
                fout(2,1,1) = fout(2,1,1) + (alpha-2)*(3-alpha)*phi^(2-alpha)*Psi^2/(Pe_phi*u^2)*u_z*chi ...
                    + (alpha-2)*(3-alpha)*phi^(2-alpha)*Psi^2/(Pe_phi*u^2)*u_z*chi ...
                    + 2* ( -(alpha-2)*phi^(3-alpha)*Psi^2 + (alpha-2)*a*Psi)/(Pe_phi*u^3)*u_z^2*chi^2;
                fout(2,1,2) = fout(2,1,2) - (-2*(alpha-2)*phi^(3-alpha)*Psi + (alpha-2)*a)/(Pe_phi*u^2)*u_z*chi;
                fout(2,2,1) = fout(2,1,2);
                fout(2,1,4) = fout(2,1,4) + (alpha-2)*(3-alpha)*phi^(2-alpha)*Psi^2/(Pe_phi*u^2)*u_z*phi ...
                    - Pe_phi*u_z/delta - ( -(alpha-2)*phi^(3-alpha)*Psi^2 + (alpha-2)*a*Psi)/(Pe_phi*u^2)*u_z ...
                    + 2* ( -(alpha-2)*phi^(3-alpha)*Psi^2 + (alpha-2)*a*Psi)/(Pe_phi*u^3)*u_z^2*chi*phi;
                fout(2,4,1) = fout(2,1,4);
                fout(2,2,4) = fout(2,2,4) - ( -2*(alpha-2)*phi^(3-alpha)*Psi + (alpha-2)*a)/(Pe_phi*u^2)*u_z*phi;
                fout(2,4,2) = fout(2,2,4);
                fout(2,4,4) = fout(2,4,4) + 2*( -(alpha-2)*phi^(3-alpha)*Psi^2 + (alpha-2)*a*Psi)/(Pe_phi*u^3)*u_z^2*phi^2;
                fout(3,1,1) = fout(3,1,1) - (3-alpha)*phi^(2-alpha)*Psi*omega/(Pe_phi*u^2)*u_z*chi ...
                    - (3-alpha)*phi^(2-alpha)*Psi*omega/(Pe_phi*u^2)*u_z*chi ...
                    + 2*(- a*omega + phi^(3-alpha)*Psi*omega)/(Pe_phi*u^3)*u_z^2*chi^2;
                fout(3,1,2) = fout(3,1,2) - phi^(3-alpha)*omega/(Pe_phi*u^2)*u_z*chi;
                fout(3,2,1) = fout(3,1,2);
                fout(3,1,3) = fout(3,1,3)  - (- a + phi^(3-alpha)*Psi)/(Pe_phi*u^2)*u_z*chi;
                fout(3,3,1) = fout(3,1,3);
                fout(3,3,4) =  - (-a + phi^(3-alpha)*Psi)/(Pe_phi*u^2)*u_z*phi;
                fout(3,4,3) = fout(3,3,4);
                fout(3,1,4) = fout(3,1,4) - (3-alpha)*phi^(2-alpha)*Psi*omega/(Pe_phi*u^2)*u_z*phi ...
                    - (- a*omega + phi^(3-alpha)*Psi*omega)/(Pe_phi*u^2)*u_z ...
                    + 2*(- a*omega + phi^(3-alpha)*Psi*omega)/(Pe_phi*u^3)*u_z^2*chi*phi;
                fout(3,4,1) = fout(3,1,4);
                fout(3,2,4) = fout(3,2,4) - phi^(3-alpha)*omega/(Pe_phi*u^2)*u_z*phi;
                fout(3,4,2) = fout(3,2,4);
                fout(3,4,4) = fout(3,4,4) + 2*(- a*omega + phi^(3-alpha)*Psi*omega)/(Pe_phi*u^3)*u_z^2*phi^2;
                fout(4,1,1) = fout(4,1,1) - (3-alpha)*phi^(2-alpha)*chi*Psi/(Pe_phi*u)^2*u_z*chi ...
                    - (3-alpha)*phi^(2-alpha)*chi*Psi/(Pe_phi*u^2)*u_z*chi ...
                    + 2*(- a*chi + phi^(3-alpha)*chi*Psi)/(Pe_phi*u^3)*u_z^2*chi^2;
                fout(4,1,2) = fout(4,1,2) - phi^(3-alpha)*chi/(Pe_phi*u^2)*u_z*chi;
                fout(4,2,1) = fout(4,1,2);
                fout(4,1,4) = fout(4,1,4) -   (3-alpha)*phi^(2-alpha)*chi*Psi/(Pe_phi*u^2)*u_z*phi ...
                    - (- a + phi^(3-alpha)*Psi)/(Pe_phi*u^2)*u_z*chi ...
                    - (- a*chi + phi^(3-alpha)*chi*Psi)/(Pe_phi*u^2)*u_z ...
                    + 2*(- a*chi + phi^(3-alpha)*chi*Psi)/(Pe_phi*u^3)*u_z^2*chi*phi;
                fout(4,4,1) = fout(4,1,4);
                fout(4,2,4) = fout(4,2,4) - phi^(3-alpha)*chi/(Pe_phi*u^2)*u_z*phi;
                fout(4,4,2) = fout(4,2,4);
                fout(4,4,4) = fout(4,4,4) - ( -a + phi^(3-alpha)*Psi)/(Pe_phi*u^2)*u_z*phi ...
                    - (- a  + phi^(3-alpha)*Psi)/(Pe_phi*u^2)*u_z*phi ...
                    + 2*(- a*chi + phi^(3-alpha)*chi*Psi)/(Pe_phi*u^3)*u_z^2*phi^2;
            end
        else
            fout(1,1,1) =  (alpha-1)*(alpha-3)*a*phi^((alpha-5)/2)/(4*Pe_phi*u);
            fout(1,1,2) = -1/(Pe_phi*u);
            fout(1,2,1) = fout(1,1,2);
            fout(2,1,1) =  -g*(alpha-1)*(alpha-2)*phi^(alpha-3)/delta ...
                + (alpha-1)*(alpha-3)*(alpha-5)*a*phi^((alpha-7)/2)*Psi/(8*Pe_phi*u);
            fout(2,1,2) = (alpha-1)*(alpha-3)*a*phi^((alpha-5)/2)/(4*Pe_phi*u);
            fout(2,2,1) = fout(2,1,2);
            fout(2,2,2) = -(alpha-1)/(Pe_phi*u);
            fout(3,1,1) = (alpha-1)*(alpha-3)*Pe_phi*u_z*phi^((alpha-5)/2)/4 ...
                - (alpha-3)*(alpha-5)*a*phi^((alpha-7)/2)*omega/(4*Pe_phi*u);
            fout(3,1,3) = -(alpha-3)*a*phi^((alpha-5)/2)/(2*Pe_phi*u);
            fout(3,3,1) = fout(3,1,3);
            fout(3,2,3) = 1/(Pe_phi*u);
            fout(3,3,2) = fout(3,2,3);
            fout(4,1,1) = (alpha-3)*(alpha-5)*phi^((alpha-7)/2)*(1-a*chi/(Pe_phi*u))/4;
            fout(4,1,4) = -(alpha-3)*phi^((alpha-5)/2)*a/(Pe_phi*u)/2;
            fout(4,4,1) = fout(4,1,4);
            fout(4,2,4) = 1/(Pe_phi*u);
            fout(4,4,2) = fout(4,2,4);
            if u_z ~= 0
                fout(1,1,1) = fout(1,1,1) - ((alpha-1)*a*phi^((alpha-3)/2)/2 - Psi)/(Pe_phi*u^2)*u_z*chi ...
                     - ((alpha-1)*a*phi^((alpha-3)/2)/2 - Psi)/(Pe_phi*u^2)*u_z*chi ...
                     + 2*(a*phi^((alpha-1)/2) - phi*Psi)/(Pe_phi*u^3)*u_z^2*chi^2;
                fout(1,1,2) = fout(1,1,2) + phi/(Pe_phi*u^2)*u_z*chi;
                fout(1,2,1) = fout(1,1,2);
                fout(1,1,4) = fout(1,1,4) - ((alpha-1)*a*phi^((alpha-3)/2)/2 - Psi)/(Pe_phi*u^2)*u_z*phi ...
                    - (a*phi^((alpha-1)/2) - phi*Psi)/(Pe_phi*u^2)*u_z ...
                    + 2* (a*phi^((alpha-1)/2) - phi*Psi)/(Pe_phi*u^3)*u_z^2*chi*phi;
                fout(1,4,1) = fout(1,1,4);
                fout(1,2,4) = fout(1,2,4) +  phi/(Pe_phi*u^2)*u_z*phi;
                fout(1,4,2) = fout(1,2,4);
                fout(1,4,4) = fout(1,4,4) + 2*(a*phi^((alpha-1)/2) - phi*Psi)/(Pe_phi*u^3)*u_z^2*phi^2; 
                fout(2,1,1) = fout(2,1,1) - (alpha-1)*(alpha-3)*a*phi^((alpha-5)/2)*Psi/(4*Pe_phi*u^2)*u_z*chi ...
                    - (alpha-1)*(alpha-3)*a*phi^((alpha-5)/2)*Psi/(4*Pe_phi*u^2)*u_z*chi ...
                    + 2*( -(alpha-1)*Psi^2 + (alpha-1)*a*phi^((alpha-3)/2)*Psi)/(2*Pe_phi*u^3)*u_z^2*chi^2;
                fout(2,1,2) = fout(2,1,2) - ( -2*(alpha-1)*Psi + (alpha-1)*a*phi^((alpha-3)/2))/(2*Pe_phi*u^2)*u_z*chi;
                fout(2,2,1) = fout(2,1,2);
                fout(2,1,4) = fout(2,1,4) - (alpha-1)*(alpha-3)*a*phi^((alpha-5)/2)*Psi/(4*Pe_phi*u^2)*u_z*phi ...
                    - Pe_phi*u_z/delta - ( -(alpha-1)*Psi^2 + (alpha-1)*a*phi^((alpha-3)/2)*Psi)/(2*Pe_phi*u^2)*u_z ...
                    + 2*( -(alpha-1)*Psi^2 + (alpha-1)*a*phi^((alpha-3)/2)*Psi)/(2*Pe_phi*u^3)*u_z^2*chi*phi;
                fout(2,4,1) = fout(2,1,4);
                fout(2,2,4) = fout(2,2,4) - ( -(alpha-1)*Psi + (alpha-1)*a*phi^((alpha-3)/2)/2)/(Pe_phi*u^2)*u_z*phi;
                fout(2,4,2) = fout(2,2,4);
                fout(2,4,4) = fout(2,4,4) + 2*( -(alpha-1)*Psi^2 + (alpha-1)*a*phi^((alpha-3)/2)*Psi)/(2*Pe_phi*u^3)*u_z^2*phi^2;
                
                %fout(3,1) = (alpha-1)*Pe_phi*u_z*phi^((alpha-3)/2)/2 ...
                %- (alpha-3)*a*phi^((alpha-5)/2)*omega/(2*Pe_phi*u);
                %fout(3,1) = fout(3,1) - ( - a*phi^((alpha-3)/2)*omega + Psi*omega)/(Pe_phi*u^2)*u_z*chi;
                
                fout(3,1,1) = fout(3,1,1) + (alpha-3)*a*phi^((alpha-5)/2)*omega/(2*Pe_phi*u^2)*u_z*chi ...
                    + (alpha-3)*a*phi^((alpha-5)/2)*omega/(2*Pe_phi*u^2)*u_z*chi ...
                    + 2*( - a*phi^((alpha-3)/2)*omega + Psi*omega)/(Pe_phi*u^3)*u_z^2*chi^2;
                fout(3,1,2) = fout(3,1,2) - omega/(Pe_phi*u^2)*u_z*chi;
                fout(3,2,1) = fout(3,1,2);
                fout(3,1,3) = fout(3,1,3) - ( - a*phi^((alpha-3)/2) + Psi)/(Pe_phi*u^2)*u_z*chi;
                fout(3,3,1) = fout(3,1,3);
                fout(3,1,4) = fout(3,1,4) + (alpha-3)*a*phi^((alpha-5)/2)*omega/(2*Pe_phi*u^2)*u_z*phi ...
                    - ( - a*phi^((alpha-3)/2)*omega + Psi*omega)/(Pe_phi*u^2)*u_z ...
                    + 2*( - a*phi^((alpha-3)/2)*omega + Psi*omega)/(Pe_phi*u^3)*u_z^2*chi*phi;
                fout(3,4,1) = fout(3,1,4);
                fout(3,2,4) = fout(3,2,4) - omega/(Pe_phi*u^2)*u_z*phi;
                fout(3,4,2) = fout(3,2,4);
                fout(3,3,4) = fout(3,3,4) - ( -a*phi^((alpha-3)/2)+ Psi)/(Pe_phi*u^2)*u_z*phi;
                fout(3,4,3) = fout(3,3,4);
                fout(3,4,4) = fout(3,4,4) + 2*( - a*phi^((alpha-3)/2)*omega + Psi*omega)/(Pe_phi*u^3)*u_z^2*phi^2;
                fout(4,1,1) = fout(4,1,1) + (alpha-3)*phi^((alpha-5)/2)*a*chi/(2*Pe_phi*u^2)*u_z*chi ...
                     + (alpha-3)*a*phi^((alpha-5)/2)*chi/(2*Pe_phi*u^2)*u_z*chi ...
                     + 2*( -a*phi^((alpha-3)/2)*chi + chi*Psi)/(Pe_phi*u^3)*u_z^2*chi^2;
                 fout(4,1,2) = fout(4,1,2) -  chi/(Pe_phi*u^2)*u_z*chi;
                 fout(4,2,1) = fout(4,1,2);
                 fout(4,1,4) = fout(4,1,4) + (alpha-3)*phi^((alpha-5)/2)*a*chi/(2*Pe_phi*u^2)*u_z*phi ...
                     - ( -a*phi^((alpha-3)/2) + Psi)/(Pe_phi*u^2)*u_z*chi ... 
                     - ( -a*phi^((alpha-3)/2)*chi + chi*Psi)/(Pe_phi*u^2)*u_z ...
                     + 2*( -a*phi^((alpha-3)/2)*chi + chi*Psi)/(Pe_phi*u^3)*u_z^2*chi*phi;
                 fout(4,4,1) = fout(4,1,4);
                 fout(4,2,4) = fout(4,2,4) - chi/(Pe_phi*u^2)*u_z*phi;
                 fout(4,4,2) = fout(4,2,4);
                 fout(4,4,4) = fout(4,4,4) - (-a*phi^((alpha-3)/2) + Psi)/(Pe_phi*u^2)*u_z*phi ...
                      - ( -a*phi^((alpha-3)/2) + Psi)/(Pe_phi*u^2)*u_z*phi ...
                      + 2*( -a*phi^((alpha-3)/2)*chi + chi*Psi)/(Pe_phi*u^3)*u_z^2*phi^2;
            end
        end
end

end

