function [fout Dfout] = temperate_steady_v3(v_in,parameters)
%[fout Dfout] = temperate_steady_v3(v_in,parameters)
%Function whose roots define the multiple shooting method solution to the
%steady state temperate ice problem in one dimension as described in the
%documentation to steady_solver.m, which uses a Newton solver to find these
%roots. See also the documentation to temperate_ode_transformed.m on
%details of the handling of the regions in whicn the transformed o.d.e.
%system is solved
%
%Dependencies: see steady_solver.m documentation
%
%Christian Schoof, January 2015

%identify partition of temperate region
if isfield(parameters,'partition')
    shoot_grid = parameters.partition.grid;
else
    shoot_grid = [0; 1];
end
if min(shoot_grid) < 0 || max(shoot_grid) > 1
    error('The values in parameters.partition.grid need to lie between 0 and 1')
end
if ~issorted(shoot_grid) &&  ~issorted(-shoot_grid)
    error('The values in parameters.partition.grid need to be in ascending or descending order')
end
n_pts = length(shoot_grid);

%transformation parameters
if isfield(parameters,'transform') && (~isfield(parameters.transform,'transform') || parameters.transform.transform)
    transform = true;
    transform_grid = parameters.transform.grid;
    if ~issorted(transform_grid) && ~issorted(-transform_grid)
        error('The values in parameters.transform.grid must be in ascending or descending order')
    end
else
    transform = false;
end

if transform, n_pts_transform = length(transform_grid); end

%weights for compliance with boundary conditions
if isfield(parameters,'weights')
    weights = parameters.weights;
else
    weights.N_0 = 1;
    weights.phi_0 = 1;
    weights.q_0 = 1;
end

%integration nodes and weights
if isfield(parameters,'integration')
    [parameters.integration.nodes, parameters.integration.weights] = gaussquad(64,1);
end

%extract or create physical model parameters
if isfield(parameters,'model')
    if ~isfield(parameters.model,'a')
        parameters.model.a = 1;
    end
    if ~isfield(parameters.model,'Pe')
        parameters.model.Pe = 2;
    end
    if ~isfield(parameters.model,'parameters.model.Pe_phi')
        parameters.model.Pe_phi = 1;
    end
    if ~isfield(parameters.model,'g')
        parameters.model.g = -1;
    end
    if ~isfield(parameters.model,'delta')
        parameters.model.delta = 1;
    end
    if ~isfield(parameters.model,'alpha')
        parameters.model.alpha = 3;
    end
    if ~isfield(parameters.model,'u_0')
        parameters.model.u_0 = 0;
    end
    if ~isfield(parameters.model,'u_z')
        if parameters.model.u_0 == 0
            parameters.model.u_z = 1;
        else
            parameters.model.u_z = 0;
        end
    end
    if parameters.model.u_0 ~= 0 && parameters.model.u_z ~= 0
        error('either base velocity u_0 or strain rate u_z must be zero; to have a non-zero velocity and non-zero strain rate at the boundaries, choose both boundaries z_c and z_t to be non-zero instead')
    end
    if ~isfield(parameters.model,'T_0')
        parameters.model.T_0 = -1;
    end
    if ~isfield(parameters.model,'N_0')
        parameters.model.N_0 = -1;
    end
    if ~isfield(parameters.model,'phi_0')
        parameters.model.phi_0 = 1;
    end
    if ~isfield(parameters.model,'z_c')
        parameters.model.z_c = 1;
    end
    if ~isfield(parameters.model,'z_ct')
        parameters.model.z_ct = 0.5;
    end
    if ~isfield(parameters.model,'z_t')
        parameters.model.z_t = 0;
    end
else
    warning('no model parameters supplied')
    parameters.model.a = 1;
    parameters.model.Pe = 2;
    parameters.model.parameters.model.Pe_phi = 1;
    parameters.model.g = -1;
    parameters.model.delta = 1;
    parameters.model.alpha = 3;
    parameters.model.u_0 = 1;
    parameters.model.u_z = 0;
    parameters.model.T_0 = -1;
    parameters.model.N_0 = -1;
    parameters.model.phi_0 = 1;
    parameters.model.z_c = 1;
    parameters.model.z_t = 0;
    parameters.model.z_ct = 0.5;
end



%Test Jacobian (optional)
if nargin > 1 && isfield(parameters,'flags') && isfield(parameters.flags,'test') && parameters.flags.test          
    fout.parameters = parameters;
    parameters.model.direction = round(rand(1)*2)*2-3;
    parameters.flags.test = false; %steps test flag being passed onto shooting methods
    if isfield(parameters.flags,'identity') && parameters.flags.identity
        fout.tempsol = Jacobian_test_single(@tempsol,ones(3,1),parameters,sqrt(eps));
        parameters.functions.evolve = @temperate_ode;
        parameters.t_span = [1 1.5];
        fout.temp_shoot = Jacobian_test_single(@shoot,ones(3,1),parameters,100*sqrt(eps));
        parameters.functions.evolve = @temperate_ode_transformed;
        parameters.t_span = [0 -.5];
        fout.transform_shoot = Jacobian_test_single(@shoot,ones(4,1),parameters,100*sqrt(eps));            
    elseif isfield(parameters.flags,'local') && parameters.flags.local
        fout.tempsol = Jacobian_test_single(@tempsol,v_in(1:3),parameters,sqrt(eps));
        parameters.functions.evolve = @temperate_ode;
        parameters.t_span = [1 1.5];
        fout.temp_shoot = Jacobian_test_single(@shoot,v_in(4:6),parameters,100*sqrt(eps));
        parameters.functions.evolve = @temperate_ode_transformed;
        parameters.t_span = [0 -.5];
        fout.transform_shoot = Jacobian_test_single(@shoot,v_in(7:10),parameters,100*sqrt(eps)); 
    else
        fout.tempsol = Jacobian_test_single(@tempsol,rand(3,1),parameters,sqrt(eps));
        parameters.functions.evolve = @temperate_ode;
        parameters.t_span = 0.5*rand(2,1);
        fout.temp_shoot = Jacobian_test_single(@shoot,rand(3,1),parameters,100*sqrt(eps));
        parameters.functions.evolve = @temperate_ode_transformed;
        parameters.t_span = -.5*rand(2,1);
        fout.transform_shoot = Jacobian_test_single(@shoot,rand(4,1),parameters,100*sqrt(eps)); 
    end
    if nargout > 1, Dfout = []; end
    return        
end
          
%handle each problem type separately

%% warm base, upward movement or cold base, downward movement
if ((parameters.model.z_c > parameters.model.z_t) && (parameters.model.u_0 > 0 || parameters.model.u_z > 0)) ...
        || ((parameters.model.z_c < parameters.model.z_t) && (parameters.model.u_0 < 0 || parameters.model.u_z < 0)) 
    %test size of input
    if length(v_in) ~= n_pts*3
        error('Input length incorrect')
    end  
    %cold-temperate transition position
    z_ct = v_in(1);
    if sign(z_ct - parameters.model.z_c) == sign(z_ct - parameters.model.z_t)
        error('Cold-temperate transition must lie between the cold and temperate boundaries')
    end
    %temperature solution
    v = v_in(1:3);
    if nargout == 1
        ftemp1 = tempsol(v,parameters);
    else
        [ftemp1, Dftemp1] = tempsol(v,parameters);
    end
    %shooting solution of temperate ice problem
    parameters.functions.evolve = @temperate_ode;
    if parameters.model.u_0 == 0 && parameters.model.z_t == 0 && min(shoot_grid) == 0
        error('The equation for phi becomes singular at the boundary of the domain, regularize by displacing the boundary z_t')
    end
    ftemp2 = zeros(3,n_pts-1); Dftemp2 = zeros(3,3,n_pts-1); Dx_ctftemp2 = zeros(3,n_pts-1);
    for ii = 1:n_pts-1
        v = v_in(ii*3+1:(ii+1)*3);
        parameters.t_span = z_ct + [shoot_grid(ii) shoot_grid(ii+1)]*(parameters.model.z_t-z_ct);
        if nargout == 1
            ftemp2(:,ii) = shoot(v,parameters);
        else
            [ftemp2(:,ii), Dftemp2(:,:,ii)] = shoot(v,parameters);
            Dx_ctftemp2(:,ii) = -Dftemp2(:,:,ii)*temperate_ode(z_ct + shoot_grid(ii)*(parameters.model.z_t-z_ct),v,0,parameters.model)*(1-shoot_grid(ii)) ...
                + (1-shoot_grid(ii+1))*temperate_ode(z_ct + shoot_grid(ii+1)*(parameters.model.z_t-z_ct),ftemp2(:,ii),0,parameters.model);
        end
    end
    %initialize output
    fout = zeros(3*n_pts,1);
    if nargout > 1
        Dfout = zeros(3*n_pts);
    end
    %temperature Dirichlet boundary conditions
    fout(1) = ftemp1(1);
    fout(2) = ftemp1(2);
    if nargout > 1  %add Jacobian
        Dfout(1,1:3) = Dftemp1(1,:);
        Dfout(2,1:3) = Dftemp1(2,:);
    end
    %boundary conditions on temperate ice problem, form depends on shooting
    %direction
    if shoot_grid(1) < shoot_grid(end) 
        %cold-temperate transition boundary
        fout(3) = v_in(6) - ftemp1(3);   %Stefan condition 
        fout(4) = weights.q_0* (v_in(6) - parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct)*v_in(4));  %zero Darcy flux
        %bottom of domain
        fout(5) = weights.N_0* (ftemp2(2,end) - parameters.model.N_0);   %Dirichlet condition on effective pressure
        fout(6) = weights.phi_0* (ftemp2(1,end) - parameters.model.phi_0); %Cauchy condition on porosity
        if abs(parameters.model.z_t) < 100*sqrt(eps) && parameters.model.u_0 == 0 && parameters.model.phi_0 ~= parameters.model.a/parameters.model.N_0;
            warning('The outflow problem with vanishing velocity at the boundary (u_0 = 0) does not appear to have a solution; in the small delta limit the solution takes the "wrong" hyperbolic flux branch and cannot satisfy the Darcy flow boundary condition at the cold-temperate boundary')
        end
        if nargout > 1 %add Jacobian
            Dfout(3,1:3) = - Dftemp1(3,:);
            Dfout(3,6) = 1;
            Dfout(4,1) = weights.q_0*(-parameters.model.Pe_phi*parameters.model.u_z*v_in(4));
            Dfout(4,4) = weights.q_0*( - parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct));
            Dfout(4,6) = weights.q_0;
            Dfout(5,3*(n_pts-1)+(1:3)) = weights.N_0*Dftemp2(2,:,end);  Dfout(5,1) = weights.N_0*Dx_ctftemp2(2,end);
            Dfout(6,3*(n_pts-1)+(1:3)) = weights.phi_0*Dftemp2(1,:,end);  Dfout(6,1) = weights.phi_0*Dx_ctftemp2(1,end);
        end
    else
        %cold-temperate transition boundary
        fout(3) = ftemp2(3,end) - ftemp1(3);   %Stefan condition
        fout(4) = weights.q_0*(ftemp2(3,end) - parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct)*ftemp2(1,end));  %zero Darcy flux
        %bottom of domain
        fout(5) = weights.N_0*(v_in(5) - parameters.model.N_0);   %Dirichlet condition on effective pressure
        fout(6) = weights.phi_0*(v_in(4,end) - parameters.model.phi_0); %Cauchy condition on porosity
        if abs(parameters.model.z_t) < 100*sqrt(eps) && parameters.model.u_0 == 0 && parameters.model.phi_0 ~= parameters.model.a/parameters.model.N_0;
            warning('Singular behaviour expected near temperate lower boundary with zero velocity at the boundary; expect phi_0 = a/N_0 and z_t slightly larger than zero necessary')
        end
        if nargout > 1 %add Jacobian
            Dfout(3,1:3) = - Dftemp1(3,:);
            Dfout(3,1) = Dfout(3,1) + Dx_ctftemp2(3,end);
            Dfout(3,3*(n_pts-1)+(1:3)) = Dftemp2(3,:,end);
            Dfout(4,1) = weights.q_0* (Dx_ctftemp2(3,end) -parameters.model.Pe_phi*parameters.model.u_z*ftemp2(1,end) - parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct)*Dx_ctftemp2(1,end) );
            Dfout(4,3*(n_pts-1)+(1:3)) = weights.q_0* (Dftemp2(3,:,end) - parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*z_ct)*Dftemp2(1,:,end) );
            Dfout(5,5) = weights.N_0;
            Dfout(6,4) = weights.phi_0;
        end
    end
    %matching at interior points
    for ii=2:n_pts-1
        fout(3*ii+(1:3)) = ftemp2(:,ii-1) - v_in(3*ii+(1:3));
        if nargout > 1 %add Jacobian
            Dfout(3*ii+(1:3),3*(ii-1)+(1:3)) = Dftemp2(:,:,ii-1);
            Dfout(3*ii+(1:3),3*ii+(1:3)) = -eye(3);
            Dfout(3*ii+(1:3),1) = Dx_ctftemp2(:,ii-1);
        end
    end
    
%% cold base, upward movement or warm base, downward movement
elseif parameters.model.z_c ~= parameters.model.z_t
    if (parameters.model.z_ct - parameters.model.z_c)*(parameters.model.z_ct - parameters.model.z_t) > 0
        error('Cold-temperate transition must lie between the cold and temperate boundaries')
    end
    %test size of input
    if transform
        if length(v_in) ~= (n_pts-2)*3 + (n_pts_transform-1)*4
            error('Input length incorrect')
        end
    else
        if length(v_in) ~= (n_pts-1)*3
            error('Input length incorrect')
        end
    end
    if transform    %remove first or last segment of the partition of the temperate region to allow for transformation, if appropriate
        ftemp1 = zeros(3,n_pts-2); Dftemp1 = zeros(3,3,n_pts-2);
        if shoot_grid(1) > shoot_grid(end)
            shoot_min = 1; shoot_max = n_pts-2;
        else
            shoot_min = 2; shoot_max = n_pts-1;
        end
    else
        if min(shoot_grid == 0)
            error('problem for effective pressure becomes singular at inflow boundary; regularize by displacing smallest value of grid in paramteers.partition.grid away from 0 and using a sufficiently fine grid partition near that point, or using transformed problem')
        end
        ftemp1 = zeros(3,n_pts-1); Dftemp1 = zeros(3,3,n_pts-1);
        shoot_min = 1; shoot_max = n_pts - 1;
    end
    %next, shoot in each segment
    parameters.functions.evolve = @temperate_ode;
    for ii = shoot_min:shoot_max
        v = v_in((ii-shoot_min)*3+1:(ii-shoot_min+1)*3);
        parameters.t_span = parameters.model.z_ct + [shoot_grid(ii) shoot_grid(ii+1)]*(parameters.model.z_t-parameters.model.z_ct);
        if nargout == 1
            ftemp1(:,ii-shoot_min+1) = shoot(v,parameters);
        else
            [ftemp1(:,ii-shoot_min+1), Dftemp1(:,:,ii-shoot_min+1)] = shoot(v,parameters);    
        end
    end
    %next, shoot over each segment of the transformed region 
    if transform
        parameters.functions.evolve = @temperate_ode_transformed;
        if (parameters.model.u_0 > 0 || parameters.model.u_z > 0)
            parameters.model.direction = +1;
        else
            parameters.model.direction = -1;
        end
        ftemp2 = zeros(4,n_pts_transform-1); Dftemp2 = zeros(4,4,n_pts_transform-1);
        for ii = 1:n_pts_transform-1
            parameters.t_span = [transform_grid(ii) transform_grid(ii+1)]; 
            v = v_in((n_pts-2)*3+(ii-1)*4+1:(n_pts-2)*3+ii*4);
            if nargout == 1
                ftemp2(:,ii) = shoot(v,parameters);
            else
                [ftemp2(:,ii), Dftemp2(:,:,ii)] = shoot(v,parameters);
            end
        end
    end
    %assemble output
    %first, boundary conditions at cold-temperate transition
    if transform
        fout = zeros((n_pts-2)*3+(n_pts_transform-1)*4,1);
        if nargout > 2
            Dfout = zeros((n_pts-2)*3+(n_pts_transform-1)*4);
        end
        if parameters.model.alpha < 2
            error('behaviour for alpha < 2 not covered by code at present')
        end
        %parameters in the leading order behaviour of omega and chi near
        %the fixed point
        c_omega = parameters.model.Pe_phi^2*parameters.model.direction*(parameters.model.u_0+parameters.model.u_z*parameters.model.z_ct)*parameters.model.u_z/(2*parameters.model.a);
        chi_0 = parameters.model.direction*parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*parameters.model.z_ct)/parameters.model.a;
        c_chi = c_omega/parameters.model.a;
        if transform_grid(1) > transform_grid(end)
            fout(1) = weights.q_0* (ftemp2(3,end)/ftemp2(1,end)-c_omega); %this sets the flux condition at the boundary to zero
            fout(2) = weights.phi_0* ((ftemp2(4,end)-chi_0)/ftemp2(1,end) - c_chi);%this is the chi-condition, effectively setting the porosity at the boundary.
            if nargout > 1
                Dfout(1,(n_pts-2)*3+(n_pts_transform-2)*4+(1:4)) = weights.q_0 * (Dftemp2(3,:,end)/ftemp2(1,end) - ftemp2(3,end)/ftemp2(1,end)^2*Dftemp2(1,:,end));
                Dfout(2,(n_pts-2)*3+(n_pts_transform-2)*4+(1:4)) = weights.phi_0* (Dftemp2(4,:,end)/ftemp2(1,end) ...
                    - (ftemp2(4,end)-chi_0)/ftemp2(1,end)^2*Dftemp2(1,:,end) );                        
            end
        else
            fout(1) = weights.q_0* (v_in((n_pts-2)*3+3)/v_in((n_pts-2)*3+1) - c_omega)   ;  %this sets the flux condition at the boundary to zero
            fout(2) = weights.phi_0* ((v_in((n_pts-2)*3+4) - chi_0)/v_in((n_pts-2)*3+1) -c_chi);   %this is the chi-condition, effectively setting the porosity at the boundary. 
            if nargout > 1
                Dfout(1,(n_pts-2)*3+3) = weights.q_0/v_in((n_pts-2)*3+1);
                Dfout(1,(n_pts-2)*3+1) = -weights.q_0* v_in((n_pts-2)*3+3)/v_in((n_pts-2)*3+1)^2;
                Dfout(2,(n_pts-2)*3+4) = weights.phi_0/v_in((n_pts-2)*3+1);
                Dfout(2,(n_pts-2)*3+1) = -weights.phi_0* (v_in((n_pts-2)*3+4) - chi_0)/v_in((n_pts-2)*3+1)^2;
            end
        end
    else
        fout = zeros((n_pts-1)*3,1);
        if nargout > 2
            Dfout = zeros((n_pts-1)*3);
        end
        if shoot_grid(1) > shoot_grid(end)
            fout(1) = weights.q_0* (ftemp1(3,end) - parameters.model.a*(parameters.model.z_t-parameters.model.z_ct).*shoot_grid(end));
            fout(2) = weights.phi_0* (ftemp1(1,end) - parameters.model.a*abs(parameters.model.z_t-parameters.model.z_ct).*shoot_grid(end)/(parameters.model.parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*parameters.model.z_ct)));
            if nargout > 1
                Dfout(1,3*(n_pts-2)+(1:3)) = weights.q_0* Dftemp1(3,:,end);
                Dfout(2,3*(n_pts-2)+(1:3)) = weights.phi_0* Dftemp1(1,:,end);
            end
        else
            fout(1) = weights.q_0* (v_in(3) - parameters.model.a*(parameters.model.z_t-parameters.model.z_ct).*shoot_grid(end));
            fout(2) = weights.phi_0* ( v_in(1) - parameters.model.a*abs(parameters.model.z_t-parameters.model.z_ct).*shoot_grid(end)/(parameters.model.parameters.model.Pe_phi*(parameters.model.u_0 + parameters.model.u_z*parameters.model.z_ct)));
            if nargout > 1
                Dfout(1,3) = weights.q_0;
                Dfout(2,1) = weights.phi_0;
            end
        end
    end
    %next, boundary conditions at temperate boundary
    if shoot_grid(1) > shoot_grid(end)
        fout(3) = weights.N_0*(v_in(2) - parameters.model.N_0);
        if nargout > 1
           Dfout(3,2) = weights.N_0; 
        end
    else
        fout(3) = weights.N_0*(ftemp1(2,end) - parameters.model.N_0);
        if nargout > 1
            if transform  %where the initial conditions for the uppermost untransformed interval are located depends on whether the transformed system is used near the boundary or not
                Dfout(3,(n_pts-3)*3+(1:3)) = weights.N_0*Dftemp1(2,:,end);
            else
                Dfout(3,(n_pts-2)*3+(1:3)) = weights.N_0*Dftemp1(2,:,end);
            end
        end
    end
    %next, match intervals in shooting grid
    for ii = shoot_min:shoot_max-1
        fout((ii-shoot_min+1)*3+(1:3)) = ftemp1(:,ii-shoot_min+1) - v_in(3*(ii-shoot_min+1)+(1:3));
        if nargout > 1 %add Jacobian
            Dfout(3*(ii-shoot_min+1)+(1:3),3*(ii-shoot_min)+(1:3)) = Dftemp1(:,:,ii-shoot_min+1);
            Dfout(3*(ii-shoot_min+1)+(1:3),3*(ii-shoot_min+1)+(1:3)) = -eye(3);
        end
    end
    %next, match transformed and untransformed boundaries and intervals
    %in transformed part
    if transform
    %match transformed intervals
        for ii = 2:n_pts_transform-1
            fout(3*(n_pts-2)+(ii-2)*4+(1:4)) = ftemp2(:,ii-1) - v_in(3*(n_pts-2)+(ii-1)*4+(1:4));
            if nargout > 1
                Dfout(3*(n_pts-2)+(ii-2)*4+(1:4),3*(n_pts-2)+(ii-2)*4+(1:4)) = Dftemp2(:,:,ii-1);
                Dfout(3*(n_pts-2)+(ii-2)*4+(1:4),3*(n_pts-2)+(ii-1)*4+(1:4)) = -eye(4);                   
            end
        end
        %match transformed and untransformed
        %need to use Psi = phi^beta*N with beta = alpha-2 for 2 < alpha
        %<= 3, beta = (alpha-1)/2 for alpha > 3; omega = (q -
        %a*(z-z_ct))*sign(z_t-z_c)/phi; chi = phi*|z-z_ct|
        if parameters.model.alpha <= 3
            beta = parameters.model.alpha-2;
        else
            beta = (parameters.model.alpha-1)/2;
        end
        if shoot_grid(1) > shoot_grid(end)
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+1) = ftemp1(1,end);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+2) = ftemp1(2,end)*ftemp1(1,end)^beta;
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+3) = ...
                parameters.model.direction*(ftemp1(3,end) - parameters.model.a*(parameters.model.z_t - parameters.model.z_ct)*shoot_grid(end-1))/ftemp1(1,end);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+4) = abs(parameters.model.z_t - parameters.model.z_ct)*shoot_grid(end-1)/ftemp1(1,end);
            if nargout > 1
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+1,3*(n_pts-3)+(1:3)) = Dftemp1(1,:,end);
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+2,3*(n_pts-3)+(1:3)) = Dftemp1(2,:,end)*ftemp1(1,end)^beta + beta*ftemp1(2,end)*ftemp1(1,end)^(beta-1)*Dftemp1(1.,:,end);
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+3,3*(n_pts-3)+(1:3)) = parameters.model.direction*(Dftemp1(3,:,end)/ftemp1(1,end) - ...
                    (ftemp1(3,end) - parameters.model.a*(parameters.model.z_t - parameters.model.z_ct)*shoot_grid(end-1))/ftemp1(1,end)^2*Dftemp1(1,:,end));
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+4,3*(n_pts-3)+(1:3)) = -Dftemp1(1,:,end)*abs(parameters.model.z_t - parameters.model.z_ct)*shoot_grid(end-1)/ftemp1(1,end)^2;
            end 
        else
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+1) = v_in(1);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+2) = v_in(2)*v_in(1)^beta;
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+3) = ...
                parameters.model.direction*(v_in(3) - parameters.model.a*(parameters.model.z_t - parameters.model.z_ct)*shoot_grid(2))/v_in(1);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+4) = abs(parameters.model.z_t - parameters.model.z_ct)*shoot_grid(2)/v_in(1);
            if nargout > 1
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+1,1) = 1;
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+2,1) = beta*v_in(2)*v_in(1)^(beta-1);
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+2,2) = v_in(1)^beta;                        
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+3,1) = -parameters.model.direction*(v_in(3) - parameters.model.a*(parameters.model.z_t - parameters.model.z_ct)*shoot_grid(2))/v_in(1)^2;
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+3,3) = parameters.model.direction/v_in(1); 
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+4,1) = -abs(parameters.model.z_t - parameters.model.z_ct)*shoot_grid(2)/v_in(1)^2;
            end
        end
        if transform_grid(1) >  transform_grid(end)
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+1) = fout(3*(n_pts-2)+4*(n_pts_transform-2)+1)-v_in(3*(n_pts-2)+1);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+2) = fout(3*(n_pts-2)+4*(n_pts_transform-2)+2)-v_in(3*(n_pts-2)+2);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+3) = fout(3*(n_pts-2)+4*(n_pts_transform-2)+3)-v_in(3*(n_pts-2)+3);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+4) = fout(3*(n_pts-2)+4*(n_pts_transform-2)+4) -v_in(3*(n_pts-2)+4);
            if nargout > 1
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+1,3*(n_pts-2)+1) = -1;
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+2,3*(n_pts-2)+2) = -1;
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+3,3*(n_pts-2)+3) = -1;
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+4,3*(n_pts-2)+4) = -1;
            end
        else
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+3)
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+4)
            ftemp2(4*(n_pts_transform-2)+3)
            ftemp2(4*(n_pts_transform-2)+4)
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+1) = fout(3*(n_pts-2)+4*(n_pts_transform-2)+1)-ftemp2(4*(n_pts_transform-2)+1);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+2) = fout(3*(n_pts-2)+4*(n_pts_transform-2)+2)-ftemp2(4*(n_pts_transform-2)+2);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+3) = fout(3*(n_pts-2)+4*(n_pts_transform-2)+3)-ftemp2(4*(n_pts_transform-2)+3);
            fout(3*(n_pts-2)+4*(n_pts_transform-2)+4) = fout(3*(n_pts-2)+4*(n_pts_transform-2)+4)-ftemp2(4*(n_pts_transform-2)+4);
            if nargout > 1
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+1,3*(n_pts-2)+4*(n_pts_transform-2)+(1:4)) = -Dftemp2(1,:,end);
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+2,3*(n_pts-2)+4*(n_pts_transform-2)+(1:4)) = -Dftemp2(2,:,end);
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+3,3*(n_pts-2)+4*(n_pts_transform-2)+(1:4)) = -Dftemp2(3,:,end);
                Dfout(3*(n_pts-2)+4*(n_pts_transform-2)+4,3*(n_pts-2)+4*(n_pts_transform-2)+(1:4)) = -Dftemp2(4,:,end);
            end
        end  
    end
else
    error('the cold and temperate boundaries cannot be located at the same position')        
end

if isfield(parameters,'display') && parameters.display
    
end

end
