function [fout, Dfout] = tempsol(v,parameters)
% [fout, Dfout] = tempsol(v,parameters)
%Computes boundary values of the solution to
% Pe*u*T_z - T_zz = a
%as a function of two constants of integration and of a free boundary, for
%cases of u = u_0 = constant or u = u_z*z with u_z = constant. The general
%solution for these two cases can be written as
% T = A/(Pe*u)*exp(Pe*u*z) + a*z/(Pe*u) + B;
%if u is constant, or as
% T = -2*a/(Pe*u_z)*int_0^z dawson_minus(sqrt(Pe*u_z/2)*z')*sqrt(Pe*u_z/2) dz'
%           + A*sqrt(2/(Pe*u_z))*exp(Pe*u_z*z^2/2)*dawson_plus(sqrt(Pe*u_z/2)*z) + B;
%for u_z > 0, and
%T =  -2*a/(Pe*|u_z|)*int_0^z dawson_plus(sqrt(Pe*|u_z|/2)*z')*sqrt(Pe*|u_z|/2) dz'
%            + A*sqrt(pi/(2*Pe*|u_z|))*erf(sqrt(Pe*|u_z|/2)*z) + B;
%
%Input:
%   v:  3-by-1 vector [z_ct; A; B]. z_ct is the location of the free
%       boundary at which some of the boundary values are computed, A and B
%       are the constants of integration in the solutions above
%   parameters: optional parameter structure containing fields
%       model: substructure containing the fields
%           Pe: Peclet number, defaults to 2
%           a:  Dissipation rate, defaults to 1
%           u_0: Constant part of velocity field defaults to 1
%           u_z: Strain rate, defaults to 0
%               NOTE that only one of u_0, u_z can be non-zero
%           z_c: Location of fixed boundary on which Dirichlet condition is
%               evaluated, defaults to 1
%           T_0: parameter in temperature boundary condition at z_c,
%               defaults to -1
%       integration: optional substructure containing fields
%           nodes: integration nodes in [0,1] interval
%           weights: corresponding weights for integration over [0,1]
%                   interval
%                   weights and nodes need to be vectors of the same size
%
%Output:
%   fout:   3-b-y-1 vector in the form
%               [T(z_c)-T_0; T(z_ct); Pe*u(z_ct)*T(z_ct) - T_z(z_ct)]
%   Dfout:  Jacobian matrix of fout with respect to v'
%
%Dependencies: ensure that dawson_plus, dawson_minus, faddeeva, gaussquad
%are in MATALB path
%
%Christian Schoof, January 2015

%extract solution degrees of freedom
z_ct = v(1);
A = v(2);
B = v(3);
%extract parameters 
if nargin < 2 || ~isfield(parameters,'model') || ~isfield(parameters.model,'Pe')
    Pe = 2;
else
    Pe = parameters.model.Pe;
end
if nargin < 2 || ~isfield(parameters,'model') || ~isfield(parameters.model,'a')
    a = 1;
else
    a = parameters.model.a;
end
if nargin < 2 || ~isfield(parameters,'model') || ~isfield(parameters.model,'u_0')
    u_0 = 1;
else
    u_0 = parameters.model.u_0;
end
if nargin < 2 || ~isfield(parameters,'model') || ~isfield(parameters.model,'u_z')
    u_z = 0;
else
    u_z = parameters.model.u_z;
end
if nargin < 2 || ~isfield(parameters,'model') || ~isfield(parameters.model,'T_0')
    T_0 = -1;
else
    T_0 = parameters.model.T_0;
end
if nargin < 2|| ~isfield(parameters,'model') || ~isfield(parameters.model,'z_c')
    z_c = 1;
else
    z_c = parameters.model.z_c;
end
%extract nodes and weights
if nargin < 2 || ~isfield(parameters,'integration')
    [x_int, w_int] = gaussquad(64,1);
else
    x_int = parameters.integration.nodes;
    w_int = parameters.integration.weights;
end
%compute output, go trough different cases
if u_0 ~= 0
    if u_z ~= 0, error('This code is currently not set up for base velocity and velocity gradient to be simultaneously non-zero'), end
    T_c = A/(Pe*u_0)*exp(Pe*u_0*z_c) + a*z_c/(Pe*u_0) + B;
    T_ct = A/(Pe*u_0)*exp(Pe*u_0*z_ct) + a*z_ct/(Pe*u_0) + B;
    q_ct = - ( A*exp(Pe*u_0*z_ct) + a/(Pe*u_0));
    if nargout > 1
        DT_c = zeros(1,3);
        DT_c(2) = 1/(Pe*u_0)*exp(Pe*u_0*z_c);
        DT_c(3) = 1;
        DT_ct = zeros(1,3);
        DT_ct(1) = - q_ct;
        DT_ct(2) = 1/(Pe*u_0)*exp(Pe*u_0*z_ct);
        DT_ct(3) = 1;
        Dq_ct = zeros(1,3);
        Dq_ct(1) = Pe*u_0*q_ct + a;
        Dq_ct(2) = - exp(Pe*u_0*z_ct);
    end
elseif u_z > 0
    Pe = Pe*u_z;
    if z_c == 0
        T_c = B;
    else
        T_c = -2*a/Pe*sum(w_int.*dawson_minus(x_int*sqrt(Pe/2)*z_c)*sqrt(Pe/2)*z_c)...
            + A*sqrt(2/Pe)*exp(Pe*z_c^2/2)*dawson_plus(sqrt(Pe/2)*z_c) + B;
    end
    T_ct = -2*a/Pe*sum(w_int.*dawson_minus(x_int*sqrt(Pe/2)*z_ct)*sqrt(Pe/2)*z_ct)...
           + A*sqrt(2/Pe)*exp(Pe*z_ct^2/2)*dawson_plus(sqrt(Pe/2)*z_ct) + B;
    q_ct = -( A*exp(Pe*z_ct^2/2) - a*sqrt(2/Pe)*dawson_minus(sqrt(Pe/2)*z_ct) );
    if nargout > 1
       DT_c = zeros(1,3);
       DT_c(2) = sqrt(2/Pe)*exp(Pe*z_c^2/2)*dawson_plus(sqrt(Pe/2)*z_c);
       DT_c(3) = 1;
       DT_ct = zeros(1,3);
       DT_ct(1) = -q_ct;
       DT_ct(2) = sqrt(2/Pe)*exp(Pe*z_ct^2/2)*dawson_plus(sqrt(Pe/2)*z_ct);
       DT_ct(3) = 1;
       Dq_ct = zeros(1,3);
       Dq_ct(1) = Pe*z_ct*q_ct + a;
       Dq_ct(2) = -exp(Pe*z_ct^2/2);
    end
elseif u_z < 0
    Pe = Pe*abs(u_z);
     if z_c == 0
        T_c = B;
    else
        T_c = -2*a/Pe*sum(w_int.*dawson_plus(x_int*sqrt(Pe/2)*z_c)*sqrt(Pe/2)*z_c) ...
            + A*sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z_c) + B;
    end
    T_ct = -2*a/Pe*sum(w_int.*dawson_plus(x_int*sqrt(Pe/2)*z_ct)*sqrt(Pe/2)*z_ct) ...
            + A*sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z_ct) + B;
    q_ct = -( A*exp(-Pe*z_ct^2/2) - a*sqrt(2/Pe)*dawson_plus(sqrt(Pe/2)*z_ct) );
    if nargout > 1
       DT_c = zeros(1,3);
       DT_c(2) = sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z_c);
       DT_c(3) = 1;
       DT_ct = zeros(1,3);
       DT_ct(1) = -q_ct;
       DT_ct(2) = sqrt(pi/(2*Pe))*erf(sqrt(Pe/2)*z_ct);
       DT_ct(3) = 1;
       Dq_ct = zeros(1,3);
       Dq_ct(1) = -Pe*z_ct*q_ct + a;
       Dq_ct(2) = -exp(-Pe*z_ct^2/2);
    end   
end
%assemble output
fout = [T_c-T_0; T_ct; q_ct];
if nargout > 1
    Dfout = [DT_c; DT_ct; Dq_ct];
end
end